@@ -2064,7 +2064,7 @@ bool CaptureContext::OpenRGPProfile(const rdcstr &filename)
   return true;
 }
 
-rdcstr CaptureContext::GetResourceNameUnsuffixed(ResourceId id)
+rdcstr CaptureContext::GetResourceNameUnsuffixed(ResourceId id) const
 {
   if(id == ResourceId())
     return tr("No Resource");
@@ -2072,7 +2072,7 @@ rdcstr CaptureContext::GetResourceNameUnsuffixed(ResourceId id)
   if(m_ReplacedToOrigResources.contains(id))
     return GetResourceName(m_ReplacedToOrigResources[id]);
 
-  ResourceDescription *desc = GetResource(id);
+  const ResourceDescription *desc = GetResource(id);
 
   if(desc)
     return GetResourceNameUnsuffixed(desc);
@@ -2082,7 +2082,7 @@ rdcstr CaptureContext::GetResourceNameUnsuffixed(ResourceId id)
   return tr("Unknown Resource %1").arg(num);
 }
 
-rdcstr CaptureContext::GetResourceNameUnsuffixed(const ResourceDescription *desc)
+rdcstr CaptureContext::GetResourceNameUnsuffixed(const ResourceDescription *desc) const
 {
   if(m_CustomNames.contains(desc->resourceId))
     return m_CustomNames[desc->resourceId];
@@ -2090,7 +2090,7 @@ rdcstr CaptureContext::GetResourceNameUnsuffixed(const ResourceDescription *desc
   return desc->name;
 }
 
-rdcstr CaptureContext::GetResourceName(ResourceId id)
+rdcstr CaptureContext::GetResourceName(ResourceId id) const
 {
   rdcstr ret = GetResourceNameUnsuffixed(id);
 
@@ -2108,7 +2108,7 @@ bool CaptureContext::IsAutogeneratedName(ResourceId id)
   if(m_CustomNames.contains(id))
     return false;
 
-  ResourceDescription *desc = GetResource(id);
+  const ResourceDescription *desc = GetResource(id);
 
   if(desc)
     return desc->autogeneratedName;
@@ -2450,8 +2450,8 @@ void CaptureContext::ShowResourceInspector()
   m_MainWindow->showResourceInspector();
 }
 
-IShaderViewer *CaptureContext::EditShader(ResourceId id, ShaderStage stage,
-                                          const rdcstr &entryPoint, const rdcstrpairs &files,
+IShaderViewer *CaptureContext::EditShader(ResourceId id, ShaderStage stage, const rdcstr &entryPoint,
+                                          const rdcstrpairs &files, KnownShaderTool knownTool,
                                           ShaderEncoding shaderEncoding, ShaderCompileFlags flags,
                                           IShaderViewer::SaveCallback saveCallback,
                                           IShaderViewer::RevertCallback revertCallback)
@@ -2479,8 +2479,8 @@ IShaderViewer *CaptureContext::EditShader(ResourceId id, ShaderStage stage,
         revertCallback(ctx, view, id);
     };
 
-    viewer = ShaderViewer::EditShader(*this, id, stage, entryPoint, files, shaderEncoding, flags,
-                                      replaceSaveCallback, replaceRevertCallback,
+    viewer = ShaderViewer::EditShader(*this, id, stage, entryPoint, files, knownTool, shaderEncoding,
+                                      flags, replaceSaveCallback, replaceRevertCallback,
                                       [this](ShaderViewer *view, bool closed) {
                                         SetModification(CaptureModifications::EditedShaders);
                                         if(closed)
@@ -2493,8 +2493,9 @@ IShaderViewer *CaptureContext::EditShader(ResourceId id, ShaderStage stage,
   }
   else
   {
-    viewer = ShaderViewer::EditShader(*this, id, stage, entryPoint, files, shaderEncoding, flags,
-                                      saveCallback, revertCallback, NULL, m_MainWindow->Widget());
+    viewer =
+        ShaderViewer::EditShader(*this, id, stage, entryPoint, files, knownTool, shaderEncoding,
+                                 flags, saveCallback, revertCallback, NULL, m_MainWindow->Widget());
   }
 
   return viewer;
