@@ -166,18 +166,18 @@ public:
   bool OpenRGPProfile(const rdcstr &filename) override;
   IRGPInterop *GetRGPInterop() override { return m_RGP; }
   const rdcarray<ActionDescription> &CurRootActions() override { return *m_Actions; }
-  ResourceDescription *GetResource(ResourceId id) override { return m_Resources[id]; }
+  const ResourceDescription *GetResource(ResourceId id) const override { return m_Resources[id]; }
   const rdcarray<ResourceDescription> &GetResources() override { return m_ResourceList; }
-  rdcstr GetResourceName(ResourceId id) override;
-  rdcstr GetResourceNameUnsuffixed(ResourceId id) override;
+  rdcstr GetResourceName(ResourceId id) const override;
+  rdcstr GetResourceNameUnsuffixed(ResourceId id) const override;
   bool IsAutogeneratedName(ResourceId id) override;
   bool HasResourceCustomName(ResourceId id) override;
   void SetResourceCustomName(ResourceId id, const rdcstr &name) override;
-  int32_t ResourceNameCacheID() override { return m_CustomNameCachedID; }
+  int32_t ResourceNameCacheID() const override { return m_CustomNameCachedID; }
   TextureDescription *GetTexture(ResourceId id) override { return m_Textures[id]; }
   const rdcarray<TextureDescription> &GetTextures() override { return m_TextureList; }
   BufferDescription *GetBuffer(ResourceId id) override { return m_Buffers[id]; }
-  const rdcarray<BufferDescription> &GetBuffers() override { return m_BufferList; }
+  const rdcarray<BufferDescription> &GetBuffers() const override { return m_BufferList; }
   const ActionDescription *GetAction(uint32_t eventId) override
   {
     return GetAction(*m_Actions, eventId);
@@ -245,8 +245,9 @@ public:
   void ShowResourceInspector() override;
 
   IShaderViewer *EditShader(ResourceId id, ShaderStage stage, const rdcstr &entryPoint,
-                            const rdcstrpairs &files, ShaderEncoding shaderEncoding,
-                            ShaderCompileFlags flags, IShaderViewer::SaveCallback saveCallback,
+                            const rdcstrpairs &files, KnownShaderTool knownTool,
+                            ShaderEncoding shaderEncoding, ShaderCompileFlags flags,
+                            IShaderViewer::SaveCallback saveCallback,
                             IShaderViewer::RevertCallback revertCallback) override;
 
   void ApplyShaderEdit(IShaderViewer *viewer, ResourceId id, ShaderStage stage,
@@ -323,7 +324,7 @@ private:
   void LoadEdits(const QString &data);
 
   void CacheResources();
-  rdcstr GetResourceNameUnsuffixed(const ResourceDescription *desc);
+  rdcstr GetResourceNameUnsuffixed(const ResourceDescription *desc) const;
 
   float m_LoadProgress = 0.0f;
   float m_PostloadProgress = 0.0f;
