@@ -90,6 +90,10 @@ DOCUMENT(R"(Specifies the panel to add a menu item into.
 .. data:: TextureViewer
 
   The :class:`TextureViewer`.
+
+.. data:: BufferViewer
+
+  Any non-mesh previewing :class:`BufferViewer`.
 )");
 enum class PanelMenu
 {
@@ -98,6 +102,7 @@ enum class PanelMenu
   PipelineStateViewer,
   MeshPreview,
   TextureViewer,
+  BufferViewer,
 };
 
 DECLARE_REFLECTION_ENUM(PanelMenu);
@@ -905,6 +910,98 @@ passed nothing will happen.
 )");
   virtual void SetComboOptions(QWidget *combo, const rdcarray<rdcstr> &options) = 0;
 
+  DOCUMENT(R"(Get the number of options in a drop-down combo box. If another type of widget is
+passed ``0`` will be returned.
+
+:param QWidget combo: The combo box.
+:return: The current number of options.
+:rtype: int
+)")
+  virtual size_t GetComboCount(QWidget *combo) = 0;
+
+  DOCUMENT(R"(Select the current option in a drop-down combo box. If another type of widget or
+an unknown option is passed, nothing will happen.
+
+:param QWidget combo: The combo box.
+:param str option: The option to select.
+)")
+  virtual void SelectComboOption(QWidget *combo, const rdcstr &option) = 0;
+
+  DOCUMENT(R"(Create a progress bar widget.
+
+By default the progress bar has minimum and maximum values of 0 and 100. These can be changed with
+:meth:`SetProgressBarRange`.
+
+:param bool horizontal: the progress bar orientation, true for horizontal otherwise vertical.
+:return: The handle to the newly created widget.
+:rtype: QWidget
+
+)")
+  virtual QWidget *CreateProgressBar(bool horizontal) = 0;
+
+  DOCUMENT(R"(Reset a progress bar widget.
+
+Rewinds the progress bar's indicator and hides the indicator's label (theme dependent). If you want
+to keep the label visible, call :meth:`SetProgressBarValue(0)` instead. The minimum and maximum values
+are not changed.
+
+:param QWidget pbar: the progress bar.
+)")
+  virtual void ResetProgressBar(QWidget *pbar) = 0;
+
+  DOCUMENT(R"(Set the progress bar's current value.
+
+Attempting to change the current value outside the minimum and maximum range does not affect
+the current value.
+
+:param QWidget pbar: the progress bar.
+:param int value: the new current value.
+)")
+  virtual void SetProgressBarValue(QWidget *pbar, int32_t value) = 0;
+
+  DOCUMENT(R"(Set the progress bar's current value relative to the existing value.
+
+:param QWidget pbar: the progress bar.
+:param int delta: the relative value to update the current value.
+)")
+  virtual void UpdateProgressBarValue(QWidget *pbar, int32_t delta) = 0;
+
+  DOCUMENT(R"(Get the progress bar's current value.
+
+:param QWidget pbar: the progress bar.
+:return: The current value of the progress bar.
+:rtype: int
+)")
+  virtual int32_t GetProgressBarValue(QWidget *pbar) = 0;
+
+  DOCUMENT(R"(Set a progress bar's minimum and maximum values.
+
+If maximum is smaller than minimum, minimum is set as the maximum, too. If the current value falls
+outside the new range, the progress bar is reset. Use range (0, 0) to set the progress bar to
+indeterminated state (the progress cannot be estimated or is not being calculated).
+
+:param QWidget pbar: the progress bar.
+:param int minimum: the minimum value.
+:param int maximum: the maximum value.
+)")
+  virtual void SetProgressBarRange(QWidget *pbar, int32_t minimum, int32_t maximum) = 0;
+
+  DOCUMENT(R"(Get the minimum value of the progress bar's range.
+
+:param QWidget pbar: the progress bar.
+:return: The minimum value of the range.
+:rtype: int
+)")
+  virtual int32_t GetProgressBarMinimum(QWidget *pbar) = 0;
+
+  DOCUMENT(R"(Get the maximum value of the progress bar's range.
+
+:param QWidget pbar: the progress bar.
+:return: The maximum value of the range.
+:rtype: int
+)")
+  virtual int32_t GetProgressBarMaximum(QWidget *pbar) = 0;
+
 protected:
   DOCUMENT("");
   IMiniQtHelper() = default;
