@@ -36,22 +36,6 @@ static const QString hlsl_stage2[arraydim<ShaderStage>()] = {
     lit("vs"), lit("hs"), lit("ds"), lit("gs"), lit("ps"), lit("cs"),
 };
 
-template <>
-rdcstr DoStringise(const KnownShaderTool &el)
-{
-  BEGIN_ENUM_STRINGISE(KnownShaderTool);
-  {
-    STRINGISE_ENUM_CLASS_NAMED(Unknown, "Custom Tool");
-    STRINGISE_ENUM_CLASS_NAMED(SPIRV_Cross, "SPIRV-Cross");
-    STRINGISE_ENUM_CLASS_NAMED(spirv_dis, "spirv-dis");
-    STRINGISE_ENUM_CLASS_NAMED(glslangValidatorGLSL, "glslang (GLSL)");
-    STRINGISE_ENUM_CLASS_NAMED(glslangValidatorHLSL, "glslang (HLSL)");
-    STRINGISE_ENUM_CLASS_NAMED(spirv_as, "spirv-as");
-    STRINGISE_ENUM_CLASS_NAMED(dxc, "dxc");
-  }
-  END_ENUM_STRINGISE();
-}
-
 static QString tmpPath(const QString &filename)
 {
   return QDir(QDir::tempPath()).absoluteFilePath(filename);
@@ -171,15 +155,65 @@ static ShaderToolOutput RunTool(const ShaderProcessingTool &tool, QWidget *windo
 
   QString processStatus;
 
+  QProcess::ProcessError error = process.error();
+
   if(process.exitStatus() == QProcess::CrashExit)
+    error = QProcess::Crashed;
+
+  switch(error)
   {
-    processStatus = QApplication::translate("ShaderProcessingTool", "Process crashed with code %1.")
-                        .arg(process.exitCode());
-  }
-  else
-  {
-    processStatus = QApplication::translate("ShaderProcessingTool", "Process exited with code %1.")
-                        .arg(process.exitCode());
+    case QProcess::FailedToStart:
+    {
+      if(QDir::isAbsolutePath(tool.executable))
+      {
+        if(!QFile::exists(tool.executable))
+        {
+          processStatus =
+              QApplication::translate("ShaderProcessingTool",
+                                      "Process couldn't be started, \"%1\" does not exist.")
+                  .arg(tool.executable);
+        }
+        else
+        {
+          processStatus = QApplication::translate(
+                              "ShaderProcessingTool",
+                              "Process couldn't be started, is \"%1\" a working executable?")
+                              .arg(tool.executable);
+        }
+      }
+      else
+      {
+        processStatus =
+            QApplication::translate(
+                "ShaderProcessingTool",
+                "Process couldn't be started, \"%1\" was located as \"%2\" but didn't start.")
+                .arg(tool.executable)
+                .arg(path);
+      }
+      break;
+    }
+    case QProcess::Crashed:
+    {
+      processStatus =
+          QApplication::translate("ShaderProcessingTool", "Process crashed with code %1.")
+              .arg(process.exitCode());
+      break;
+    }
+    case QProcess::ReadError:
+    case QProcess::WriteError:
+    {
+      processStatus =
+          QApplication::translate("ShaderProcessingTool", "Process failed during I/O with code %1.")
+              .arg(process.exitCode());
+      break;
+    }
+    case QProcess::Timedout:        // shouldn't happen, we don't use a timeout
+    case QProcess::UnknownError:    // return value if nothing went wrong
+    {
+      processStatus =
+          QApplication::translate("ShaderProcessingTool", "Process exited with code %1.")
+              .arg(process.exitCode());
+    }
   }
 
   ret.log = QApplication::translate("ShaderProcessingTool",
@@ -201,6 +235,9 @@ ShaderToolOutput ShaderProcessingTool::DisassembleShader(QWidget *window,
                                                          rdcstr arguments) const
 {
   QStringList argList = ParseArgsList(arguments.isEmpty() ? DefaultArguments() : arguments);
+  // always append IO arguments for known tools, so we read/write to our own files and override any
+  // dangling output specified file in the embedded command line
+  argList.append(ParseArgsList(IOArguments()));
 
   QString input_file, output_file;
 
@@ -253,6 +290,9 @@ ShaderToolOutput ShaderProcessingTool::CompileShader(QWidget *window, rdcstr sou
                                                      rdcstr arguments) const
 {
   QStringList argList = ParseArgsList(arguments.isEmpty() ? DefaultArguments() : arguments);
+  // always append IO arguments for known tools, so we read/write to our own files and override any
+  // dangling output specified file in the embedded command line
+  argList.append(ParseArgsList(IOArguments()));
 
   QString input_file, output_file;
 
