@@ -29,6 +29,7 @@
 #include <QGridLayout>
 #include <QGroupBox>
 #include <QHBoxLayout>
+#include <QProgressBar>
 #include <QPushButton>
 #include <QRadioButton>
 #include <QVBoxLayout>
@@ -419,7 +420,7 @@ void MiniQtHelper::SetWidgetFont(QWidget *widget, const rdcstr &font, int32_t fo
 
   QFont f = widget->font();
 
-  if(font.empty())
+  if(!font.empty())
     f.setFamily(font);
   if(fontSize != 0)
     f.setPointSize(fontSize);
@@ -697,6 +698,123 @@ void MiniQtHelper::SetComboOptions(QWidget *combo, const rdcarray<rdcstr> &optio
   for(const rdcstr &o : options)
     texts << o;
 
-  comb->clear();
-  comb->addItems(texts);
+  if(comb)
+  {
+    comb->clear();
+    comb->addItems(texts);
+  }
+}
+
+size_t MiniQtHelper::GetComboCount(QWidget *combo)
+{
+  if(!combo)
+    return 0;
+  const QComboBox *comb = qobject_cast<const QComboBox *>(combo);
+
+  if(comb)
+    return comb->count();
+
+  return 0;
+}
+
+void MiniQtHelper::SelectComboOption(QWidget *combo, const rdcstr &option)
+{
+  if(!combo)
+    return;
+  QComboBox *comb = qobject_cast<QComboBox *>(combo);
+
+  if(comb)
+    comb->setCurrentText(option);
+}
+
+QWidget *MiniQtHelper::CreateProgressBar(bool horizontal)
+{
+  QProgressBar *w = new QProgressBar();
+
+  w->setOrientation(horizontal ? Qt::Horizontal : Qt::Vertical);
+
+  return w;
+}
+
+void MiniQtHelper::ResetProgressBar(QWidget *pbar)
+{
+  if(!pbar)
+    return;
+
+  QProgressBar *pb = qobject_cast<QProgressBar *>(pbar);
+
+  if(pb)
+    pb->reset();
+}
+
+void MiniQtHelper::SetProgressBarValue(QWidget *pbar, int32_t value)
+{
+  if(!pbar)
+    return;
+
+  QProgressBar *pb = qobject_cast<QProgressBar *>(pbar);
+
+  if(pb)
+    pb->setValue(value);
+}
+
+void MiniQtHelper::UpdateProgressBarValue(QWidget *pbar, int32_t delta)
+{
+  if(!pbar)
+    return;
+
+  QProgressBar *pb = qobject_cast<QProgressBar *>(pbar);
+
+  if(pb)
+    pb->setValue(pb->value() + delta);
+}
+
+int32_t MiniQtHelper::GetProgressBarValue(QWidget *pbar)
+{
+  if(!pbar)
+    return 0;
+
+  QProgressBar *pb = qobject_cast<QProgressBar *>(pbar);
+
+  if(pb)
+    return pb->value();
+
+  return 0;
+}
+
+void MiniQtHelper::SetProgressBarRange(QWidget *pbar, int32_t minimum, int32_t maximum)
+{
+  if(!pbar)
+    return;
+
+  QProgressBar *pb = qobject_cast<QProgressBar *>(pbar);
+
+  if(pb)
+    pb->setRange(minimum, maximum);
+}
+
+int32_t MiniQtHelper::GetProgressBarMinimum(QWidget *pbar)
+{
+  if(!pbar)
+    return 0;
+
+  QProgressBar *pb = qobject_cast<QProgressBar *>(pbar);
+
+  if(pb)
+    return pb->minimum();
+
+  return 0;
+}
+
+int32_t MiniQtHelper::GetProgressBarMaximum(QWidget *pbar)
+{
+  if(!pbar)
+    return 0;
+
+  QProgressBar *pb = qobject_cast<QProgressBar *>(pbar);
+
+  if(pbar)
+    return pb->maximum();
+
+  return 0;
 }
