@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2021 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -53,9 +53,9 @@ public:
   void DeleteCapture(const rdcstr &capturefile, bool local);
 
   bool IsRunning();
-  ReplayStatus GetCreateStatus() { return m_CreateStatus; }
+  ResultDetails GetCreateStatus() { return m_CreateResult; }
   void SetFatalErrorCallback(std::function<void()> cb) { m_FatalErrorCallback = cb; }
-  ReplayStatus GetFatalError() { return m_FatalError; }
+  ResultDetails GetFatalError() { return m_FatalError; }
   float GetCurrentProcessingTime();
   QString GetCurrentProcessingTag();
   // this tagged version is for cases when we might send a request - e.g. to pick a vertex or pixel
@@ -72,7 +72,7 @@ public:
 
   void CloseThread();
 
-  ReplayStatus ConnectToRemoteServer(RemoteHost host);
+  ResultDetails ConnectToRemoteServer(RemoteHost host);
   void DisconnectFromRemoteServer();
   void ShutdownServer();
   void PingRemote();
@@ -139,8 +139,8 @@ private:
   IRemoteServer *m_OrphanedRemote = NULL;
 
   std::function<void()> m_FatalErrorCallback;
-  ReplayStatus m_FatalError = ReplayStatus::Succeeded;
+  ResultDetails m_FatalError = {ResultCode::Succeeded};
   volatile bool m_Running;
   LambdaThread *m_Thread;
-  ReplayStatus m_CreateStatus = ReplayStatus::Succeeded;
+  ResultDetails m_CreateResult = {ResultCode::Succeeded};
 };
