@@ -74,6 +74,11 @@ rdcstr ShaderProcessingTool::DefaultArguments() const
   return "";
 }
 
+rdcstr ShaderProcessingTool::IOArguments() const
+{
+  return "";
+}
+
 ShaderToolOutput ShaderProcessingTool::DisassembleShader(QWidget *window,
                                                          const ShaderReflection *shaderDetails,
                                                          rdcstr arguments) const
@@ -217,6 +222,11 @@ bool RemoteHost::IsVersionMismatch() const
   return false;
 }
 
+rdcstr RemoteHost::VersionMismatchError() const
+{
+  return rdcstr();
+}
+
 rdcstr RemoteHost::FriendlyName() const
 {
   return rdcstr();
