@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2021 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,15 +24,40 @@
 
 #pragma once
 
+#include <QMap>
 #include <QWidget>
 
 struct ICaptureContext;
+class ScintillaEdit;
+
+class RDTreeWidgetItem;
+class RDSplitter;
 
 namespace Ui
 {
 class BufferFormatSpecifier;
 }
 
+class BufferFormatList : public QObject
+{
+  Q_OBJECT
+
+  ICaptureContext &m_Ctx;
+  QMap<QString, QString> formats;
+
+public:
+  explicit BufferFormatList(ICaptureContext &ctx, QObject *parent = 0);
+  QStringList getFormats() { return formats.keys(); }
+  QString getFormat(QString name) { return formats[name]; }
+  bool hasFormat(QString name) { return formats.contains(name); }
+  void setFormat(QString name, QString format);
+
+signals:
+  void formatListUpdated();
+};
+
+extern BufferFormatList *globalFormatList;
+
 class BufferFormatSpecifier : public QWidget
 {
   Q_OBJECT
@@ -41,16 +66,29 @@ public:
   explicit BufferFormatSpecifier(QWidget *parent = 0);
   ~BufferFormatSpecifier();
 
+  void setAutoFormat(QString autoFormat);
+
   void setContext(ICaptureContext *ctx);
+  void setTitle(QString title);
+  void setErrors(const QMap<int, QString> &errors);
 
 signals:
   void processFormat(const QString &format);
 
 public slots:
-  void toggleHelp();
+  // automatic slots
+  void on_showHelp_toggled(bool help);
+  void on_loadDef_clicked();
+  void on_saveDef_clicked();
+  void on_delDef_clicked();
+  void on_savedList_keyPress(QKeyEvent *event);
+  void on_savedList_itemChanged(RDTreeWidgetItem *item, int column);
+  void on_savedList_itemDoubleClicked(RDTreeWidgetItem *item, int column);
+  void on_savedList_itemSelectionChanged();
+
+  // manual slots
   void setFormat(const QString &format);
-  void setErrors(const QString &errors);
-  void showHelp(bool help);
+  void updateFormatList();
 
 private slots:
   void on_apply_clicked();
@@ -58,4 +96,10 @@ private slots:
 private:
   Ui::BufferFormatSpecifier *ui;
   ICaptureContext *m_Ctx;
+
+  ScintillaEdit *formatText;
+
+  RDSplitter *m_Splitter = NULL;
+
+  QString m_AutoFormat;
 };
