@@ -34,6 +34,7 @@ class ResourcePreview;
 
 struct IReplayOutput;
 struct ICaptureContext;
+class RDLabel;
 
 class ResourcePreview : public QFrame
 {
@@ -41,11 +42,17 @@ class ResourcePreview : public QFrame
 
 public:
   explicit ResourcePreview(ICaptureContext &c, IReplayOutput *output, QWidget *parent = 0);
+  // create a manually rendered preview
+  explicit ResourcePreview(bool, QWidget *parent = 0);
+
+  void Initialise();
+
   ~ResourcePreview();
 
 signals:
   void clicked(QMouseEvent *e);
   void doubleClicked(QMouseEvent *e);
+  void resized(ResourcePreview *prev);
 
 public:
   void setSlotName(const QString &n);
@@ -55,6 +62,8 @@ public:
   void doubleClickEvent(QMouseEvent *e);
 
   WindowingData GetWidgetWindowingData();
+  QSize GetThumbSize();
+  void UpdateThumb(QSize s, const bytebuf &imgData);
 
   void setActive(bool b)
   {
@@ -68,8 +77,12 @@ public:
   void setSelected(bool sel);
 
 private:
+  virtual void resizeEvent(QResizeEvent *event);
+
   Ui::ResourcePreview *ui;
 
+  RDLabel *m_ManualThumbnail = NULL;
+
   bool m_Active;
   bool m_Selected = false;
 };
