@@ -76,7 +76,7 @@ struct BBoxData
 struct CBufferData
 {
   bool valid = false;
-  bool bufferBacked = false;
+  bool bytesBacked = false;
   ResourceId pipe;
   ResourceId shader;
   rdcstr entryPoint;
@@ -104,6 +104,7 @@ public:
   QWidget *Widget() override { return this; }
   void ScrollToRow(int32_t row, MeshDataStage stage = MeshDataStage::VSIn) override;
   void ScrollToColumn(int32_t column, MeshDataStage stage = MeshDataStage::VSIn) override;
+  void ShowMeshData(MeshDataStage stage) override;
   void SetCurrentInstance(int32_t instance) override;
   void SetCurrentView(int32_t view) override;
   void SetPreviewStage(MeshDataStage stage) override;
@@ -259,6 +260,7 @@ private:
   int m_IdxColWidth;
   int m_DataColWidth;
   int m_DataRowHeight;
+  const int m_ErrorColWidth = 500;
 
   int previousAxisMappingIndex = 0;
 
@@ -315,7 +317,7 @@ private:
   void EnableCameraGuessControls();
 
   void CalcColumnWidth(int maxNumRows = 1);
-  void ApplyRowAndColumnDims(int numColumns, RDTableView *view);
+  void ApplyRowAndColumnDims(int numColumns, RDTableView *view, int dataColWidth);
 
   void SyncViews(RDTableView *primary, bool selection, bool scroll);
   void UpdateHighlightVerts();
