@@ -654,6 +654,8 @@ void LiveCapture::updateAPIStatus()
     if(!m_APIs[api].supported)
     {
       apiStatus += tr(", %1 (Unsupported)").arg(api);
+      if(!m_APIs[api].supportMessage.isEmpty())
+        apiStatus += lit("\n") + m_APIs[api].supportMessage;
     }
     else if(!m_APIs[api].presenting)
     {
@@ -665,6 +667,8 @@ void LiveCapture::updateAPIStatus()
   // remove the redundant starting ", "
   apiStatus.remove(0, 2);
 
+  apiStatus.replace(QLatin1Char('\n'), lit("<br>"));
+
   ui->apiStatus->setText(apiStatus);
 
   ui->apiIcon->setVisible(nonpresenting);
@@ -677,7 +681,9 @@ QString LiveCapture::MakeText(Capture *cap)
     text += tr(" (Remote)");
 
   text += lit("\n") + cap->api;
-  if(cap->frameNumber == ~0U)
+  if(!cap->title.isEmpty())
+    text += QFormatStr("\n%1").arg(cap->title);
+  else if(cap->frameNumber == ~0U)
     text += tr("\nUser-defined Capture");
   else
     text += tr("\nFrame #%1").arg(cap->frameNumber);
@@ -1076,6 +1082,8 @@ void LiveCapture::on_captures_itemSelectionChanged()
   int numSelected = ui->captures->selectedItems().size();
 
   openButton->setEnabled(numSelected == 1);
+  saveAction->setEnabled(numSelected != 0);
+  deleteAction->setEnabled(numSelected != 0);
 
   if(ui->captures->selectedItems().size() == 1)
   {
@@ -1139,6 +1147,7 @@ void LiveCapture::captureAdded(const QString &name, const NewCaptureData &newCap
   cap->path = newCapture.path;
   cap->local = newCapture.local;
   cap->frameNumber = newCapture.frameNumber;
+  cap->title = newCapture.title;
 
   QListWidgetItem *item = new QListWidgetItem();
   item->setFlags(item->flags() | Qt::ItemIsEditable);
@@ -1342,13 +1351,11 @@ void LiveCapture::connectionThreadEntry()
 
     if(msg.type == TargetControlMessageType::RegisterAPI)
     {
-      QString api = msg.apiUse.name;
-      bool presenting = msg.apiUse.presenting;
-      bool supported = msg.apiUse.supported;
-      GUIInvoke::call(this, [this, api, presenting, supported]() {
-        m_APIs[api] = APIStatus(presenting, supported);
+      GUIInvoke::call(this, [this, msg]() {
+        m_APIs[msg.apiUse.name] =
+            APIStatus(msg.apiUse.presenting, msg.apiUse.supported, msg.apiUse.supportMessage);
 
-        if(presenting && supported)
+        if(msg.apiUse.presenting && msg.apiUse.supported)
         {
           ui->triggerImmediateCapture->setEnabled(true);
           ui->triggerDelayedCapture->setEnabled(true);
