@@ -107,6 +107,7 @@ private:
     QDateTime timestamp;
     uint32_t frameNumber;
     uint64_t byteSize;
+    QString title;
 
     QImage thumb;
 
@@ -127,9 +128,10 @@ private:
   struct APIStatus
   {
     APIStatus() = default;
-    APIStatus(bool p, bool s) : presenting(p), supported(s) {}
+    APIStatus(bool p, bool s, rdcstr m) : presenting(p), supported(s), supportMessage(m) {}
     bool presenting = false;
     bool supported = false;
+    rdcstr supportMessage;
   };
 
   Capture *GetCapture(QListWidgetItem *item);
