@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2021 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -44,6 +44,8 @@ struct EventItemTag;
 class RDLabel;
 class RDTreeWidget;
 class RDTreeWidgetItem;
+class RDLineEdit;
+class RDToolButton;
 class RDTextEdit;
 class QListWidget;
 class QCheckBox;
@@ -180,6 +182,9 @@ private slots:
   void events_keyPress(QKeyEvent *event);
   void events_contextMenu(const QPoint &pos);
   void events_currentChanged(const QModelIndex &current, const QModelIndex &previous);
+  void locationEdit_clicked();
+  void location_leave();
+  void location_keyPress(QKeyEvent *e);
 
 private:
   void ExpandNode(QModelIndex idx);
@@ -230,6 +235,9 @@ private:
   QSpacerItem *m_BookmarkSpacer;
   QMap<uint32_t, QToolButton *> m_BookmarkButtons;
 
+  RDLineEdit *m_BreadcrumbLocationText;
+  RDToolButton *m_BreadcrumbLocationEditButton;
+  QString m_InitialBreadcrumbLocation;
   MarkerBreadcrumbs *m_Breadcrumbs;
 
   struct
