@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2021 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -133,7 +133,7 @@ public:
   void showPipelineViewer() { on_action_Pipeline_State_triggered(); }
   void showCaptureDialog() { on_action_Launch_Application_triggered(); }
   void showDebugMessageView() { on_action_Errors_and_Warnings_triggered(); }
-  void showDiagnosticLogView() { on_action_View_Diagnostic_Log_File_triggered(); }
+  void showDiagnosticLogView();
   void showCommentView() { on_action_Comments_triggered(); }
   void showStatisticsViewer() { on_action_Statistics_Viewer_triggered(); }
   void showTimelineBar() { on_action_Timeline_triggered(); }
@@ -175,8 +175,6 @@ private slots:
   void on_action_Manage_Remote_Servers_triggered();
   void on_action_Settings_triggered();
   void on_action_View_Documentation_triggered();
-  void on_action_View_Diagnostic_Log_File_triggered();
-  void on_action_Diagnostic_Log_triggered() { on_action_View_Diagnostic_Log_File_triggered(); }
   void on_action_Source_on_GitHub_triggered();
   void on_action_Build_Release_Downloads_triggered();
   void on_action_Show_Tips_triggered();
@@ -297,7 +295,7 @@ private:
 
   void FillRemotesMenu(QMenu *menu, bool includeLocalhost);
 
-  void showLaunchError(ReplayStatus status);
+  void showLaunchError(ResultDetails result);
 
   bool isUnshareableDeviceInUse();
 };
