@@ -978,10 +978,12 @@ void PipelineStateViewer::shaderEdit_clicked()
 
 IShaderViewer *PipelineStateViewer::EditShader(ResourceId id, ShaderStage shaderType,
                                                const rdcstr &entry, ShaderCompileFlags compileFlags,
-                                               ShaderEncoding encoding, const rdcstrpairs &files)
+                                               KnownShaderTool knownTool,
+                                               ShaderEncoding shaderEncoding,
+                                               const rdcstrpairs &files)
 {
-  IShaderViewer *sv =
-      m_Ctx.EditShader(id, shaderType, entry, files, encoding, compileFlags, NULL, NULL);
+  IShaderViewer *sv = m_Ctx.EditShader(id, shaderType, entry, files, knownTool, shaderEncoding,
+                                       compileFlags, NULL, NULL);
 
   m_Ctx.AddDockWindow(sv->Widget(), DockReference::AddTo, this);
 
@@ -994,8 +996,22 @@ IShaderViewer *PipelineStateViewer::EditOriginalShaderSource(ResourceId id,
   QSet<uint> uniqueFiles;
   rdcstrpairs files;
 
-  for(const ShaderSourceFile &s : shaderDetails->debugInfo.files)
+  // add the entry point file first, if we have one
+  const int entryFile = shaderDetails->debugInfo.editBaseFile;
+
+  for(int i = -1; i < shaderDetails->debugInfo.files.count(); i++)
   {
+    int idx = i;
+    if(idx < 0)
+      idx = entryFile;
+    else if(idx == entryFile)
+      continue;
+
+    if(idx < 0)
+      continue;
+
+    const ShaderSourceFile &s = shaderDetails->debugInfo.files[idx];
+
     QString filename = s.filename;
 
     uint filenameHash = qHash(filename.toLower());
@@ -1011,7 +1027,8 @@ IShaderViewer *PipelineStateViewer::EditOriginalShaderSource(ResourceId id,
   }
 
   return EditShader(id, shaderDetails->stage, shaderDetails->entryPoint,
-                    shaderDetails->debugInfo.compileFlags, shaderDetails->debugInfo.encoding, files);
+                    shaderDetails->debugInfo.compileFlags, shaderDetails->debugInfo.compiler,
+                    shaderDetails->debugInfo.encoding, files);
 }
 
 IShaderViewer *PipelineStateViewer::EditDecompiledSource(const ShaderProcessingTool &tool,
@@ -1026,8 +1043,8 @@ IShaderViewer *PipelineStateViewer::EditDecompiledSource(const ShaderProcessingT
   rdcstrpairs files;
   files.push_back(rdcpair<rdcstr, rdcstr>("decompiled", source));
 
-  IShaderViewer *sv = EditShader(id, shaderDetails->stage, shaderDetails->entryPoint,
-                                 shaderDetails->debugInfo.compileFlags, tool.output, files);
+  IShaderViewer *sv = EditShader(id, shaderDetails->stage, shaderDetails->entryPoint, {},
+                                 KnownShaderTool::Unknown, tool.output, files);
 
   sv->ShowErrors(out.log);
 
@@ -1048,13 +1065,16 @@ void PipelineStateViewer::SetupShaderEditButton(QToolButton *button, ResourceId
 
   rdcarray<ShaderEncoding> accepted = m_Ctx.TargetShaderEncodings();
 
+  const ShaderDebugInfo &dbg = shaderDetails->debugInfo;
+
   // if we have original source and it's in a known format, display it as the first most preferred
   // option
-  if(!shaderDetails->debugInfo.files.empty() &&
-     shaderDetails->debugInfo.encoding != ShaderEncoding::Unknown)
+  if(!dbg.files.empty() && dbg.encoding != ShaderEncoding::Unknown)
   {
-    QAction *action =
-        new QAction(tr("Edit Source - %1").arg(shaderDetails->debugInfo.files[0].filename), menu);
+    int entryFile = qMax(0, dbg.entryLocation.fileIndex);
+    if(dbg.editBaseFile >= 0 && dbg.editBaseFile < dbg.files.count())
+      entryFile = dbg.editBaseFile;
+    QAction *action = new QAction(tr("Edit Source - %1").arg(dbg.files[entryFile].filename), menu);
     action->setIcon(Icons::page_white_edit());
 
     QObject::connect(action, &QAction::triggered, [this, shaderId, shaderDetails]() {
@@ -1116,11 +1136,13 @@ void PipelineStateViewer::SetupShaderEditButton(QToolButton *button, ResourceId
             files.push_back(rdcpair<rdcstr, rdcstr>("pseudocode", editeddisasm));
 
             EditShader(shaderId, shaderDetails->stage, shaderDetails->entryPoint,
-                       shaderDetails->debugInfo.compileFlags, ShaderEncoding::Unknown, files);
+                       shaderDetails->debugInfo.compileFlags, KnownShaderTool::Unknown,
+                       ShaderEncoding::Unknown, files);
           });
         });
       }
-      else if(shaderDetails->encoding == ShaderEncoding::DXBC)
+      else if(shaderDetails->encoding == ShaderEncoding::DXBC ||
+              shaderDetails->encoding == ShaderEncoding::DXIL)
       {
         entry = lit("EditedShader%1S").arg(ToQStr(shaderDetails->stage, GraphicsAPI::D3D11)[0]);
 
@@ -1129,7 +1151,7 @@ void PipelineStateViewer::SetupShaderEditButton(QToolButton *button, ResourceId
             "decompiled_stub.hlsl", GenerateHLSLStub(bindpointMapping, shaderDetails, entry)));
 
         EditShader(shaderId, shaderDetails->stage, entry, shaderDetails->debugInfo.compileFlags,
-                   ShaderEncoding::HLSL, files);
+                   KnownShaderTool::Unknown, ShaderEncoding::HLSL, files);
       }
 
     });
@@ -1476,6 +1498,12 @@ QString PipelineStateViewer::GetVBufferFormatString(uint32_t slot)
   return format;
 }
 
+QColor PipelineStateViewer::GetViewDetailsColor()
+{
+  return QColor::fromHslF(0.45f, 1.0f,
+                          qBound(0.25, palette().color(QPalette::Base).lightnessF(), 0.75));
+}
+
 bool PipelineStateViewer::SaveShaderFile(const ShaderReflection *shader)
 {
   if(!shader)
@@ -1490,6 +1518,7 @@ bool PipelineStateViewer::SaveShaderFile(const ShaderReflection *shader)
     case ShaderEncoding::GLSL: filter = tr("GLSL files (*.glsl)"); break;
     case ShaderEncoding::SPIRV: filter = tr("SPIR-V files (*.spv)"); break;
     case ShaderEncoding::SPIRVAsm: filter = tr("SPIR-V assembly files (*.spvasm)"); break;
+    case ShaderEncoding::DXIL: filter = tr("DXIL Shader files (*.dxbc)"); break;
     case ShaderEncoding::Unknown:
     case ShaderEncoding::Count: filter = tr("All files (*.*)"); break;
   }
