@@ -109,6 +109,8 @@ public:
 
   QString GetVBufferFormatString(uint32_t slot);
 
+  QColor GetViewDetailsColor();
+
   void setTopologyDiagram(QLabel *diagram, Topology topo);
   void setMeshViewPixmap(RDLabel *meshView);
 
@@ -146,8 +148,8 @@ private:
   QString GenerateHLSLStub(const ShaderBindpointMapping &bindpointMapping,
                            const ShaderReflection *shaderDetails, const QString &entryFunc);
   IShaderViewer *EditShader(ResourceId id, ShaderStage shaderType, const rdcstr &entry,
-                            ShaderCompileFlags compileFlags, ShaderEncoding encoding,
-                            const rdcstrpairs &files);
+                            ShaderCompileFlags compileFlags, KnownShaderTool knownTool,
+                            ShaderEncoding shaderEncoding, const rdcstrpairs &files);
   IShaderViewer *EditOriginalShaderSource(ResourceId id, const ShaderReflection *shaderDetails);
   IShaderViewer *EditDecompiledSource(const ShaderProcessingTool &tool, ResourceId id,
                                       const ShaderReflection *shaderDetails);
