@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +24,7 @@
 
 #pragma once
 
+#include <QDir>
 #include <QFrame>
 #include "Code/Interface/QRDInterface.h"
 
@@ -34,6 +35,7 @@ class VulkanPipelineStateViewer;
 
 class QXmlStreamWriter;
 
+class ComputeDebugSelector;
 class RDLabel;
 class RDTreeWidget;
 class RDTreeWidgetItem;
@@ -61,12 +63,13 @@ public:
   void OnEventChanged(uint32_t eventId);
 
   void SelectPipelineStage(PipelineStage stage);
+  ResourceId GetResource(RDTreeWidgetItem *item);
 
 private slots:
   // automatic slots
   void on_showUnused_toggled(bool checked);
   void on_showEmpty_toggled(bool checked);
-  void on_exportHTML_clicked();
+
   void on_meshView_clicked();
   void on_viAttrs_itemActivated(RDTreeWidgetItem *item, int column);
   void on_viBuffers_itemActivated(RDTreeWidgetItem *item, int column);
@@ -76,20 +79,26 @@ private slots:
 
   // manual slots
   void shaderView_clicked();
-
   void shaderSave_clicked();
+  void shaderMessages_clicked();
   void predicateBufferView_clicked();
   void resource_itemActivated(RDTreeWidgetItem *item, int column);
   void resource_hoverItemChanged(RDTreeWidgetItem *hover);
   void ubo_itemActivated(RDTreeWidgetItem *item, int column);
   void vertex_leave(QEvent *e);
 
-  void on_debugThread_clicked();
+  void on_computeDebugSelector_clicked();
+  void computeDebugSelector_beginDebug(const rdcfixedarray<uint32_t, 3> &group,
+                                       const rdcfixedarray<uint32_t, 3> &thread);
+
+  void exportHTML_clicked();
+  void exportFOZ_clicked();
 
 private:
   Ui::VulkanPipelineStateViewer *ui;
   ICaptureContext &m_Ctx;
   PipelineStateViewer &m_Common;
+  ComputeDebugSelector *m_ComputeDebugSelector;
 
   QVariantList makeSampler(const QString &bindset, const QString &slotname,
                            const VKPipe::BindingElement &descriptor);
@@ -116,7 +125,7 @@ private:
   template <typename viewType>
   bool setViewDetails(RDTreeWidgetItem *node, const viewType &view, TextureDescription *tex,
                       bool stageBitsIncluded, const QString &hiddenCombinedSampler,
-                      bool includeSampleLocations = false);
+                      bool includeSampleLocations = false, bool includeOffsets = false);
 
   template <typename viewType>
   bool setViewDetails(RDTreeWidgetItem *node, const viewType &view, BufferDescription *buf,
@@ -137,6 +146,22 @@ private:
   void exportHTML(QXmlStreamWriter &xml, const VKPipe::CurrentPass &pass);
   void exportHTML(QXmlStreamWriter &xml, const VKPipe::ConditionalRendering &cr);
 
+  QString GetFossilizeHash(ResourceId id);
+  QString GetFossilizeFilename(QDir d, uint32_t tag, ResourceId id);
+  QVariant ConvertSDObjectToFossilizeJSON(const SDObject *obj, QMap<QByteArray, QByteArray> renames);
+  void AddFossilizeNexts(QVariantMap &info, const SDObject *baseStruct);
+  QByteArray ReconstructSpecializationData(const VKPipe::Shader &sh, const SDObject *mapEntries);
+  QString GetBufferForFossilize(const SDObject *obj);
+  void EncodeFossilizeVarint(const bytebuf &spirv, bytebuf &varint);
+  void WriteFossilizeJSON(QIODevice &f, QVariantMap &contents);
+
+  void exportFOZ(QString dir, ResourceId pso);
+
+  QMenu *m_ExportMenu = NULL;
+
+  QAction *m_ExportHTML = NULL;
+  QAction *m_ExportFOZ = NULL;
+
   // keep track of the VB nodes (we want to be able to highlight them easily on hover)
   QList<RDTreeWidgetItem *> m_VBNodes;
   QList<RDTreeWidgetItem *> m_BindNodes;
