@@ -302,6 +302,56 @@ struct MiniQtInvoker : ObjectForwarder<IMiniQtHelper>
   {
     InvokeVoidFunction(&IMiniQtHelper::SetComboOptions, combo, options);
   }
+
+  size_t GetComboCount(QWidget *combo)
+  {
+    return InvokeRetFunction<size_t>(&IMiniQtHelper::GetComboCount, combo);
+  }
+
+  void SelectComboOption(QWidget *combo, const rdcstr &option)
+  {
+    InvokeVoidFunction(&IMiniQtHelper::SelectComboOption, combo, option);
+  }
+
+  QWidget *CreateProgressBar(bool horizontal)
+  {
+    return InvokeRetFunction<QWidget *>(&IMiniQtHelper::CreateProgressBar, horizontal);
+  }
+
+  void ResetProgressBar(QWidget *pbar)
+  {
+    InvokeVoidFunction(&IMiniQtHelper::ResetProgressBar, pbar);
+  }
+
+  void SetProgressBarValue(QWidget *pbar, int32_t value)
+  {
+    InvokeVoidFunction(&IMiniQtHelper::SetProgressBarValue, pbar, value);
+  }
+
+  void UpdateProgressBarValue(QWidget *pbar, int32_t delta)
+  {
+    InvokeVoidFunction(&IMiniQtHelper::UpdateProgressBarValue, pbar, delta);
+  }
+
+  int32_t GetProgressBarValue(QWidget *pbar)
+  {
+    return InvokeRetFunction<int>(&IMiniQtHelper::GetProgressBarValue, pbar);
+  }
+
+  void SetProgressBarRange(QWidget *pbar, int32_t minimum, int32_t maximum)
+  {
+    InvokeVoidFunction(&IMiniQtHelper::SetProgressBarRange, pbar, minimum, maximum);
+  }
+
+  int32_t GetProgressBarMinimum(QWidget *pbar)
+  {
+    return InvokeRetFunction<int>(&IMiniQtHelper::GetProgressBarMinimum, pbar);
+  }
+
+  int32_t GetProgressBarMaximum(QWidget *pbar)
+  {
+    return InvokeRetFunction<int>(&IMiniQtHelper::GetProgressBarMaximum, pbar);
+  }
 };
 
 struct ExtensionInvoker : ObjectForwarder<IExtensionManager>
@@ -447,13 +497,16 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
   {
     return m_Obj.CurRootActions();
   }
-  virtual ResourceDescription *GetResource(ResourceId id) override { return m_Obj.GetResource(id); }
+  virtual const ResourceDescription *GetResource(ResourceId id) const override
+  {
+    return m_Obj.GetResource(id);
+  }
   virtual const rdcarray<ResourceDescription> &GetResources() override
   {
     return m_Obj.GetResources();
   }
-  virtual rdcstr GetResourceName(ResourceId id) override { return m_Obj.GetResourceName(id); }
-  virtual rdcstr GetResourceNameUnsuffixed(ResourceId id) override
+  virtual rdcstr GetResourceName(ResourceId id) const override { return m_Obj.GetResourceName(id); }
+  virtual rdcstr GetResourceNameUnsuffixed(ResourceId id) const override
   {
     return m_Obj.GetResourceNameUnsuffixed(id);
   }
@@ -462,11 +515,14 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
   {
     return m_Obj.HasResourceCustomName(id);
   }
-  virtual int32_t ResourceNameCacheID() override { return m_Obj.ResourceNameCacheID(); }
+  virtual int32_t ResourceNameCacheID() const override { return m_Obj.ResourceNameCacheID(); }
   virtual TextureDescription *GetTexture(ResourceId id) override { return m_Obj.GetTexture(id); }
   virtual const rdcarray<TextureDescription> &GetTextures() override { return m_Obj.GetTextures(); }
   virtual BufferDescription *GetBuffer(ResourceId id) override { return m_Obj.GetBuffer(id); }
-  virtual const rdcarray<BufferDescription> &GetBuffers() override { return m_Obj.GetBuffers(); }
+  virtual const rdcarray<BufferDescription> &GetBuffers() const override
+  {
+    return m_Obj.GetBuffers();
+  }
   virtual const ActionDescription *GetAction(uint32_t eventId) override
   {
     return m_Obj.GetAction(eventId);
@@ -748,13 +804,13 @@ struct CaptureContextInvoker : ObjectForwarder<ICaptureContext>
     InvokeVoidFunction(&ICaptureContext::ShowResourceInspector);
   }
   virtual IShaderViewer *EditShader(ResourceId id, ShaderStage stage, const rdcstr &entryPoint,
-                                    const rdcstrpairs &files, ShaderEncoding shaderEncoding,
-                                    ShaderCompileFlags flags,
+                                    const rdcstrpairs &files, KnownShaderTool knownTool,
+                                    ShaderEncoding shaderEncoding, ShaderCompileFlags flags,
                                     IShaderViewer::SaveCallback saveCallback,
                                     IShaderViewer::RevertCallback revertCallback) override
   {
     return InvokeRetFunction<IShaderViewer *>(&ICaptureContext::EditShader, id, stage, entryPoint,
-                                              files, shaderEncoding, flags, saveCallback,
+                                              files, knownTool, shaderEncoding, flags, saveCallback,
                                               revertCallback);
   }
 
