@@ -26,6 +26,7 @@
 
 #include <QFrame>
 #include <QSemaphore>
+#include <QSet>
 #include <QStyledItemDelegate>
 #include "Code/Interface/QRDInterface.h"
 
@@ -91,6 +92,8 @@ struct VariableTag
 
   bool matrix = false;
   bool expanded = false;
+  bool modified = false;
+  int updateID = 0;
   bool globalSourceVar = false;
   int32_t sourceVarIdx = -1;
 
@@ -113,8 +116,8 @@ public:
 
   static ShaderViewer *EditShader(ICaptureContext &ctx, ResourceId id, ShaderStage stage,
                                   const QString &entryPoint, const rdcstrpairs &files,
-                                  ShaderEncoding shaderEncoding, ShaderCompileFlags flags,
-                                  IShaderViewer::SaveCallback saveCallback,
+                                  KnownShaderTool knownTool, ShaderEncoding shaderEncoding,
+                                  ShaderCompileFlags flags, IShaderViewer::SaveCallback saveCallback,
                                   IShaderViewer::RevertCallback revertCallback,
                                   ModifyCallback modifyCallback, QWidget *parent)
   {
@@ -122,7 +125,7 @@ public:
     ret->m_SaveCallback = saveCallback;
     ret->m_RevertCallback = revertCallback;
     ret->m_ModifyCallback = modifyCallback;
-    ret->editShader(id, stage, entryPoint, files, shaderEncoding, flags);
+    ret->editShader(id, stage, entryPoint, files, knownTool, shaderEncoding, flags);
     return ret;
   }
 
@@ -205,7 +208,8 @@ private slots:
 private:
   explicit ShaderViewer(ICaptureContext &ctx, QWidget *parent = 0);
   void editShader(ResourceId id, ShaderStage stage, const QString &entryPoint,
-                  const rdcstrpairs &files, ShaderEncoding shaderEncoding, ShaderCompileFlags flags);
+                  const rdcstrpairs &files, KnownShaderTool knownTool,
+                  ShaderEncoding shaderEncoding, ShaderCompileFlags flags);
   void debugShader(const ShaderBindpointMapping *bind, const ShaderReflection *shader,
                    ResourceId pipeline, ShaderDebugTrace *trace, const QString &debugContext);
 
@@ -300,7 +304,10 @@ private:
   size_t m_FirstSourceStateIdx = ~0U;
   rdcarray<ShaderDebugState> m_States;
   size_t m_CurrentStateIdx = 0;
-  rdcarray<ShaderVariable> m_Variables;
+  QList<ShaderVariable> m_Variables;
+  uint32_t m_UpdateID = 1;
+  QMap<QString, uint32_t> m_VariableLastUpdate;
+  QList<rdcstr> m_VariablesChanged;
 
   // true when debugging while we're populating the initial trace. Lets us queue up commands and
   // process them once we've initialised properly
@@ -314,7 +321,8 @@ private:
 
   rdcarray<BoundResourceArray> m_ReadOnlyResources;
   rdcarray<BoundResourceArray> m_ReadWriteResources;
-  QList<int> m_Breakpoints;
+  QSet<QPair<int, uint32_t>> m_Breakpoints;
+  bool m_TempBreakpoint = false;
 
   QList<QPair<ScintillaEdit *, int>> m_FindAllResults;
 
@@ -352,6 +360,12 @@ private:
   const ShaderDebugState &GetCurrentState() const;
   const ShaderDebugState &GetNextState() const;
 
+  const InstructionSourceInfo &GetPreviousInstInfo() const;
+  const InstructionSourceInfo &GetCurrentInstInfo() const;
+  const InstructionSourceInfo &GetNextInstInfo() const;
+  const InstructionSourceInfo &GetInstInfo(uint32_t instruction) const;
+  const LineColumnInfo &GetCurrentLineInfo() const;
+
   void updateDebugState();
   void markWatchStale(RDTreeWidgetItem *item);
   bool updateWatchVariable(RDTreeWidgetItem *watchItem, const RDTreeWidgetItem *varItem,
@@ -362,11 +376,14 @@ private:
   void updateAccessedResources();
 
   RDTreeWidgetItem *makeSourceVariableNode(const ShaderVariable &var, const rdcstr &debugVarPath,
-                                           bool modified, VariableTag baseTag);
+                                           VariableTag baseTag);
   RDTreeWidgetItem *makeSourceVariableNode(const SourceVariableMapping &l, int globalVarIdx,
-                                           int localVarIdx, bool modified);
-  RDTreeWidgetItem *makeDebugVariableNode(const ShaderVariable &v, rdcstr prefix, bool modified);
-  RDTreeWidgetItem *makeAccessedResourceNode(const ShaderVariable &v, bool modified);
+                                           int localVarIdx);
+  RDTreeWidgetItem *makeDebugVariableNode(const ShaderVariable &v, rdcstr prefix);
+  RDTreeWidgetItem *makeAccessedResourceNode(const ShaderVariable &v);
+
+  bool HasChanged(rdcstr debugVarName) const;
+  uint32_t CalcUpdateID(uint32_t prevID, rdcstr debugVarName) const;
 
   const ShaderVariable *GetDebugVariable(const DebugVariableReference &r);
 
