@@ -143,6 +143,7 @@ public:
 
   Subresource GetSelectedSubresource() override;
   void SetSelectedSubresource(Subresource sub) override;
+  rdcpair<int32_t, int32_t> GetPickedLocation() override;
   void GotoLocation(uint32_t x, uint32_t y) override;
   DebugOverlay GetTextureOverlay() override;
   void SetTextureOverlay(DebugOverlay overlay) override;
@@ -274,6 +275,8 @@ private:
                                  rdcarray<BoundResourceArray> &ResList, ThumbnailStrip *prevs,
                                  int &prevIndex, bool copy, bool rw);
 
+  void UI_PreviewResized(ResourcePreview *prev);
+
   void AddResourceUsageEntry(QMenu &menu, uint32_t start, uint32_t end, ResourceUsage usage);
   void OpenResourceContextMenu(ResourceId id, bool input, const rdcarray<EventUsage> &usage);
 
@@ -306,6 +309,8 @@ private:
   void ShowGotoPopup();
 
   bool ShouldFlipForGL();
+  uint32_t MipCoordFromBase(int coord, uint32_t dim);
+  uint32_t BaseCoordFromMip(int coord, uint32_t dim);
 
   void UI_UpdateFittedScale();
   void UI_SetScale(float s);
