@@ -3,3 +3,7 @@
 // not being found properly.
 
 #define QT_NAMESPACE
+
+// This is used to conditionally stop visual assist from parsing swig-generated files
+
+#define VA_IGNORE_REST_OF_FILE _asm {
