@@ -99,6 +99,15 @@ struct MeshFormat
 
   DOCUMENT("``True`` if the primitive restart index feature should be used.");
   bool allowRestart = true;
+
+  DOCUMENT(R"(A string with the status of this mesh format - only used when a mesh format is
+returned to the application detailing e.g. vertex output data.
+
+An empty string indicates no errors/problems.
+
+:type: str
+)");
+  rdcstr status;
 };
 
 DECLARE_REFLECTION_STRUCT(MeshFormat);
@@ -525,6 +534,8 @@ struct NewCaptureData
   int32_t thumbHeight = 0;
   DOCUMENT("The local path on the target system where the capture is saved.");
   rdcstr path;
+  DOCUMENT("The custom title for this capture, if empty a default title can be used.");
+  rdcstr title;
   DOCUMENT(R"(The API used for this capture, if available.
 
 .. note::
@@ -553,6 +564,12 @@ struct APIUseData
 
   DOCUMENT("``True`` if the API can be captured.");
   bool supported = false;
+
+  DOCUMENT(R"(A string message if the API is unsupported explaining why.
+
+:type: str
+)");
+  rdcstr supportMessage;
 };
 
 DECLARE_REFLECTION_STRUCT(APIUseData);
