@@ -190,6 +190,14 @@ requirements.
   {
     return IsCaptureLoaded() && IsCaptureVK() && stage == MeshDataStage::VSOut;
   }
+
+  DOCUMENT(R"(Retrieves the rasterized stream, if multiple streams are being generated in the GS.
+
+:return: The rasterized stream, or -1 of no stream is being rasterized.
+:rtype: int
+)");
+  int32_t GetRasterizedStream() const;
+
   DOCUMENT(R"(For APIs that have explicit barriers, retrieves the current layout of a resource.
 
 :param ResourceId id: The ID of the resource to query for
@@ -311,14 +319,14 @@ For some APIs that don't distinguish by entry point, this may be empty.
 :return: A boolean indicating if primitive restart is enabled.
 :rtype: bool
 )");
-  bool IsStripRestartEnabled() const;
+  bool IsRestartEnabled() const;
 
   DOCUMENT(R"(Retrieves the primitive restart index.
 
-:return: The index value that represents a strip restart not a real index.
+:return: The index value that represents a primitive restart not a real index.
 :rtype: int
 )");
-  uint32_t GetStripRestartIndex() const;
+  uint32_t GetRestartIndex() const;
 
   DOCUMENT(R"(Retrieves the currently bound vertex buffers.
 
