@@ -222,13 +222,16 @@ struct ShaderVariable
   DOCUMENT("");
   ShaderVariable()
   {
-    name = "";
     rows = columns = 0;
     type = VarType::Unknown;
     memset(&value, 0, sizeof(value));
   }
   ShaderVariable(const ShaderVariable &) = default;
   ShaderVariable &operator=(const ShaderVariable &) = default;
+#if !defined(SWIG)
+  ShaderVariable(ShaderVariable &&) = default;
+  ShaderVariable &operator=(ShaderVariable &&) = default;
+#endif
   ShaderVariable(const rdcstr &n, float x, float y, float z, float w)
   {
     name = n;
@@ -428,6 +431,10 @@ struct DebugVariableReference
       : name(name), type(type), component(component)
   {
   }
+#if !defined(SWIG)
+  DebugVariableReference(DebugVariableReference &&) = default;
+  DebugVariableReference &operator=(DebugVariableReference &&) = default;
+#endif
   bool operator==(const DebugVariableReference &o) const
   {
     return name == o.name && type == o.type && component == o.component;
@@ -474,6 +481,10 @@ struct SourceVariableMapping
   SourceVariableMapping() = default;
   SourceVariableMapping(const SourceVariableMapping &) = default;
   SourceVariableMapping &operator=(const SourceVariableMapping &) = default;
+#if !defined(SWIG)
+  SourceVariableMapping(SourceVariableMapping &&) = default;
+  SourceVariableMapping &operator=(SourceVariableMapping &&) = default;
+#endif
 
   bool operator==(const SourceVariableMapping &o) const
   {
@@ -606,6 +617,40 @@ treated as covering the code.
 };
 DECLARE_REFLECTION_STRUCT(LineColumnInfo);
 
+DOCUMENT(R"(Gives per-instruction source code mapping information, including what line(s) correspond
+to this instruction and which source variables exist
+)");
+struct InstructionSourceInfo
+{
+  DOCUMENT("");
+  InstructionSourceInfo() = default;
+  InstructionSourceInfo(const InstructionSourceInfo &) = default;
+  InstructionSourceInfo &operator=(const InstructionSourceInfo &) = default;
+
+  bool operator==(const InstructionSourceInfo &o) const { return instruction == o.instruction; }
+  bool operator<(const InstructionSourceInfo &o) const { return instruction < o.instruction; }
+  DOCUMENT("The instruction that this information is for.");
+  uint32_t instruction;
+
+  DOCUMENT(R"(The source location that this instruction corresponds to
+
+:type: LineColumnInfo
+)");
+  LineColumnInfo lineInfo;
+
+  DOCUMENT(R"(An optional mapping of which high-level source variables map to which debug variables
+and including extra type information.
+
+This list contains source variable mapping that is only valid at this instruction, and is fully
+complete & redundant including all previous source variables that are still valid at this
+instruction.
+
+:type: List[SourceVariableMapping]
+)");
+  rdcarray<SourceVariableMapping> sourceVars;
+};
+DECLARE_REFLECTION_STRUCT(InstructionSourceInfo);
+
 DOCUMENT("This stores the before and after state of a :class:`ShaderVariable`.");
 struct ShaderVariableChange
 {
@@ -613,6 +658,10 @@ struct ShaderVariableChange
   ShaderVariableChange() = default;
   ShaderVariableChange(const ShaderVariableChange &) = default;
   ShaderVariableChange &operator=(const ShaderVariableChange &) = default;
+#if !defined(SWIG)
+  ShaderVariableChange(ShaderVariableChange &&) = default;
+  ShaderVariableChange &operator=(ShaderVariableChange &&) = default;
+#endif
 
   bool operator==(const ShaderVariableChange &o) const
   {
@@ -652,11 +701,15 @@ struct ShaderDebugState
   ShaderDebugState() = default;
   ShaderDebugState(const ShaderDebugState &) = default;
   ShaderDebugState &operator=(const ShaderDebugState &) = default;
+#if !defined(SWIG)
+  ShaderDebugState(ShaderDebugState &&) = default;
+  ShaderDebugState &operator=(ShaderDebugState &&) = default;
+#endif
 
   bool operator==(const ShaderDebugState &o) const
   {
     return nextInstruction == o.nextInstruction && flags == o.flags && changes == o.changes &&
-           sourceVars == o.sourceVars && stepIndex == o.stepIndex;
+           stepIndex == o.stepIndex;
   }
   bool operator<(const ShaderDebugState &o) const
   {
@@ -668,8 +721,6 @@ struct ShaderDebugState
       return stepIndex < o.stepIndex;
     if(!(changes == o.changes))
       return changes < o.changes;
-    if(!(sourceVars == o.sourceVars))
-      return sourceVars < o.sourceVars;
     return false;
   }
 
@@ -694,18 +745,7 @@ backwards using the information.
 )");
   rdcarray<ShaderVariableChange> changes;
 
-  DOCUMENT(R"(An optional mapping of which high-level source variables map to which debug variables
-and including extra type information.
-
-This list contains source variable mapping that is only valid at this state - it is not valid at any
-other state where the lifetime of the source variable may have run out, or it may now be stored in
-a different debug variable.
-
-:type: List[SourceVariableMapping]
-)");
-  rdcarray<SourceVariableMapping> sourceVars;
-
-  DOCUMENT(R"(The function names in the current callstack at this line.
+  DOCUMENT(R"(The function names in the current callstack at this instruction.
 
 The oldest/outer function is first in the list, the newest/inner function is last.
 
@@ -812,12 +852,21 @@ If this is ``None`` then the trace is invalid.
 )");
   ShaderDebugger *debugger = NULL;
 
-  DOCUMENT(R"(An array of the same size as the number of instructions in the shader, with a mapping
-to source lines.
+  DOCUMENT(R"(An array of the same size as the number of instructions in the shader, with
+per-instruction information such as source line mapping, and source variables.
 
-:type: List[LineColumnInfo]
+.. warning::
+
+  This array is *not* indexed by instruction. Since it is common for adjacent instructions to have
+  effectively identical source information, this array only stores unique information ordered by
+  instruction. On some internal representations this may be one entry per instruction, and on others
+  it may be sparse and require a binary lookup to locate the corresponding information for an
+  instruction. If no direct match is found, the lower bound match is valid (i.e. the data for
+  instruction A before the data for instruction B is valid for all instructions in range ``[A, B)``.
+
+:type: List[InstructionSourceInfo]
 )");
-  rdcarray<LineColumnInfo> lineInfo;
+  rdcarray<InstructionSourceInfo> instInfo;
 };
 
 DECLARE_REFLECTION_STRUCT(ShaderDebugTrace);
@@ -1090,7 +1139,7 @@ struct ConstantBlock
   {
     return name == o.name && variables == o.variables && bindPoint == o.bindPoint &&
            byteSize == o.byteSize && bufferBacked == o.bufferBacked &&
-           compileConstants == o.compileConstants;
+           inlineDataBytes == o.inlineDataBytes && compileConstants == o.compileConstants;
   }
   bool operator<(const ConstantBlock &o) const
   {
@@ -1104,6 +1153,8 @@ struct ConstantBlock
       return byteSize < o.byteSize;
     if(!(bufferBacked == o.bufferBacked))
       return bufferBacked < o.bufferBacked;
+    if(!(inlineDataBytes == o.inlineDataBytes))
+      return inlineDataBytes < o.inlineDataBytes;
     if(!(compileConstants == o.compileConstants))
       return compileConstants < o.compileConstants;
     return false;
@@ -1126,6 +1177,8 @@ some other API-specific method, such as direct function calls or they may be com
 specialisation constants.
 )");
   bool bufferBacked = true;
+  DOCUMENT("``True`` if this is backed by in-line data bytes rather than a specific buffer.");
+  bool inlineDataBytes = false;
   DOCUMENT("``True`` if this is a virtual buffer listing compile-time specialisation constants.");
   bool compileConstants = false;
 };
@@ -1382,9 +1435,36 @@ The first entry in the list is always the file where the entry point is.
 )");
   rdcarray<ShaderSourceFile> files;
 
-  DOCUMENT("The :class:`ShaderEncoding` of the source. See :data:`files`.");
+  DOCUMENT(R"(The source location of the first executable line or the entry point.
+
+.. note::
+
+  The information is not guaranteed to be available depending on the underlying shader format, so
+  all of the elements are optional.
+
+:type: LineColumnInfo
+)");
+  LineColumnInfo entryLocation;
+
+  DOCUMENT(R"(The index of the file which should be used for re-editing this shader's entry point.
+
+This is an optional value, and if set to ``-1`` you should fall back to using the file specified
+in :data:`entryLocation`, and if no file is specified there then use the first file listed.
+)");
+  int32_t editBaseFile = -1;
+
+  DOCUMENT(R"(The :class:`ShaderEncoding` of the source. See :data:`files`.
+
+:type: ShaderEncoding
+)");
   ShaderEncoding encoding = ShaderEncoding::Unknown;
 
+  DOCUMENT(R"(The :class:`KnownShaderTool` of the compiling tool.
+
+:type: KnownShaderTool
+)");
+  KnownShaderTool compiler = KnownShaderTool::Unknown;
+
   DOCUMENT(R"(Indicates whether this particular shader can be debugged. In some cases even if the
 API can debug shaders in general, specific shaders cannot be debugged because they use unsupported
 functionality
