@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2021 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -52,7 +52,7 @@ rdcstr ToStr(const T &el)
 #define BEGIN_ENUM_STRINGISE(type)                               \
   using enumType = type;                                         \
   static const char unknown_prefix[] = #type "(";                \
-  static rdcliteral empty_ret = STRING_LITERAL(#type "(0)");     \
+  constexpr rdcliteral empty_ret = STRING_LITERAL(#type "(0)");  \
   static_assert(std::is_same<const type &, decltype(el)>::value, \
                 "Type in macro doesn't match el");               \
   (void)(enumType) el;                                           \
@@ -85,7 +85,7 @@ rdcstr ToStr(const T &el)
 #define BEGIN_BITFIELD_STRINGISE(type)                           \
   using enumType = type;                                         \
   static const char unknown_prefix[] = " | " #type "(";          \
-  static rdcliteral empty_ret = STRING_LITERAL(#type "(0)");     \
+  constexpr rdcliteral empty_ret = STRING_LITERAL(#type "(0)");  \
   static_assert(std::is_same<const type &, decltype(el)>::value, \
                 "Type in macro doesn't match el");               \
   uint64_t local = (uint64_t)el;                                 \
