@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -145,6 +145,25 @@ DOCUMENT(R"(Bitfield flags that could be applied to a type.
 
   Special flag to indicate that this is structure is stored as a union, meaning all children share
   the same memory and some external flag indicates which element is valid.
+
+.. data:: Important
+
+  Indicates that this object is important or significant, to aid in generating a summary/one-line
+  view of a particular chunk by only including important children.
+
+  This property can be recursive - so an important child which is a structure can have only some
+  members which are important.
+
+.. data:: ImportantChildren
+
+  Indicates that only important children should be processed, as noted in :data:`Important`. This
+  may appear on an object which has no important children - which indicates explicitly that there
+  are no important children so when summarising no parameters should be shown.
+
+.. data:: HiddenChildren
+
+  Indicates that some children are marked as hidden. This can be important for cases where the
+  number of children is important.
 )");
 enum class SDTypeFlags : uint32_t
 {
@@ -155,6 +174,9 @@ enum class SDTypeFlags : uint32_t
   NullString = 0x8,
   FixedArray = 0x10,
   Union = 0x20,
+  Important = 0x40,
+  ImportantChildren = 0x80,
+  HiddenChildren = 0x100,
 };
 
 BITMASK_OPERATORS(SDTypeFlags);
@@ -314,7 +336,7 @@ private:
 
 DECLARE_REFLECTION_STRUCT(SDObjectPODData);
 
-DOCUMENT("A ``list`` of :class:`SDObject` objects");
+DOCUMENT("INTERNAL: An array of SDObject*, mapped to a pure list in python");
 struct StructuredObjectList : public rdcarray<SDObject *>
 {
   StructuredObjectList() : rdcarray<SDObject *>() {}
@@ -375,6 +397,10 @@ private:
   friend void DoSerialise(SerialiserType &ser, SDObjectData &el);
   template <class SerialiserType>
   friend void DoSerialise(SerialiserType &ser, SDObject *el);
+  template <class SerialiserType>
+  friend void DoSerialise(SerialiserType &ser, SDObject &el);
+  template <class SerialiserType>
+  friend void DoSerialise(SerialiserType &ser, SDChunk &el);
 
   DOCUMENT("A list of :class:`SDObject` containing the children of this :class:`SDObject`.");
   mutable StructuredObjectList children;
@@ -500,7 +526,10 @@ struct SDObject
     m_Parent = NULL;
   }
 
-  DOCUMENT("Create a deep copy of this object.");
+  DOCUMENT(R"(
+:return: A new deep copy of this object, which the caller owns.
+:rtype: SDObject
+)");
   SDObject *Duplicate() const
   {
     SDObject *ret = new SDObject();
@@ -535,30 +564,30 @@ recursively through children.
 
 :param SDObject obj: The object to compare against
 :return: A boolean indicating if the object is equal to this one.
-:rtype: ``bool``
+:rtype: bool
 )");
-  bool HasEqualValue(const SDObject *o) const
+  bool HasEqualValue(const SDObject *obj) const
   {
     bool ret = true;
 
-    if(data.str != o->data.str)
+    if(data.str != obj->data.str)
     {
       ret = false;
     }
-    else if(data.basic.u != o->data.basic.u)
+    else if(data.basic.u != obj->data.basic.u)
     {
       ret = false;
     }
-    else if(data.children.size() != o->data.children.size())
+    else if(data.children.size() != obj->data.children.size())
     {
       ret = false;
     }
     else
     {
-      for(size_t c = 0; c < o->data.children.size(); c++)
+      for(size_t c = 0; c < obj->data.children.size(); c++)
       {
         PopulateChild(c);
-        ret &= data.children[c]->HasEqualValue(o->GetChild(c));
+        ret &= data.children[c]->HasEqualValue(obj->GetChild(c));
       }
     }
 
@@ -569,7 +598,7 @@ recursively through children.
   // python.
   DOCUMENT(R"(Add a new child object.
 
-:param SDObject obj: The new child to add
+:param SDObject child: The new child to add
 )");
   inline void DuplicateAndAddChild(const SDObject *child)
   {
@@ -583,7 +612,7 @@ recursively through children.
   DOCUMENT(R"(Find a child object by a given name. If no matching child is found, ``None`` is
 returned.
 
-:param str name: The name to search for.
+:param str childName: The name to search for.
 :return: A reference to the child object if found, or ``None`` if not.
 :rtype: SDObject
 )");
@@ -595,6 +624,31 @@ returned.
     return NULL;
   }
 
+  DOCUMENT(R"(Find a child object by a given name recursively. If no matching child is found,
+``None`` is returned.
+
+The order of the search is not guaranteed, so care should be taken when the name may not be unique.
+
+:param str childName: The name to search for.
+:return: A reference to the child object if found, or ``None`` if not.
+:rtype: SDObject
+)");
+  inline SDObject *FindChildRecursively(const rdcstr &childName)
+  {
+    SDObject *o = FindChild(childName);
+    if(o)
+      return o;
+
+    for(size_t i = 0; i < NumChildren(); i++)
+    {
+      o = GetChild(i)->FindChildRecursively(childName);
+      if(o)
+        return o;
+    }
+
+    return NULL;
+  }
+
   DOCUMENT(R"(Find a child object by a given index. If the index is out of bounds, ``None`` is
 returned.
 
@@ -629,6 +683,21 @@ returned.
         return GetChild(i);
     return NULL;
   }
+  inline const SDObject *FindChildRecursively(const rdcstr &childName) const
+  {
+    const SDObject *o = FindChild(childName);
+    if(o)
+      return o;
+
+    for(size_t i = 0; i < NumChildren(); i++)
+    {
+      o = GetChild(i)->FindChildRecursively(childName);
+      if(o)
+        return o;
+    }
+
+    return NULL;
+  }
   inline const SDObject *GetChild(size_t index) const
   {
     if(index < data.children.size())
@@ -670,7 +739,7 @@ returned.
   DOCUMENT(R"(Get the number of child objects.
 
 :return: The number of children this object contains.
-:rtype: ``int``
+:rtype: int
 )");
   inline size_t NumChildren() const { return data.children.size(); }
 #if !defined(SWIG)
@@ -805,14 +874,12 @@ returned.
       return false;
     return true;
   }
-  const char *Type() const { return type.name.c_str(); }
-  const char *Name() const { return name.c_str(); }
-  SDObject *SetTypeName(const char *customTypeName)
+  SDObject *SetTypeName(const rdcstr &customTypeName)
   {
     type.name = customTypeName;
     return this;
   }
-  SDObject *SetCustomString(const char *customString)
+  SDObject *SetCustomString(const rdcstr &customString)
   {
     data.str = customString;
     type.flags = SDTypeFlags::HasCustomString;
@@ -823,11 +890,17 @@ returned.
   // these are common to both python and C++
   DOCUMENT(R"(Interprets the object as a ``bool`` and returns its value.
 Invalid if the object is not actually a ``bool``.
+
+:return: The interpreted bool value.
+:rtype: bool
 )");
   inline bool AsBool() const { return data.basic.b; }
   // these are common to both python and C++
   DOCUMENT(R"(Interprets the object as a :class:`ResourceId` and returns its value.
 Invalid if the object is not actually a :class:`ResourceId`.
+
+:return: The interpreted ID.
+:rtype: ResourceId
 )");
   inline ResourceId AsResourceId() const { return data.basic.id; }
 #if defined(RENDERDOC_QT_COMPAT)
@@ -925,6 +998,8 @@ private:
   friend void DoSerialise(SerialiserType &ser, SDObject &el);
   template <class SerialiserType>
   friend void DoSerialise(SerialiserType &ser, SDChunk &el);
+  template <class SerialiserType>
+  friend void DoSerialise(SerialiserType &ser, SDObject &el, StructuredObjectList &children);
 
   void DeleteLazyGenerator() const
   {
@@ -1049,7 +1124,17 @@ inline SDObject *makeSDObject(const rdcinflexiblestr &name, QVariant val)
 }
 #endif
 
-DOCUMENT("Make a structured object out of a signed integer");
+DOCUMENT(R"(Make a structured object as a signed 64-bit integer.
+
+.. note::
+  You should ensure that the value you pass in has already been truncated to the appropriate range
+  for the storage, as the resulting object will be undefined if the value is out of the valid range.
+
+:param str name: The name of the object.
+:param int val: The integer which will be stored in the returned object.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDInt64(const rdcinflexiblestr &name, int64_t val)
 {
   SDObject *ret = new SDObject(name, "int64_t"_lit);
@@ -1059,7 +1144,17 @@ inline SDObject *makeSDInt64(const rdcinflexiblestr &name, int64_t val)
   return ret;
 }
 
-DOCUMENT("Make a structured object out of an unsigned integer");
+DOCUMENT(R"(Make a structured object as an unsigned 64-bit integer.
+
+.. note::
+  You should ensure that the value you pass in has already been truncated to the appropriate range
+  for the storage, as the resulting object will be undefined if the value is out of the valid range.
+
+:param str name: The name of the object.
+:param int val: The integer which will be stored in the returned object.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDUInt64(const rdcinflexiblestr &name, uint64_t val)
 {
   SDObject *ret = new SDObject(name, "uint64_t"_lit);
@@ -1069,7 +1164,17 @@ inline SDObject *makeSDUInt64(const rdcinflexiblestr &name, uint64_t val)
   return ret;
 }
 
-DOCUMENT("Make a structured object out of a integer, stored as signed 32-bits");
+DOCUMENT(R"(Make a structured object as a signed 32-bit integer.
+
+.. note::
+  You should ensure that the value you pass in has already been truncated to the appropriate range
+  for the storage, as the resulting object will be undefined if the value is out of the valid range.
+
+:param str name: The name of the object.
+:param int val: The integer which will be stored in the returned object.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDInt32(const rdcinflexiblestr &name, int32_t val)
 {
   SDObject *ret = new SDObject(name, "int32_t"_lit);
@@ -1079,7 +1184,17 @@ inline SDObject *makeSDInt32(const rdcinflexiblestr &name, int32_t val)
   return ret;
 }
 
-DOCUMENT("Make a structured object out of a integer, stored as unsigned 32-bits");
+DOCUMENT(R"(Make a structured object as an unsigned 32-bit integer.
+
+.. note::
+  You should ensure that the value you pass in has already been truncated to the appropriate range
+  for the storage, as the resulting object will be undefined if the value is out of the valid range.
+
+:param str name: The name of the object.
+:param int val: The integer which will be stored in the returned object.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDUInt32(const rdcinflexiblestr &name, uint32_t val)
 {
   SDObject *ret = new SDObject(name, "uint32_t"_lit);
@@ -1089,7 +1204,17 @@ inline SDObject *makeSDUInt32(const rdcinflexiblestr &name, uint32_t val)
   return ret;
 }
 
-DOCUMENT("Make a structured object out of a floating point value");
+DOCUMENT(R"(Make a structured object as a 32-bit float.
+
+.. note::
+  You should ensure that the value you pass in has already been truncated to the appropriate range
+  for the storage, as the resulting object will be undefined if the value is out of the valid range.
+
+:param str name: The name of the object.
+:param float val: The float which will be stored in the returned object.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDFloat(const rdcinflexiblestr &name, float val)
 {
   SDObject *ret = new SDObject(name, "float"_lit);
@@ -1099,7 +1224,13 @@ inline SDObject *makeSDFloat(const rdcinflexiblestr &name, float val)
   return ret;
 }
 
-DOCUMENT("Make a structured object out of a boolean value");
+DOCUMENT(R"(Make a structured object as a boolean value.
+
+:param str name: The name of the object.
+:param bool val: The bool which will be stored in the returned object.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDBool(const rdcinflexiblestr &name, bool val)
 {
   SDObject *ret = new SDObject(name, "bool"_lit);
@@ -1109,7 +1240,13 @@ inline SDObject *makeSDBool(const rdcinflexiblestr &name, bool val)
   return ret;
 }
 
-DOCUMENT("Make a structured object out of a string");
+DOCUMENT(R"(Make a structured object as a string value.
+
+:param str name: The name of the object.
+:param str val: The string which will be stored in the returned object.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDString(const rdcinflexiblestr &name, const rdcstr &val)
 {
   SDObject *ret = new SDObject(name, "string"_lit);
@@ -1119,7 +1256,13 @@ inline SDObject *makeSDString(const rdcinflexiblestr &name, const rdcstr &val)
   return ret;
 }
 
-DOCUMENT("Make a structured object out of a ResourceId");
+DOCUMENT(R"(Make a structured object as a ResourceId value.
+
+:param str name: The name of the object.
+:param ResourceId val: The ID which will be stored in the returned object.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDResourceId(const rdcinflexiblestr &name, ResourceId val)
 {
   SDObject *ret = new SDObject(name, "ResourceId"_lit);
@@ -1129,7 +1272,17 @@ inline SDObject *makeSDResourceId(const rdcinflexiblestr &name, ResourceId val)
   return ret;
 }
 
-DOCUMENT("Make a structured object out of an enumeration value");
+DOCUMENT(R"(Make a structured object as an enum value.
+
+.. note::
+  The enum will be stored just as an integer value, but the string name of the enumeration value can
+  be set with :meth:`SDObject.SetCustomString` if desired.
+
+:param str name: The name of the object.
+:param int val: The integer value of the enum itself.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDEnum(const rdcinflexiblestr &name, uint32_t val)
 {
   SDObject *ret = new SDObject(name, "enum"_lit);
@@ -1139,7 +1292,14 @@ inline SDObject *makeSDEnum(const rdcinflexiblestr &name, uint32_t val)
   return ret;
 }
 
-DOCUMENT("Make an array-type structured object");
+DOCUMENT(R"(Make a structured object which is an array.
+
+The array will be created empty, and new members can be added using methods on :class:`SDObject`.
+
+:param str name: The name of the object.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDArray(const rdcinflexiblestr &name)
 {
   SDObject *ret = new SDObject(name, "array"_lit);
@@ -1147,7 +1307,15 @@ inline SDObject *makeSDArray(const rdcinflexiblestr &name)
   return ret;
 }
 
-DOCUMENT("Make an struct-type structured object");
+DOCUMENT(R"(Make a structured object which is a struct.
+
+The struct will be created empty, and new members can be added using methods on :class:`SDObject`.
+
+:param str name: The name of the object.
+:param str structtype: The typename of the struct.
+:return: The new object, owner by the caller.
+:rtype: SDObject
+)");
 inline SDObject *makeSDStruct(const rdcinflexiblestr &name, const rdcinflexiblestr &structtype)
 {
   SDObject *ret = new SDObject(name, structtype);
@@ -1165,9 +1333,9 @@ inline SDObject *makeSDStruct(const rdcinflexiblestr &name, const rdcinflexibles
   {                                                                                                 \
     SDObject *ptr = makeSDFunc(name, value);                                                        \
     if(customString)                                                                                \
-      ptr->SetCustomString(customString);                                                           \
+      ptr->SetCustomString(rdcstr(customString));                                                   \
     if(customTypeName)                                                                              \
-      ptr->SetTypeName(customTypeName);                                                             \
+      ptr->SetTypeName(rdcstr(customTypeName));                                                     \
     return ptr;                                                                                     \
   }
 
@@ -1220,7 +1388,10 @@ struct SDChunk : public SDObject
   DOCUMENT("The :class:`SDChunkMetaData` with the metadata for this chunk.");
   SDChunkMetaData metadata;
 
-  DOCUMENT("Create a deep copy of this chunk.");
+  DOCUMENT(R"(
+:return: A new deep copy of this chunk, which the caller owns.
+:rtype: SDChunk
+)");
   SDChunk *Duplicate() const
   {
     SDChunk *ret = new SDChunk();
@@ -1248,7 +1419,7 @@ protected:
 
 DECLARE_REFLECTION_STRUCT(SDChunk);
 
-DOCUMENT("A ``list`` of :class:`SDChunk` objects");
+DOCUMENT("INTERNAL: An array of SDChunk*, mapped to a pure list in python");
 struct StructuredChunkList : public rdcarray<SDChunk *>
 {
   StructuredChunkList() : rdcarray<SDChunk *>() {}
@@ -1285,7 +1456,7 @@ DECLARE_REFLECTION_STRUCT(StructuredChunkList);
 
 DECLARE_REFLECTION_STRUCT(bytebuf);
 
-DOCUMENT("A ``list`` of ``bytes`` objects");
+DOCUMENT("INTERNAL: An array of bytebuf*, mapped to a pure list of bytes in python");
 struct StructuredBufferList : public rdcarray<bytebuf *>
 {
   StructuredBufferList() : rdcarray<bytebuf *>() {}
@@ -1364,15 +1535,25 @@ public:
       delete buf;
   }
 
-  DOCUMENT("A ``list`` of :class:`SDChunk` objects with the chunks in order.");
+  DOCUMENT(R"(The chunks in the file in order.
+
+:type: List[SDChunk]
+)");
   StructuredChunkList chunks;
 
-  DOCUMENT("A ``list`` of serialised buffers stored as ``bytes`` objects");
+  DOCUMENT(R"(The buffers in the file, as referenced by the chunks in :data:`chunks`.
+
+:type: List[bytes]
+)");
   StructuredBufferList buffers;
 
   DOCUMENT("The version of this structured stream, typically only used internally.");
   uint64_t version = 0;
 
+  DOCUMENT(R"(Swaps the contents of this file with another.
+
+:param SDFile other: The other file to swap with.
+)");
   inline void Swap(SDFile &other)
   {
     chunks.swap(other.chunks);
