@@ -38,19 +38,21 @@ struct BindingElement
 
   bool operator==(const BindingElement &o) const
   {
-    return dynamicallyUsed == o.dynamicallyUsed && viewResourceId == o.viewResourceId &&
-           resourceResourceId == o.resourceResourceId && samplerResourceId == o.samplerResourceId &&
-           immutableSampler == o.immutableSampler && inlineBlock == o.inlineBlock &&
-           viewFormat == o.viewFormat && swizzle == o.swizzle && firstMip == o.firstMip &&
-           firstSlice == o.firstSlice && numMips == o.numMips && numSlices == o.numSlices &&
-           byteOffset == o.byteOffset && byteSize == o.byteSize && filter == o.filter &&
-           addressU == o.addressU && addressV == o.addressV && addressW == o.addressW &&
-           mipBias == o.mipBias && maxAnisotropy == o.maxAnisotropy &&
+    return type == o.type && dynamicallyUsed == o.dynamicallyUsed &&
+           viewResourceId == o.viewResourceId && resourceResourceId == o.resourceResourceId &&
+           samplerResourceId == o.samplerResourceId && immutableSampler == o.immutableSampler &&
+           inlineBlock == o.inlineBlock && viewFormat == o.viewFormat && swizzle == o.swizzle &&
+           firstMip == o.firstMip && firstSlice == o.firstSlice && numMips == o.numMips &&
+           numSlices == o.numSlices && byteOffset == o.byteOffset && byteSize == o.byteSize &&
+           filter == o.filter && addressU == o.addressU && addressV == o.addressV &&
+           addressW == o.addressW && mipBias == o.mipBias && maxAnisotropy == o.maxAnisotropy &&
            compareFunction == o.compareFunction && minLOD == o.minLOD && maxLOD == o.maxLOD &&
            borderColor == o.borderColor && unnormalized == o.unnormalized;
   }
   bool operator<(const BindingElement &o) const
   {
+    if(!(type == o.type))
+      return type < o.type;
     if(!(dynamicallyUsed == o.dynamicallyUsed))
       return dynamicallyUsed < o.dynamicallyUsed;
     if(!(viewResourceId == o.viewResourceId))
@@ -104,6 +106,8 @@ struct BindingElement
     return false;
   }
 
+  DOCUMENT("The :class:`BindType` of this binding element.");
+  BindType type = BindType::Unknown;
   DOCUMENT("The :class:`ResourceId` of the current view object, if one is in use.");
   ResourceId viewResourceId;    // bufferview, imageview, attachmentview
   DOCUMENT("The :class:`ResourceId` of the current underlying buffer or image object.");
@@ -232,7 +236,7 @@ struct DescriptorBinding
   {
     return descriptorCount == o.descriptorCount && dynamicallyUsedCount == o.dynamicallyUsedCount &&
            firstUsedIndex == o.firstUsedIndex && lastUsedIndex == o.lastUsedIndex &&
-           type == o.type && stageFlags == o.stageFlags && binds == o.binds;
+           stageFlags == o.stageFlags && binds == o.binds;
   }
   bool operator<(const DescriptorBinding &o) const
   {
@@ -244,8 +248,6 @@ struct DescriptorBinding
       return firstUsedIndex < o.firstUsedIndex;
     if(!(lastUsedIndex == o.lastUsedIndex))
       return lastUsedIndex < o.lastUsedIndex;
-    if(!(type == o.type))
-      return type < o.type;
     if(!(stageFlags == o.stageFlags))
       return stageFlags < o.stageFlags;
     if(!(binds == o.binds))
@@ -278,8 +280,6 @@ to avoid redundant iteration in very large descriptor arrays with a small subset
 For more information see :data:`VKBindingElement.dynamicallyUsed`.
 )");
   int32_t lastUsedIndex = 0x7fffffff;
-  DOCUMENT("The :class:`BindType` of this binding.");
-  BindType type = BindType::Unknown;
   DOCUMENT("The :class:`ShaderStageMask` where this binding is visible.");
   ShaderStageMask stageFlags = ShaderStageMask::Unknown;
 
@@ -685,6 +685,12 @@ struct TransformFeedback
 :type: List[VKXFBBuffer]
 )");
   rdcarray<XFBBuffer> buffers;
+
+  DOCUMENT(R"(Which stream-out stream is being used for rasterization.
+
+:type: int
+)");
+  uint32_t rasterizedStream = 0;
 };
 
 DOCUMENT("Describes a render area in the current framebuffer.");
@@ -781,6 +787,9 @@ and a fragment in none of them is discarded.
   disabled, since with no rectangles no fragment can be inside one.
 )");
   bool discardRectanglesExclusive = true;
+
+  DOCUMENT(R"(Whether depth clip range is set to [-1, 1] through VK_EXT_depth_clip_control.)");
+  bool depthNegativeOneToOne = false;
 };
 
 DOCUMENT("Describes the rasterizer state in the pipeline.");
@@ -992,6 +1001,9 @@ struct RenderPass
   DOCUMENT("Whether or not dynamic rendering is currently suspended.");
   bool suspended = false;
 
+  DOCUMENT("Whether or not there is a potential feedback loop.");
+  bool feedbackLoop = false;
+
   DOCUMENT("The index of the current active subpass.");
   uint32_t subpass;
 
@@ -1076,6 +1088,13 @@ If the list is empty, fdm_offset is disabled and rendering is as normal.
 :type: List[Offset]
 )");
   rdcarray<Offset> fragmentDensityOffsets;
+
+  DOCUMENT(R"(If VK_EXT_multisampled_render_to_single_sampled is enabled, contains the number of
+samples used to render this subpass.
+
+If the subpass is not internally multisampled, tileOnlyMSAASampleCount is set to 0.
+)");
+  uint32_t tileOnlyMSAASampleCount = 0;
 };
 
 DOCUMENT("Describes a single attachment in a framebuffer object.");
