@@ -198,7 +198,7 @@ D3D12DebugManager::D3D12DebugManager(WrappedID3D12Device *wrapper)
   sampDesc.MinLOD = 0;
   sampDesc.MaxLOD = FLT_MAX;
   sampDesc.MipLODBias = 0.0f;
-  sampDesc.ComparisonFunc = D3D12_COMPARISON_FUNC_ALWAYS;
+  sampDesc.ComparisonFunc = D3D12_COMPARISON_FUNC_NEVER;
 
   m_pDevice->CreateSampler(&sampDesc, samp);
 
@@ -259,12 +259,12 @@ D3D12DebugManager::D3D12DebugManager(WrappedID3D12Device *wrapper)
     rm->SetInternalResource(m_MeshRootSig);
   }
 
-  if(!CreateMathIntrinsicsResources())
+  if(!CreateShaderDebugResources())
   {
     RDCERR("Failed to create resources for shader debugging math intrinsics");
-    SAFE_RELEASE(m_MathIntrinsicsRootSig);
+    SAFE_RELEASE(m_ShaderDebugRootSig);
     SAFE_RELEASE(m_MathIntrinsicsPso);
-    SAFE_RELEASE(m_MathIntrinsicsResultBuffer);
+    SAFE_RELEASE(m_ShaderDebugResultBuffer);
   }
 
   {
@@ -433,9 +433,12 @@ D3D12DebugManager::~D3D12DebugManager()
   SAFE_RELEASE(m_MeshPS);
   SAFE_RELEASE(m_MeshRootSig);
 
-  SAFE_RELEASE(m_MathIntrinsicsRootSig);
+  SAFE_RELEASE(m_ShaderDebugRootSig);
   SAFE_RELEASE(m_MathIntrinsicsPso);
-  SAFE_RELEASE(m_MathIntrinsicsResultBuffer);
+  SAFE_RELEASE(m_ShaderDebugResultBuffer);
+  SAFE_RELEASE(m_TexSamplePso);
+  for(auto it = m_OffsetTexSamplePso.begin(); it != m_OffsetTexSamplePso.end(); ++it)
+    it->second->Release();
 
   SAFE_RELEASE(m_ArrayMSAARootSig);
   SAFE_RELEASE(m_FullscreenVS);
@@ -477,67 +480,59 @@ D3D12DebugManager::~D3D12DebugManager()
   RenderDoc::Inst().UnregisterMemoryRegion(this);
 }
 
-bool D3D12DebugManager::CreateMathIntrinsicsResources()
+bool D3D12DebugManager::CreateShaderDebugResources()
 {
-  rdcstr csProgram =
-      "RWStructuredBuffer<float4> outval : register(u0);\n"
-      "cbuffer srcOper : register(b0) { float4 inval; };\n"
-      "cbuffer srcInstr : register(b1) { uint operation; };\n";
+  rdcstr hlsl = GetEmbeddedResource(shaderdebug_hlsl);
 
-  // Assign constants to each supported instruction
-  csProgram += StringFormat::Fmt("static const uint OPCODE_RCP = %u;\n", DXBCBytecode::OPCODE_RCP);
-  csProgram += StringFormat::Fmt("static const uint OPCODE_RSQ = %u;\n", DXBCBytecode::OPCODE_RSQ);
-  csProgram += StringFormat::Fmt("static const uint OPCODE_EXP = %u;\n", DXBCBytecode::OPCODE_EXP);
-  csProgram += StringFormat::Fmt("static const uint OPCODE_LOG = %u;\n", DXBCBytecode::OPCODE_LOG);
-  csProgram +=
-      StringFormat::Fmt("static const uint OPCODE_SINCOS = %u;\n", DXBCBytecode::OPCODE_SINCOS);
-
-  csProgram +=
-      "[numthreads(1, 1, 1)]\n"
-      "void main(){\n"
-      "  switch(operation){\n"
-      "    case OPCODE_RCP: outval[0] = rcp(inval); break;\n"
-      "    case OPCODE_RSQ: outval[0] = rsqrt(inval); break;\n"
-      "    case OPCODE_EXP: outval[0] = exp2(inval); break;\n"
-      "    case OPCODE_LOG: outval[0] = log2(inval); break;\n"
-      "    case OPCODE_SINCOS: sincos(inval, outval[0], outval[1]); break;\n"
-      "  }\n}\n";
+  D3D12RootSignature rootSig;
 
   ID3DBlob *csBlob = NULL;
-  UINT flags = D3DCOMPILE_DEBUG | D3DCOMPILE_WARNINGS_ARE_ERRORS;
-  if(m_pDevice->GetShaderCache()->GetShaderBlob(csProgram.c_str(), "main", flags, {}, "cs_5_0",
+  if(m_pDevice->GetShaderCache()->GetShaderBlob(hlsl.c_str(), "RENDERDOC_DebugMathOp",
+                                                D3DCOMPILE_WARNINGS_ARE_ERRORS, {}, "cs_5_0",
                                                 &csBlob) != "")
   {
     RDCERR("Failed to create shader to calculate math intrinsic");
     return false;
   }
 
-  D3D12RootSignature rootSig;
-
-  // Constants param for the operation inputs
   D3D12RootSignatureParameter constantParam;
-  constantParam.ParameterType = D3D12_ROOT_PARAMETER_TYPE_32BIT_CONSTANTS;
+  constantParam.ParameterType = D3D12_ROOT_PARAMETER_TYPE_CBV;
   constantParam.ShaderVisibility = D3D12_SHADER_VISIBILITY_ALL;
-  constantParam.Constants.Num32BitValues = 4;    // Input is 4 floats
-  constantParam.Constants.ShaderRegister = 0;
-  constantParam.Constants.RegisterSpace = 0;
+  constantParam.Descriptor.Flags = D3D12_ROOT_DESCRIPTOR_FLAG_NONE;
+  constantParam.Descriptor.ShaderRegister = 0;
+  constantParam.Descriptor.RegisterSpace = 0;
   rootSig.Parameters.push_back(constantParam);
 
-  // Constants param for the opcode
-  constantParam.Constants.Num32BitValues = 1;
-  constantParam.Constants.ShaderRegister = 1;
-  constantParam.Constants.RegisterSpace = 0;
-  rootSig.Parameters.push_back(constantParam);
-
-  // UAV param for the output
   D3D12RootSignatureParameter uavParam;
   uavParam.ParameterType = D3D12_ROOT_PARAMETER_TYPE_UAV;
   uavParam.ShaderVisibility = D3D12_SHADER_VISIBILITY_ALL;
-  uavParam.Descriptor.ShaderRegister = 0;
+  uavParam.Descriptor.ShaderRegister = 1;
   uavParam.Descriptor.RegisterSpace = 0;
   uavParam.Descriptor.Flags = D3D12_ROOT_DESCRIPTOR_FLAG_NONE;
   rootSig.Parameters.push_back(uavParam);
 
+  D3D12_DESCRIPTOR_RANGE1 range = {};
+  range.BaseShaderRegister = 0;
+  range.RegisterSpace = 0;
+  range.NumDescriptors = 25;
+  range.OffsetInDescriptorsFromTableStart = 0;
+  range.RangeType = D3D12_DESCRIPTOR_RANGE_TYPE_SRV;
+
+  D3D12RootSignatureParameter srvParam;
+  srvParam.ParameterType = D3D12_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE;
+  srvParam.ShaderVisibility = D3D12_SHADER_VISIBILITY_ALL;
+  srvParam.ranges.push_back(range);
+  rootSig.Parameters.push_back(srvParam);
+
+  range.NumDescriptors = 2;
+  range.RangeType = D3D12_DESCRIPTOR_RANGE_TYPE_SAMPLER;
+
+  D3D12RootSignatureParameter sampParam;
+  sampParam.ParameterType = D3D12_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE;
+  sampParam.ShaderVisibility = D3D12_SHADER_VISIBILITY_ALL;
+  sampParam.ranges.push_back(range);
+  rootSig.Parameters.push_back(sampParam);
+
   ID3DBlob *root = m_pDevice->GetShaderCache()->MakeRootSig(rootSig);
   if(root == NULL)
   {
@@ -545,9 +540,10 @@ bool D3D12DebugManager::CreateMathIntrinsicsResources()
     SAFE_RELEASE(csBlob);
     return false;
   }
-  HRESULT hr = m_pDevice->CreateRootSignature(0, root->GetBufferPointer(), root->GetBufferSize(),
-                                              __uuidof(ID3D12RootSignature),
-                                              (void **)&m_MathIntrinsicsRootSig);
+
+  HRESULT hr =
+      m_pDevice->CreateRootSignature(0, root->GetBufferPointer(), root->GetBufferSize(),
+                                     __uuidof(ID3D12RootSignature), (void **)&m_ShaderDebugRootSig);
   m_pDevice->InternalRef();
   SAFE_RELEASE(root);
   if(FAILED(hr))
@@ -556,10 +552,10 @@ bool D3D12DebugManager::CreateMathIntrinsicsResources()
     SAFE_RELEASE(csBlob);
     return false;
   }
-  m_pDevice->GetResourceManager()->SetInternalResource(m_MathIntrinsicsRootSig);
+  m_pDevice->GetResourceManager()->SetInternalResource(m_ShaderDebugRootSig);
 
   D3D12_COMPUTE_PIPELINE_STATE_DESC psoDesc;
-  psoDesc.pRootSignature = m_MathIntrinsicsRootSig;
+  psoDesc.pRootSignature = m_ShaderDebugRootSig;
   psoDesc.CS.BytecodeLength = csBlob->GetBufferSize();
   psoDesc.CS.pShaderBytecode = csBlob->GetBufferPointer();
   psoDesc.NodeMask = 0;
@@ -574,7 +570,6 @@ bool D3D12DebugManager::CreateMathIntrinsicsResources()
   if(FAILED(hr))
   {
     RDCERR("Failed to create PSO for shader debugging HRESULT: %s", ToStr(hr).c_str());
-    SAFE_RELEASE(m_MathIntrinsicsRootSig);
     return false;
   }
   m_pDevice->GetResourceManager()->SetInternalResource(m_MathIntrinsicsPso);
@@ -583,7 +578,7 @@ bool D3D12DebugManager::CreateMathIntrinsicsResources()
   D3D12_RESOURCE_DESC rdesc;
   ZeroMemory(&rdesc, sizeof(D3D12_RESOURCE_DESC));
   rdesc.Dimension = D3D12_RESOURCE_DIMENSION_BUFFER;
-  rdesc.Width = sizeof(Vec4f) * 2;    // Output buffer is 2x float4
+  rdesc.Width = sizeof(Vec4f) * 6;
   rdesc.Height = 1;
   rdesc.DepthOrArraySize = 1;
   rdesc.MipLevels = 1;
@@ -601,21 +596,173 @@ bool D3D12DebugManager::CreateMathIntrinsicsResources()
   heapProps.VisibleNodeMask = 1;
 
   hr = m_pDevice->CreateCommittedResource(
-      &heapProps, D3D12_HEAP_FLAG_NONE, &rdesc, D3D12_RESOURCE_STATE_UNORDERED_ACCESS, NULL,
-      __uuidof(ID3D12Resource), (void **)&m_MathIntrinsicsResultBuffer);
+      &heapProps, D3D12_HEAP_FLAG_NONE, &rdesc, D3D12_RESOURCE_STATE_COMMON, NULL,
+      __uuidof(ID3D12Resource), (void **)&m_ShaderDebugResultBuffer);
   m_pDevice->InternalRef();
   if(FAILED(hr))
   {
     RDCERR("Failed to create buffer for pixel shader debugging HRESULT: %s", ToStr(hr).c_str());
-    SAFE_RELEASE(m_MathIntrinsicsRootSig);
-    SAFE_RELEASE(m_MathIntrinsicsPso);
     return false;
   }
-  m_pDevice->GetResourceManager()->SetInternalResource(m_MathIntrinsicsResultBuffer);
+  m_pDevice->GetResourceManager()->SetInternalResource(m_ShaderDebugResultBuffer);
+
+  // Create UAV to store the computed results
+  D3D12_UNORDERED_ACCESS_VIEW_DESC uavDesc;
+  ZeroMemory(&uavDesc, sizeof(D3D12_UNORDERED_ACCESS_VIEW_DESC));
+  uavDesc.Format = DXGI_FORMAT_UNKNOWN;
+  uavDesc.ViewDimension = D3D12_UAV_DIMENSION_BUFFER;
+  uavDesc.Buffer.NumElements = 1;
+  uavDesc.Buffer.StructureByteStride = sizeof(Vec4f) * 6;
+
+  D3D12_CPU_DESCRIPTOR_HANDLE uav = GetCPUHandle(SHADER_DEBUG_UAV);
+  m_pDevice->CreateUnorderedAccessView(m_ShaderDebugResultBuffer, NULL, &uavDesc, uav);
+
+  ID3DBlob *vsBlob = NULL;
+  if(m_pDevice->GetShaderCache()->GetShaderBlob(hlsl.c_str(), "RENDERDOC_DebugSampleVS",
+                                                D3DCOMPILE_WARNINGS_ARE_ERRORS, {}, "vs_5_0",
+                                                &vsBlob) != "")
+  {
+    RDCERR("Failed to create shader to do texture sample");
+    SAFE_RELEASE(vsBlob);
+    return false;
+  }
+
+  ID3DBlob *psBlob = NULL;
+  if(m_pDevice->GetShaderCache()->GetShaderBlob(hlsl.c_str(), "RENDERDOC_DebugSamplePS",
+                                                D3DCOMPILE_WARNINGS_ARE_ERRORS, {}, "ps_5_0",
+                                                &psBlob) != "")
+  {
+    RDCERR("Failed to create shader to do texture sample");
+    SAFE_RELEASE(vsBlob);
+    SAFE_RELEASE(psBlob);
+    return false;
+  }
+
+  D3D12_GRAPHICS_PIPELINE_STATE_DESC pipeDesc = {};
+
+  pipeDesc.pRootSignature = m_ShaderDebugRootSig;
+  pipeDesc.VS.BytecodeLength = vsBlob->GetBufferSize();
+  pipeDesc.VS.pShaderBytecode = vsBlob->GetBufferPointer();
+  pipeDesc.PS.BytecodeLength = psBlob->GetBufferSize();
+  pipeDesc.PS.pShaderBytecode = psBlob->GetBufferPointer();
+  pipeDesc.RasterizerState.FillMode = D3D12_FILL_MODE_SOLID;
+  pipeDesc.RasterizerState.CullMode = D3D12_CULL_MODE_NONE;
+  pipeDesc.SampleMask = 0xFFFFFFFF;
+  pipeDesc.SampleDesc.Count = 1;
+  pipeDesc.IBStripCutValue = D3D12_INDEX_BUFFER_STRIP_CUT_VALUE_DISABLED;
+  pipeDesc.PrimitiveTopologyType = D3D12_PRIMITIVE_TOPOLOGY_TYPE_TRIANGLE;
+  pipeDesc.NumRenderTargets = 1;
+  pipeDesc.RTVFormats[0] = DXGI_FORMAT_R32G32B32A32_FLOAT;
+  pipeDesc.DSVFormat = DXGI_FORMAT_UNKNOWN;
+  pipeDesc.BlendState.RenderTarget[0].BlendEnable = FALSE;
+  pipeDesc.BlendState.RenderTarget[0].SrcBlend = D3D12_BLEND_SRC_ALPHA;
+  pipeDesc.BlendState.RenderTarget[0].DestBlend = D3D12_BLEND_INV_SRC_ALPHA;
+  pipeDesc.BlendState.RenderTarget[0].BlendOp = D3D12_BLEND_OP_ADD;
+  pipeDesc.BlendState.RenderTarget[0].SrcBlendAlpha = D3D12_BLEND_SRC_ALPHA;
+  pipeDesc.BlendState.RenderTarget[0].DestBlendAlpha = D3D12_BLEND_INV_SRC_ALPHA;
+  pipeDesc.BlendState.RenderTarget[0].BlendOpAlpha = D3D12_BLEND_OP_ADD;
+  pipeDesc.BlendState.RenderTarget[0].RenderTargetWriteMask = 0;
+
+  hr = m_pDevice->CreateGraphicsPipelineState(&pipeDesc, __uuidof(ID3D12PipelineState),
+                                              (void **)&m_TexSamplePso);
+  m_pDevice->InternalRef();
+  SAFE_RELEASE(vsBlob);
+  SAFE_RELEASE(psBlob);
+  if(FAILED(hr))
+  {
+    RDCERR("Failed to create PSO for shader debugging HRESULT: %s", ToStr(hr).c_str());
+    return false;
+  }
+  m_pDevice->GetResourceManager()->SetInternalResource(m_TexSamplePso);
 
   return true;
 }
 
+ID3D12PipelineState *D3D12DebugManager::GetTexSamplePso(const int8_t offsets[3])
+{
+  uint32_t offsKey = offsets[0] | (offsets[1] << 8) | (offsets[2] << 16);
+  if(offsKey == 0)
+    return m_TexSamplePso;
+
+  ID3D12PipelineState *ps = m_OffsetTexSamplePso[offsKey];
+  if(ps)
+    return ps;
+
+  D3D12ShaderCache *shaderCache = m_pDevice->GetShaderCache();
+
+  shaderCache->SetCaching(true);
+
+  rdcstr hlsl = GetEmbeddedResource(shaderdebug_hlsl);
+
+  hlsl = StringFormat::Fmt("#define debugSampleOffsets int4(%d,%d,%d,0)\n\n%s", offsets[0],
+                           offsets[1], offsets[2], hlsl.c_str());
+
+  ID3DBlob *vsBlob = NULL;
+  if(m_pDevice->GetShaderCache()->GetShaderBlob(hlsl.c_str(), "RENDERDOC_DebugSampleVS",
+                                                D3DCOMPILE_WARNINGS_ARE_ERRORS, {}, "vs_5_0",
+                                                &vsBlob) != "")
+  {
+    RDCERR("Failed to create shader to do texture sample");
+    SAFE_RELEASE(vsBlob);
+    return false;
+  }
+
+  ID3DBlob *psBlob = NULL;
+  if(m_pDevice->GetShaderCache()->GetShaderBlob(hlsl.c_str(), "RENDERDOC_DebugSamplePS",
+                                                D3DCOMPILE_WARNINGS_ARE_ERRORS, {}, "ps_5_0",
+                                                &psBlob) != "")
+  {
+    RDCERR("Failed to create shader to do texture sample");
+    SAFE_RELEASE(vsBlob);
+    SAFE_RELEASE(psBlob);
+    return false;
+  }
+
+  D3D12_GRAPHICS_PIPELINE_STATE_DESC pipeDesc = {};
+
+  pipeDesc.pRootSignature = m_ShaderDebugRootSig;
+  pipeDesc.VS.BytecodeLength = vsBlob->GetBufferSize();
+  pipeDesc.VS.pShaderBytecode = vsBlob->GetBufferPointer();
+  pipeDesc.PS.BytecodeLength = psBlob->GetBufferSize();
+  pipeDesc.PS.pShaderBytecode = psBlob->GetBufferPointer();
+  pipeDesc.RasterizerState.FillMode = D3D12_FILL_MODE_SOLID;
+  pipeDesc.RasterizerState.CullMode = D3D12_CULL_MODE_NONE;
+  pipeDesc.SampleMask = 0xFFFFFFFF;
+  pipeDesc.SampleDesc.Count = 1;
+  pipeDesc.IBStripCutValue = D3D12_INDEX_BUFFER_STRIP_CUT_VALUE_DISABLED;
+  pipeDesc.PrimitiveTopologyType = D3D12_PRIMITIVE_TOPOLOGY_TYPE_TRIANGLE;
+  pipeDesc.NumRenderTargets = 1;
+  pipeDesc.RTVFormats[0] = DXGI_FORMAT_R32G32B32A32_FLOAT;
+  pipeDesc.DSVFormat = DXGI_FORMAT_UNKNOWN;
+  pipeDesc.BlendState.RenderTarget[0].BlendEnable = FALSE;
+  pipeDesc.BlendState.RenderTarget[0].SrcBlend = D3D12_BLEND_SRC_ALPHA;
+  pipeDesc.BlendState.RenderTarget[0].DestBlend = D3D12_BLEND_INV_SRC_ALPHA;
+  pipeDesc.BlendState.RenderTarget[0].BlendOp = D3D12_BLEND_OP_ADD;
+  pipeDesc.BlendState.RenderTarget[0].SrcBlendAlpha = D3D12_BLEND_SRC_ALPHA;
+  pipeDesc.BlendState.RenderTarget[0].DestBlendAlpha = D3D12_BLEND_INV_SRC_ALPHA;
+  pipeDesc.BlendState.RenderTarget[0].BlendOpAlpha = D3D12_BLEND_OP_ADD;
+  pipeDesc.BlendState.RenderTarget[0].RenderTargetWriteMask = 0;
+
+  ID3D12PipelineState *pso = NULL;
+  HRESULT hr = m_pDevice->CreateGraphicsPipelineState(&pipeDesc, __uuidof(ID3D12PipelineState),
+                                                      (void **)&pso);
+  m_pDevice->InternalRef();
+  SAFE_RELEASE(vsBlob);
+  SAFE_RELEASE(psBlob);
+  if(FAILED(hr))
+  {
+    RDCERR("Failed to create PSO for shader debugging HRESULT: %s", ToStr(hr).c_str());
+    return false;
+  }
+  m_pDevice->GetResourceManager()->SetInternalResource(pso);
+
+  m_OffsetTexSamplePso[offsKey] = pso;
+
+  shaderCache->SetCaching(false);
+
+  return pso;
+}
+
 ID3D12Resource *D3D12DebugManager::MakeCBuffer(UINT64 size)
 {
   ID3D12Resource *ret;
@@ -965,16 +1112,73 @@ void D3D12DebugManager::FillWithDiscardPattern(ID3D12GraphicsCommandListX *cmd,
     return;
   }
 
+  static const uint32_t PatternBatchWidth = 256;
+  static const uint32_t PatternBatchHeight = 256;
+
   // see if we already have a buffer with texels in the desired format, if not then create it
   ID3D12Resource *buf = m_DiscardPatterns[desc.Format];
 
   if(buf == NULL)
   {
-    bytebuf pattern = GetDiscardPattern(type, MakeResourceFormat(desc.Format), 256);
+    bytebuf pattern = GetDiscardPattern(type, MakeResourceFormat(desc.Format));
 
-    buf = MakeCBuffer(pattern.size());
+    buf = MakeCBuffer(pattern.size() * (PatternBatchWidth / DiscardPatternWidth) *
+                      (PatternBatchHeight / DiscardPatternHeight));
 
-    FillBuffer(buf, 0, pattern.data(), pattern.size());
+    D3D12_RANGE range = {0, 0};
+    byte *ptr = NULL;
+    HRESULT hr = buf->Map(0, &range, (void **)&ptr);
+    m_pDevice->CheckHRESULT(hr);
+
+    if(ptr)
+    {
+      DXGI_FORMAT fmt = desc.Format;
+      int passes = 1;
+
+      if(IsDepthAndStencilFormat(fmt))
+      {
+        fmt = DXGI_FORMAT_R32_FLOAT;
+        passes = 2;
+      }
+
+      byte *dst = ptr;
+      size_t srcOffset = 0;
+
+      // row pitch is the same for depth and stencil
+      const uint32_t srcRowPitch = GetByteSize(DiscardPatternWidth, 1, 1, fmt, 0);
+
+      for(int pass = 0; pass < passes; pass++)
+      {
+        const uint32_t numHorizBatches = PatternBatchWidth / DiscardPatternWidth;
+        const uint32_t srcRowByteLength = GetByteSize(DiscardPatternWidth, 1, 1, fmt, 0);
+        const uint32_t dstRowByteLength = GetByteSize(PatternBatchWidth, 1, 1, fmt, 0);
+        uint32_t numDstRows = PatternBatchHeight;
+        uint32_t numSrcRows = DiscardPatternHeight;
+        if(IsBlockFormat(fmt))
+        {
+          numDstRows /= 4;
+          numSrcRows /= 4;
+        }
+
+        for(uint32_t row = 0; row < numDstRows; row++)
+        {
+          byte *src = pattern.data() + srcOffset + (row % numSrcRows) * srcRowPitch;
+          for(uint32_t x = 0; x < numHorizBatches; x++)
+            memcpy(dst + srcRowByteLength * x, src, srcRowByteLength);
+
+          dst += dstRowByteLength;
+        }
+
+        if(passes == 2)
+        {
+          fmt = DXGI_FORMAT_R8_UINT;
+          srcOffset =
+              GetByteSize(DiscardPatternWidth, DiscardPatternHeight, 1, DXGI_FORMAT_R32_FLOAT, 0);
+        }
+      }
+
+      buf->Unmap(0, NULL);
+    }
 
     m_DiscardPatterns[desc.Format] = buf;
   }
@@ -1020,8 +1224,7 @@ void D3D12DebugManager::FillWithDiscardPattern(ID3D12GraphicsCommandListX *cmd,
       else
       {
         fmt = DXGI_FORMAT_R8_TYPELESS;
-        bufOffset +=
-            GetByteSize(DiscardPatternWidth, DiscardPatternHeight, 1, DXGI_FORMAT_R32_FLOAT, 0);
+        bufOffset += GetByteSize(PatternBatchWidth, PatternBatchHeight, 1, DXGI_FORMAT_R32_FLOAT, 0);
       }
     }
 
@@ -1036,20 +1239,19 @@ void D3D12DebugManager::FillWithDiscardPattern(ID3D12GraphicsCommandListX *cmd,
         int32_t rectWidth = RDCMIN(LONG(RDCMAX(1U, (UINT)desc.Width >> mip)), r.right);
         int32_t rectHeight = RDCMIN(LONG(RDCMAX(1U, (UINT)desc.Height >> mip)), r.bottom);
 
-        for(int32_t y = r.top; y < rectHeight; y += DiscardPatternHeight)
+        for(int32_t y = r.top; y < rectHeight; y += PatternBatchHeight)
         {
-          for(int32_t x = r.left; x < rectWidth; x += DiscardPatternWidth)
+          for(int32_t x = r.left; x < rectWidth; x += PatternBatchWidth)
           {
             src.Type = D3D12_TEXTURE_COPY_TYPE_PLACED_FOOTPRINT;
             src.pResource = buf;
             src.PlacedFootprint.Offset = bufOffset;
             src.PlacedFootprint.Footprint.Format = fmt;
             src.PlacedFootprint.Footprint.RowPitch =
-                AlignUp(GetRowPitch(DiscardPatternWidth, fmt, 0), 256U);
-            src.PlacedFootprint.Footprint.Width =
-                RDCMIN(DiscardPatternWidth, uint32_t(rectWidth - x));
+                AlignUp(GetRowPitch(PatternBatchWidth, fmt, 0), 256U);
+            src.PlacedFootprint.Footprint.Width = RDCMIN(PatternBatchWidth, uint32_t(rectWidth - x));
             src.PlacedFootprint.Footprint.Height =
-                RDCMIN(DiscardPatternHeight, uint32_t(rectHeight - y));
+                RDCMIN(PatternBatchHeight, uint32_t(rectHeight - y));
             src.PlacedFootprint.Footprint.Depth = 1;
 
             cmd->CopyTextureRegion(&dst, x, y, z, &src, NULL);
@@ -1061,6 +1263,16 @@ void D3D12DebugManager::FillWithDiscardPattern(ID3D12GraphicsCommandListX *cmd,
     std::swap(b.Transition.StateBefore, b.Transition.StateAfter);
 
     cmd->ResourceBarrier(1, &b);
+
+    // workaround possible nvidia driver bug? without this depth-stencil discards of only one region
+    // (smaller than 256x256) will get corrupted if both depth and stencil are copied to.
+    if(desc.Flags & D3D12_RESOURCE_FLAG_ALLOW_DEPTH_STENCIL)
+    {
+      b.Type = D3D12_RESOURCE_BARRIER_TYPE_UAV;
+      b.UAV.pResource = NULL;
+
+      cmd->ResourceBarrier(1, &b);
+    }
   }
 }
 
@@ -1085,6 +1297,13 @@ D3D12_CPU_DESCRIPTOR_HANDLE D3D12DebugManager::GetCPUHandle(DSVSlot slot)
   return ret;
 }
 
+D3D12_CPU_DESCRIPTOR_HANDLE D3D12DebugManager::GetCPUHandle(SamplerSlot slot)
+{
+  D3D12_CPU_DESCRIPTOR_HANDLE ret = samplerHeap->GetCPUDescriptorHandleForHeapStart();
+  ret.ptr += slot * sizeof(D3D12Descriptor);
+  return ret;
+}
+
 D3D12_GPU_DESCRIPTOR_HANDLE D3D12DebugManager::GetGPUHandle(CBVUAVSRVSlot slot)
 {
   D3D12_GPU_DESCRIPTOR_HANDLE ret = cbvsrvuavHeap->GetGPUDescriptorHandleForHeapStart();
@@ -1219,6 +1438,15 @@ void D3D12DebugManager::SetDescriptorHeaps(ID3D12GraphicsCommandList *list, bool
   }
 }
 
+void D3D12DebugManager::SetDescriptorHeaps(rdcarray<ResourceId> &heaps, bool cbvsrvuav, bool samplers)
+{
+  heaps.clear();
+  if(cbvsrvuav)
+    heaps.push_back(GetResID(cbvsrvuavHeap));
+  if(samplers)
+    heaps.push_back(GetResID(samplerHeap));
+}
+
 D3D12_CPU_DESCRIPTOR_HANDLE D3D12DebugManager::GetUAVClearHandle(CBVUAVSRVSlot slot)
 {
   D3D12_CPU_DESCRIPTOR_HANDLE ret = uavClearHeap->GetCPUDescriptorHandleForHeapStart();
@@ -1915,7 +2143,7 @@ void D3D12Replay::VertexPicking::Init(WrappedID3D12Device *device, D3D12DebugMan
     heapProps.VisibleNodeMask = 1;
 
     hr = device->CreateCommittedResource(&heapProps, D3D12_HEAP_FLAG_NONE, &pickResultDesc,
-                                         D3D12_RESOURCE_STATE_UNORDERED_ACCESS, NULL,
+                                         D3D12_RESOURCE_STATE_COMMON, NULL,
                                          __uuidof(ID3D12Resource), (void **)&ResultBuf);
 
     ResultBuf->SetName(L"m_PickResultBuf");
@@ -2142,7 +2370,7 @@ void D3D12Replay::HistogramMinMax::Init(WrappedID3D12Device *device, D3D12DebugM
     heapProps.VisibleNodeMask = 1;
 
     hr = device->CreateCommittedResource(&heapProps, D3D12_HEAP_FLAG_NONE, &minmaxDesc,
-                                         D3D12_RESOURCE_STATE_UNORDERED_ACCESS, NULL,
+                                         D3D12_RESOURCE_STATE_COMMON, NULL,
                                          __uuidof(ID3D12Resource), (void **)&MinMaxTileBuffer);
 
     MinMaxTileBuffer->SetName(L"m_MinMaxTileBuffer");
@@ -2204,7 +2432,7 @@ void D3D12Replay::HistogramMinMax::Init(WrappedID3D12Device *device, D3D12DebugM
     minmaxDesc.Width = 2 * sizeof(Vec4f);
 
     hr = device->CreateCommittedResource(&heapProps, D3D12_HEAP_FLAG_NONE, &minmaxDesc,
-                                         D3D12_RESOURCE_STATE_UNORDERED_ACCESS, NULL,
+                                         D3D12_RESOURCE_STATE_COMMON, NULL,
                                          __uuidof(ID3D12Resource), (void **)&MinMaxResultBuffer);
 
     MinMaxResultBuffer->SetName(L"m_MinMaxResultBuffer");
