@@ -77,6 +77,9 @@ enum CBVUAVSRVSlot
   STENCIL_MSAA_SRV16x,
   STENCIL_MSAA_SRV32x,
 
+  FIRST_SHADDEBUG_SRV,
+  LAST_SHADDEBUG_SRV = FIRST_SHADDEBUG_SRV + 25,
+
   MAX_SRV_SLOT,
 };
 
@@ -98,6 +101,8 @@ enum SamplerSlot
   POINT_SAMP,
   FIRST_SAMP = POINT_SAMP,
   LINEAR_SAMP,
+  SHADDEBUG_SAMPLER0,
+  SHADDEBUG_SAMPLER1,
 };
 
 enum DSVSlot
@@ -138,9 +143,11 @@ public:
   D3D12_GPU_VIRTUAL_ADDRESS UploadConstants(const void *data, size_t size);
 
   ID3D12RootSignature *GetMeshRootSig() { return m_MeshRootSig; }
-  ID3D12RootSignature *GetMathIntrinsicsRootSig() { return m_MathIntrinsicsRootSig; }
+  ID3D12RootSignature *GetShaderDebugRootSig() { return m_ShaderDebugRootSig; }
   ID3D12PipelineState *GetMathIntrinsicsPso() { return m_MathIntrinsicsPso; }
-  ID3D12Resource *GetMathIntrinsicsResultBuffer() { return m_MathIntrinsicsResultBuffer; }
+  ID3D12PipelineState *GetTexSamplePso(const int8_t offsets[3]);
+  ID3D12Resource *GetShaderDebugResultBuffer() { return m_ShaderDebugResultBuffer; }
+  ID3D12Resource *GetReadbackBuffer() { return m_ReadbackBuffer; }
   ID3D12GraphicsCommandListX *ResetDebugList();
   void ResetDebugAlloc();
 
@@ -151,6 +158,7 @@ public:
   D3D12_CPU_DESCRIPTOR_HANDLE GetCPUHandle(CBVUAVSRVSlot slot);
   D3D12_CPU_DESCRIPTOR_HANDLE GetCPUHandle(RTVSlot slot);
   D3D12_CPU_DESCRIPTOR_HANDLE GetCPUHandle(DSVSlot slot);
+  D3D12_CPU_DESCRIPTOR_HANDLE GetCPUHandle(SamplerSlot slot);
 
   D3D12_GPU_DESCRIPTOR_HANDLE GetGPUHandle(CBVUAVSRVSlot slot);
   D3D12_GPU_DESCRIPTOR_HANDLE GetGPUHandle(RTVSlot slot);
@@ -159,6 +167,7 @@ public:
 
   D3D12_CPU_DESCRIPTOR_HANDLE GetTempDescriptor(const D3D12Descriptor &desc, size_t idx = 0);
 
+  void SetDescriptorHeaps(rdcarray<ResourceId> &heaps, bool cbvsrvuav, bool samplers);
   void SetDescriptorHeaps(ID3D12GraphicsCommandList *list, bool cbvsrvuav, bool samplers);
 
   D3D12_CPU_DESCRIPTOR_HANDLE GetUAVClearHandle(CBVUAVSRVSlot slot);
@@ -174,7 +183,7 @@ public:
   void CopyArrayToTex2DMS(ID3D12Resource *destMS, ID3D12Resource *srcArray, UINT selectedSlice);
 
 private:
-  bool CreateMathIntrinsicsResources();
+  bool CreateShaderDebugResources();
 
   rdcpair<ID3D12PipelineState *, ID3D12PipelineState *> D3D12DebugManager::GetMSToArrayPSOs(
       DXGI_FORMAT format);
@@ -205,9 +214,11 @@ private:
   std::map<uint64_t, MeshDisplayPipelines> m_CachedMeshPipelines;
 
   // Shader debugging resources
-  ID3D12RootSignature *m_MathIntrinsicsRootSig = NULL;
+  ID3D12RootSignature *m_ShaderDebugRootSig = NULL;
   ID3D12PipelineState *m_MathIntrinsicsPso = NULL;
-  ID3D12Resource *m_MathIntrinsicsResultBuffer = NULL;
+  ID3D12Resource *m_ShaderDebugResultBuffer = NULL;
+  ID3D12PipelineState *m_TexSamplePso = NULL;
+  std::map<uint32_t, ID3D12PipelineState *> m_OffsetTexSamplePso;
 
   // GetBufferData
   static const uint64_t m_ReadbackSize = 16 * 1024 * 1024;
