@@ -54,13 +54,27 @@ struct D3D12InitParams
   UINT SDKVersion = 0;
 
   // check if a frame capture section version is supported
-  static const uint64_t CurrentVersion = 0xD;
+  static const uint64_t CurrentVersion = 0xE;
 
   static bool IsSupportedVersion(uint64_t ver);
 };
 
 DECLARE_REFLECTION_STRUCT(D3D12InitParams);
 
+struct QueueReadbackData
+{
+  Threading::CriticalSection lock;
+  ID3D12Resource *readbackBuf = NULL;
+  byte *readbackMapped = NULL;
+  uint64_t readbackSize = 0;
+  ID3D12GraphicsCommandList *lists[6] = {};
+  ID3D12CommandAllocator *allocs[6] = {};
+
+  void Resize(uint64_t size);
+
+  WrappedID3D12Device *device;
+};
+
 class WrappedID3D12Device;
 class WrappedID3D12Resource;
 class WrappedID3D12PipelineState;
@@ -588,7 +602,9 @@ private:
   WrappedID3D12CommandQueue *m_Queue;
 
   ID3D12CommandAllocator *m_Alloc = NULL, *m_DataUploadAlloc = NULL;
-  ID3D12GraphicsCommandList *m_DataUploadList = NULL;
+  QueueReadbackData m_QueueReadbackData;
+  ID3D12GraphicsCommandList *m_DataUploadList[64] = {};
+  size_t m_CurDataUpload = 0;
   ID3D12DescriptorHeap *m_RTVHeap = NULL;
   ID3D12Fence *m_GPUSyncFence;
   HANDLE m_GPUSyncHandle;
@@ -633,7 +649,9 @@ private:
   D3D12TextRenderer *m_TextRenderer = NULL;
 
   std::set<ResourceId> m_UploadResourceIds;
-  std::map<uint64_t, ID3D12Resource *> m_UploadBuffers;
+  std::set<ResourceId> m_ModResources;
+  rdcflatmap<uint64_t, ID3D12Resource *> m_UploadBuffers;
+  rdcflatmap<uint64_t, D3D12_RANGE> m_UploadRanges;
 
   Threading::CriticalSection m_MapsLock;
   rdcarray<MapState> m_Maps;
@@ -682,6 +700,8 @@ private:
   RDResult m_FailedReplayResult = ResultCode::APIReplayFailed;
 
   bool m_AppControlledCapture = false;
+  bool m_FirstFrameCapture = false;
+  void *m_FirstFrameCaptureWindow = NULL;
 
   Threading::RWLock m_CapTransitionLock;
   CaptureState m_State;
@@ -713,6 +733,7 @@ private:
   // used both on capture and replay side to track resource states. Only locked
   // in capture
   std::map<ResourceId, SubresourceStateVector> m_ResourceStates;
+  std::unordered_map<ResourceId, FrameRefType> m_BindlessFrameRefs;
   Threading::CriticalSection m_ResourceStatesLock;
 
   // used on replay only. Contains the initial resource states before any barriers - this allows us
@@ -733,6 +754,9 @@ private:
     WrappedID3D12CommandQueue *queue;
   };
 
+  bool m_BindlessResourceUseActive = false;
+  FrameRefType BindlessRefTypeForRes(ID3D12Resource *wrapped);
+
   std::map<IDXGISwapper *, SwapPresentInfo> m_SwapChains;
   std::map<ResourceId, DXGI_FORMAT> m_BackbufferFormat;
 
@@ -797,6 +821,8 @@ public:
   void FirstFrame(IDXGISwapper *swapper);
   const ActionDescription *GetAction(uint32_t eventId);
 
+  QueueReadbackData &GetQueueReadbackData() { return m_QueueReadbackData; }
+  bool IsBindlessResourceUseActive() const { return m_BindlessResourceUseActive; }
   ResourceId GetFrameCaptureResourceId() { return m_FrameCaptureRecord->GetResourceID(); }
   void AddDebugMessage(MessageCategory c, MessageSeverity sv, MessageSource src, rdcstr d);
   void AddDebugMessage(const DebugMessage &msg);
@@ -841,7 +867,7 @@ public:
     m_ReplayAGS = ags;
   }
   const ReplayOptions &GetReplayOptions() { return m_ReplayOptions; }
-  uint64_t GetLogVersion() { return m_SectionVersion; }
+  uint64_t GetCaptureVersion() { return m_SectionVersion; }
   CaptureState GetState() { return m_State; }
   D3D12Replay *GetReplay() { return m_Replay; }
   WrappedID3D12CommandQueue *GetQueue() { return m_Queue; }
@@ -898,6 +924,8 @@ public:
 
   ID3D12GraphicsCommandListX *GetNewList();
   ID3D12GraphicsCommandListX *GetInitialStateList();
+
+  bool IsReadOnlyResource(ResourceId id) { return m_ModResources.find(id) == m_ModResources.end(); }
   void CloseInitialStateList();
   ID3D12Resource *GetUploadBuffer(uint64_t chunkOffset, uint64_t byteSize);
   void ApplyInitialContents();
@@ -914,9 +942,9 @@ public:
   void GPUSyncAllQueues();
 
   RDCDriver GetFrameCaptureDriver() { return RDCDriver::D3D12; }
-  void StartFrameCapture(void *dev, void *wnd);
-  bool EndFrameCapture(void *dev, void *wnd);
-  bool DiscardFrameCapture(void *dev, void *wnd);
+  void StartFrameCapture(DeviceOwnedWindow devWnd);
+  bool EndFrameCapture(DeviceOwnedWindow devWnd);
+  bool DiscardFrameCapture(DeviceOwnedWindow devWnd);
 
   template <typename SerialiserType>
   bool Serialise_Present(SerialiserType &ser, ID3D12Resource *PresentedImage, UINT SyncInterval,
