@@ -298,6 +298,9 @@ bool WrappedID3D12Device::Serialise_CreateCommittedResource1(
 
       GetResourceManager()->AddLiveResource(pResource, ret);
 
+      if(desc.Flags & D3D12_RESOURCE_FLAG_ALLOW_UNORDERED_ACCESS)
+        m_ModResources.insert(GetResID(ret));
+
       SubresourceStateVector &states = m_ResourceStates[GetResID(ret)];
       states.fill(GetNumSubresources(m_pDevice, &desc), InitialResourceState);
 
@@ -359,10 +362,21 @@ HRESULT WrappedID3D12Device::CreateCommittedResource1(
      riidResource != __uuidof(ID3D12Resource2))
     return E_NOINTERFACE;
 
+  const D3D12_RESOURCE_DESC *pCreateDesc = pDesc;
+  D3D12_RESOURCE_DESC localDesc;
+
+  if(pDesc && pDesc->Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE2D && pDesc->SampleDesc.Count > 1)
+  {
+    localDesc = *pDesc;
+    // need to be able to create SRVs of MSAA textures to copy out their contents
+    localDesc.Flags &= ~D3D12_RESOURCE_FLAG_DENY_SHADER_RESOURCE;
+    pCreateDesc = &localDesc;
+  }
+
   void *realptr = NULL;
   HRESULT ret;
   SERIALISE_TIME_CALL(ret = m_pDevice4->CreateCommittedResource1(
-                          pHeapProperties, HeapFlags, pDesc, InitialResourceState,
+                          pHeapProperties, HeapFlags, pCreateDesc, InitialResourceState,
                           pOptimizedClearValue, Unwrap(pProtectedSession), riidResource, &realptr));
 
   ID3D12Resource *real = NULL;
@@ -408,6 +422,8 @@ HRESULT WrappedID3D12Device::CreateCommittedResource1(
       SubresourceStateVector &states = m_ResourceStates[wrapped->GetResourceID()];
 
       states.fill(GetNumSubresources(m_pDevice, pDesc), InitialResourceState);
+
+      m_BindlessFrameRefs[wrapped->GetResourceID()] = BindlessRefTypeForRes(wrapped);
     }
 
     if(riidResource == __uuidof(ID3D12Resource))
@@ -426,6 +442,9 @@ HRESULT WrappedID3D12Device::CreateCommittedResource1(
       {
         wrapped->AddRef();
         m_RefBuffers.push_back(wrapped);
+        if(m_BindlessResourceUseActive)
+          GetResourceManager()->MarkResourceFrameReferenced(wrapped->GetResourceID(),
+                                                            BindlessRefTypeForRes(wrapped));
       }
     }
   }
