@@ -119,6 +119,9 @@ bool WrappedID3D12Device::Serialise_CreateCommittedResource2(
 
       GetResourceManager()->AddLiveResource(pResource, ret);
 
+      if(desc.Flags & D3D12_RESOURCE_FLAG_ALLOW_UNORDERED_ACCESS)
+        m_ModResources.insert(GetResID(ret));
+
       SubresourceStateVector &states = m_ResourceStates[GetResID(ret)];
       // D3D12_RESOURCE_DESC is the same as the start of D3D12_RESOURCE_DESC1
       D3D12_RESOURCE_DESC desc0;
@@ -183,10 +186,21 @@ HRESULT WrappedID3D12Device::CreateCommittedResource2(
      riidResource != __uuidof(ID3D12Resource2))
     return E_NOINTERFACE;
 
+  const D3D12_RESOURCE_DESC1 *pCreateDesc = pDesc;
+  D3D12_RESOURCE_DESC1 localDesc;
+
+  if(pDesc && pDesc->Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE2D && pDesc->SampleDesc.Count > 1)
+  {
+    localDesc = *pDesc;
+    // need to be able to create SRVs of MSAA textures to copy out their contents
+    localDesc.Flags &= ~D3D12_RESOURCE_FLAG_DENY_SHADER_RESOURCE;
+    pCreateDesc = &localDesc;
+  }
+
   void *realptr = NULL;
   HRESULT ret;
   SERIALISE_TIME_CALL(ret = m_pDevice8->CreateCommittedResource2(
-                          pHeapProperties, HeapFlags, pDesc, InitialResourceState,
+                          pHeapProperties, HeapFlags, pCreateDesc, InitialResourceState,
                           pOptimizedClearValue, Unwrap(pProtectedSession), riidResource, &realptr));
 
   ID3D12Resource *real = NULL;
@@ -238,6 +252,8 @@ HRESULT WrappedID3D12Device::CreateCommittedResource2(
       D3D12_RESOURCE_DESC desc0;
       memcpy(&desc0, pDesc, sizeof(desc0));
       states.fill(GetNumSubresources(m_pDevice, &desc0), InitialResourceState);
+
+      m_BindlessFrameRefs[wrapped->GetResourceID()] = BindlessRefTypeForRes(wrapped);
     }
 
     if(riidResource == __uuidof(ID3D12Resource))
@@ -256,6 +272,9 @@ HRESULT WrappedID3D12Device::CreateCommittedResource2(
       {
         wrapped->AddRef();
         m_RefBuffers.push_back(wrapped);
+        if(m_BindlessResourceUseActive)
+          GetResourceManager()->MarkResourceFrameReferenced(wrapped->GetResourceID(),
+                                                            BindlessRefTypeForRes(wrapped));
       }
     }
   }
@@ -344,6 +363,9 @@ bool WrappedID3D12Device::Serialise_CreatePlacedResource1(
 
       GetResourceManager()->AddLiveResource(pResource, ret);
 
+      if(Descriptor.Flags & D3D12_RESOURCE_FLAG_ALLOW_UNORDERED_ACCESS)
+        m_ModResources.insert(GetResID(ret));
+
       // D3D12_RESOURCE_DESC is the same as the start of D3D12_RESOURCE_DESC1
       D3D12_RESOURCE_DESC desc0;
       memcpy(&desc0, &Descriptor, sizeof(desc0));
@@ -469,6 +491,8 @@ HRESULT WrappedID3D12Device::CreatePlacedResource1(ID3D12Heap *pHeap, UINT64 Hea
       D3D12_RESOURCE_DESC desc0;
       memcpy(&desc0, pDesc, sizeof(desc0));
       states.fill(GetNumSubresources(m_pDevice, &desc0), InitialState);
+
+      m_BindlessFrameRefs[wrapped->GetResourceID()] = BindlessRefTypeForRes(wrapped);
     }
 
     if(riid == __uuidof(ID3D12Resource))
@@ -487,6 +511,9 @@ HRESULT WrappedID3D12Device::CreatePlacedResource1(ID3D12Heap *pHeap, UINT64 Hea
       {
         wrapped->AddRef();
         m_RefBuffers.push_back(wrapped);
+        if(m_BindlessResourceUseActive)
+          GetResourceManager()->MarkResourceFrameReferenced(wrapped->GetResourceID(),
+                                                            BindlessRefTypeForRes(wrapped));
       }
     }
   }
