@@ -87,7 +87,7 @@ static D3D12FeedbackKey GetDirectHeapAccessKey()
 
 static bool AnnotateDXBCShader(const DXBC::DXBCContainer *dxbc, uint32_t space,
                                const std::map<D3D12FeedbackKey, D3D12FeedbackSlot> &slots,
-                               bytebuf &editedBlob)
+                               uint32_t maxSlot, bytebuf &editedBlob)
 {
   using namespace DXBCBytecode;
   using namespace DXBCBytecode::Edit;
@@ -174,9 +174,11 @@ static bool AnnotateDXBCShader(const DXBC::DXBCContainer *dxbc, uint32_t space,
       if(u.first == ~0U && u.second == ~0U)
         u = editor.DeclareUAV(desc, space, 0, 0);
 
+      // clamp to number of slots
+      editor.InsertOperation(i++, oper(OPCODE_UMIN, {temp(t).swizzle(0), imm(maxSlot), idx}));
       // resource base plus index
-      editor.InsertOperation(i++,
-                             oper(OPCODE_IADD, {temp(t).swizzle(0), imm(it->second.Slot()), idx}));
+      editor.InsertOperation(
+          i++, oper(OPCODE_IADD, {temp(t).swizzle(0), temp(t).swizzle(0), imm(it->second.Slot())}));
       // multiply by 4 for byte index
       editor.InsertOperation(i++,
                              oper(OPCODE_ISHL, {temp(t).swizzle(0), temp(t).swizzle(0), imm(2)}));
@@ -202,7 +204,7 @@ static bool AnnotateDXBCShader(const DXBC::DXBCContainer *dxbc, uint32_t space,
 
 static bool AnnotateDXILShader(const DXBC::DXBCContainer *dxbc, uint32_t space,
                                const std::map<D3D12FeedbackKey, D3D12FeedbackSlot> &slots,
-                               bytebuf &editedBlob)
+                               uint32_t maxSlot, bytebuf &editedBlob)
 {
   DXIL::ProgramEditor editor(dxbc, slots.size() + 64, editedBlob);
 
@@ -280,7 +282,7 @@ static bool AnnotateDXILShader(const DXBC::DXBCContainer *dxbc, uint32_t space,
     createHandleFromBinding = editor.DeclareFunction(createHandleBaseFunction);
   }
 
-  // get the atomic function we'll need
+  // get the functions we'll need
   const DXIL::Function *atomicBinOp = editor.GetFunctionByName("dx.op.atomicBinOp.i32");
   if(!atomicBinOp)
   {
@@ -314,6 +316,68 @@ static bool AnnotateDXILShader(const DXBC::DXBCContainer *dxbc, uint32_t space,
     atomicBinOp = editor.DeclareFunction(atomicFunc);
   }
 
+  const DXIL::Function *binOp = editor.GetFunctionByName("dx.op.binOp.i32");
+  if(!binOp)
+  {
+    DXIL::Type binopType;
+    binopType.type = DXIL::Type::Function;
+    // return type
+    binopType.inner = i32;
+    binopType.members = {i32, i32, i32};
+
+    const DXIL::Type *funcType = editor.AddType(binopType);
+
+    DXIL::Function binopFunc;
+    binopFunc.name = "dx.op.binOp.i32";
+    binopFunc.funcType = funcType;
+    binopFunc.external = true;
+
+    for(const DXIL::AttributeSet &attrs : editor.GetAttributeSets())
+    {
+      if(attrs.functionSlot &&
+         attrs.functionSlot->params == (DXIL::Attribute::NoUnwind | DXIL::Attribute::ReadNone))
+      {
+        binopFunc.attrs = &attrs;
+        break;
+      }
+    }
+
+    // we haven't implemented adding attribute sets since their encoding is obtuse, so if we can't
+    // get the 'real' binop attributes try to get the next most conservative one
+    if(!binopFunc.attrs)
+    {
+      for(const DXIL::AttributeSet &attrs : editor.GetAttributeSets())
+      {
+        if(attrs.functionSlot &&
+           attrs.functionSlot->params == (DXIL::Attribute::NoUnwind | DXIL::Attribute::ReadOnly))
+        {
+          binopFunc.attrs = &attrs;
+          break;
+        }
+      }
+    }
+
+    if(!binopFunc.attrs)
+    {
+      for(const DXIL::AttributeSet &attrs : editor.GetAttributeSets())
+      {
+        if(attrs.functionSlot && attrs.functionSlot->params == DXIL::Attribute::NoUnwind)
+        {
+          binopFunc.attrs = &attrs;
+          break;
+        }
+      }
+    }
+
+    if(!binopFunc.attrs)
+      RDCWARN("Couldn't find existing nounwind readnone attr set");
+
+    // should we add a set here? or assume the attrs don't matter because this is a builtin function
+    // anyway?
+
+    binOp = editor.DeclareFunction(binopFunc);
+  }
+
   // while we're iterating through the metadata to add our UAV, we'll also note the shader-local
   // register IDs of each SRV/UAV with slots, and record the base slot in this array for easy access
   // later when annotating dx.op.createHandle calls. We also need to know the base register because
@@ -900,13 +964,30 @@ static bool AnnotateDXILShader(const DXBC::DXBCContainer *dxbc, uint32_t space,
       DXIL::Instruction *slotPlusBase = editor.AddInstruction(f, i, op);
       i++;
 
+      op.op = DXIL::Operation::Call;
+      op.type = i32;
+      op.funcCall = binOp;
+      op.args = {
+          // dx.op.binOp.i32 UMin opcode
+          DXIL::Value(editor.GetOrAddConstant(f, DXIL::Constant(i32, 40U))),
+          // slotPlusBase
+          DXIL::Value(slotPlusBase),
+          // max slot
+          DXIL::Value(editor.GetOrAddConstant(f, DXIL::Constant(i32, maxSlot))),
+      };
+
+      // slotPlusBaseClamped = min(slotPlusBase, maxSlot)
+      DXIL::Instruction *slotPlusBaseClamped = editor.AddInstruction(f, i, op);
+      i++;
+
       op.op = DXIL::Operation::ShiftLeft;
       op.type = i32;
       op.args = {
-          DXIL::Value(slotPlusBase), DXIL::Value(editor.GetOrAddConstant(f, DXIL::Constant(i32, 2U))),
+          DXIL::Value(slotPlusBaseClamped),
+          DXIL::Value(editor.GetOrAddConstant(f, DXIL::Constant(i32, 2U))),
       };
 
-      // byteOffset = slotPlusBase << 2
+      // byteOffset = slotPlusBaseClamped << 2
       DXIL::Instruction *byteOffset = editor.AddInstruction(f, i, op);
       i++;
 
@@ -1081,13 +1162,15 @@ static bool AnnotateDXILShader(const DXBC::DXBCContainer *dxbc, uint32_t space,
   return true;
 }
 
-static void AddArraySlots(WrappedID3D12PipelineState::ShaderEntry *shad, uint32_t space,
+static bool AddArraySlots(WrappedID3D12PipelineState::ShaderEntry *shad, uint32_t space,
                           uint32_t maxDescriptors,
                           std::map<D3D12FeedbackKey, D3D12FeedbackSlot> &slots, uint32_t &numSlots,
                           bytebuf &editedBlob, D3D12_SHADER_BYTECODE &desc, bool directHeapAccess)
 {
   if(!shad)
-    return;
+    return false;
+
+  bool dynamicUsed = false;
 
   ShaderReflection &refl = shad->GetDetails();
   const ShaderBindpointMapping &mapping = shad->GetMapping();
@@ -1103,6 +1186,7 @@ static void AddArraySlots(WrappedID3D12PipelineState::ShaderEntry *shad, uint32_
 
       slots[key].SetSlot(numSlots);
       numSlots += RDCMIN(maxDescriptors, bind.arraySize);
+      dynamicUsed = true;
     }
     else if(bind.arraySize <= 1 && bind.used)
     {
@@ -1130,6 +1214,7 @@ static void AddArraySlots(WrappedID3D12PipelineState::ShaderEntry *shad, uint32_
 
       slots[key].SetSlot(numSlots);
       numSlots += RDCMIN(maxDescriptors, bind.arraySize);
+      dynamicUsed = true;
     }
     else if(bind.arraySize <= 1 && bind.used)
     {
@@ -1159,16 +1244,17 @@ static void AddArraySlots(WrappedID3D12PipelineState::ShaderEntry *shad, uint32_
     D3D12FeedbackKey key = GetDirectHeapAccessKey();
     slots[key].SetSlot(numSlots);
     numSlots += maxDescriptors;
+    dynamicUsed = true;
   }
 
   // if we haven't encountered any array slots, no need to do any patching
-  if(numSlots == numReservedSlots)
-    return;
+  if(!dynamicUsed)
+    return false;
 
   // only SM5.1 can have dynamic array indexing
   if(shad->GetDXBC()->m_Version.Major == 5 && shad->GetDXBC()->m_Version.Minor == 1)
   {
-    if(AnnotateDXBCShader(shad->GetDXBC(), space, slots, editedBlob))
+    if(AnnotateDXBCShader(shad->GetDXBC(), space, slots, numSlots, editedBlob))
     {
       if(!D3D12_Debug_FeedbackDumpDirPath().empty())
         FileIO::WriteAll(D3D12_Debug_FeedbackDumpDirPath() + "/before_dxbc_" +
@@ -1186,7 +1272,7 @@ static void AddArraySlots(WrappedID3D12PipelineState::ShaderEntry *shad, uint32_
   }
   else if(shad->GetDXBC()->m_Version.Major >= 6)
   {
-    if(AnnotateDXILShader(shad->GetDXBC(), space, slots, editedBlob))
+    if(AnnotateDXILShader(shad->GetDXBC(), space, slots, numSlots, editedBlob))
     {
       // strip ILDB because it's valid code (with debug info) and who knows what might use it
       DXBC::DXBCContainer::StripChunk(editedBlob, DXBC::FOURCC_ILDB);
@@ -1213,18 +1299,77 @@ static void AddArraySlots(WrappedID3D12PipelineState::ShaderEntry *shad, uint32_
       desc.BytecodeLength = editedBlob.size();
     }
   }
+
+  return true;
 }
 
-void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
+struct D3D12StatCallback : public D3D12ActionCallback
+{
+  D3D12StatCallback(WrappedID3D12Device *dev, ID3D12QueryHeap *heap)
+      : m_pDevice(dev), m_PipeStatsQueryHeap(heap)
+  {
+    m_pDevice->GetQueue()->GetCommandData()->m_ActionCallback = this;
+  }
+  ~D3D12StatCallback() { m_pDevice->GetQueue()->GetCommandData()->m_ActionCallback = NULL; }
+  void PreDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
+  {
+    if(cmd->GetType() == D3D12_COMMAND_LIST_TYPE_DIRECT)
+      cmd->BeginQuery(m_PipeStatsQueryHeap, D3D12_QUERY_TYPE_PIPELINE_STATISTICS, 0);
+  }
+
+  bool PostDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
+  {
+    if(cmd->GetType() == D3D12_COMMAND_LIST_TYPE_DIRECT)
+      cmd->EndQuery(m_PipeStatsQueryHeap, D3D12_QUERY_TYPE_PIPELINE_STATISTICS, 0);
+    return false;
+  }
+
+  void PostRedraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override {}
+  // we don't need to distinguish, call the Draw functions
+  void PreDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override { PreDraw(eid, cmd); }
+  bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
+  {
+    return PostDraw(eid, cmd);
+  }
+  void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
+  {
+    PostRedraw(eid, cmd);
+  }
+  void PreMisc(uint32_t eid, ActionFlags flags, ID3D12GraphicsCommandListX *cmd) override
+  {
+    if(flags & ActionFlags::PassBoundary)
+      return;
+    PreDraw(eid, cmd);
+  }
+  bool PostMisc(uint32_t eid, ActionFlags flags, ID3D12GraphicsCommandListX *cmd) override
+  {
+    if(flags & ActionFlags::PassBoundary)
+      return false;
+    return PostDraw(eid, cmd);
+  }
+  void PostRemisc(uint32_t eid, ActionFlags flags, ID3D12GraphicsCommandListX *cmd) override
+  {
+    if(flags & ActionFlags::PassBoundary)
+      return;
+    PostRedraw(eid, cmd);
+  }
+
+  void PreCloseCommandList(ID3D12GraphicsCommandListX *cmd) override{};
+  void AliasEvent(uint32_t primary, uint32_t alias) override {}
+  WrappedID3D12Device *m_pDevice;
+  ID3D12QueryHeap *m_PipeStatsQueryHeap;
+};
+
+bool D3D12Replay::FetchShaderFeedback(uint32_t eventId)
 {
   if(m_BindlessFeedback.Usage.find(eventId) != m_BindlessFeedback.Usage.end())
-    return;
+    return false;
 
   if(!D3D12_BindlessFeedback())
-    return;
+    return false;
 
   if(m_pDevice->HasFatalError())
-    return;
+    return false;
 
   // create it here so we won't re-run any code if the event is re-selected. We'll mark it as valid
   // if it actually has any data in it later.
@@ -1233,7 +1378,11 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
   const ActionDescription *action = m_pDevice->GetAction(eventId);
 
   if(action == NULL || !(action->flags & (ActionFlags::Dispatch | ActionFlags::Drawcall)))
-    return;
+  {
+    // deliberately show no bindings as used for non-draws
+    result.valid = true;
+    return false;
+  }
 
   result.compute = bool(action->flags & ActionFlags::Dispatch);
 
@@ -1248,7 +1397,8 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
   if(!pipe)
   {
     RDCERR("Can't fetch shader feedback, no pipeline state bound");
-    return;
+    result.valid = true;
+    return false;
   }
 
   bytebuf editedBlob[(uint32_t)ShaderStage::Count];
@@ -1277,12 +1427,17 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
   m_pDevice->GetShaderCache()->LoadDXC();
 #endif
 
+  bool dynamicAccessPerStage[6] = {};
+
   if(result.compute)
   {
     ID3D12RootSignature *sig = rm->GetCurrentAs<ID3D12RootSignature>(rs.compute.rootsig);
 
     if(!sig)
-      return;
+    {
+      result.valid = true;
+      return false;
+    }
 
     modsig = ((WrappedID3D12RootSignature *)sig)->sig;
     bool directHeapAccess =
@@ -1290,15 +1445,19 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
                          D3D12_ROOT_SIGNATURE_FLAG_SAMPLER_HEAP_DIRECTLY_INDEXED)) != 0;
     space = modsig.maxSpaceIndex;
 
-    AddArraySlots(pipe->CS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Compute], numSlots,
-                  editedBlob[(uint32_t)ShaderStage::Compute], pipeDesc.CS, directHeapAccess);
+    dynamicAccessPerStage[5] = AddArraySlots(
+        pipe->CS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Compute], numSlots,
+        editedBlob[(uint32_t)ShaderStage::Compute], pipeDesc.CS, directHeapAccess);
   }
   else
   {
     ID3D12RootSignature *sig = rm->GetCurrentAs<ID3D12RootSignature>(rs.graphics.rootsig);
 
     if(!sig)
-      return;
+    {
+      result.valid = true;
+      return false;
+    }
 
     modsig = ((WrappedID3D12RootSignature *)sig)->sig;
     bool directHeapAccess =
@@ -1307,22 +1466,31 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
 
     space = modsig.maxSpaceIndex;
 
-    AddArraySlots(pipe->VS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Vertex], numSlots,
-                  editedBlob[uint32_t(ShaderStage::Vertex)], pipeDesc.VS, directHeapAccess);
-    AddArraySlots(pipe->HS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Hull], numSlots,
-                  editedBlob[uint32_t(ShaderStage::Hull)], pipeDesc.HS, directHeapAccess);
-    AddArraySlots(pipe->DS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Domain], numSlots,
-                  editedBlob[uint32_t(ShaderStage::Domain)], pipeDesc.DS, directHeapAccess);
-    AddArraySlots(pipe->GS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Geometry], numSlots,
-                  editedBlob[uint32_t(ShaderStage::Geometry)], pipeDesc.GS, directHeapAccess);
-    AddArraySlots(pipe->PS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Pixel], numSlots,
-                  editedBlob[uint32_t(ShaderStage::Pixel)], pipeDesc.PS, directHeapAccess);
+    dynamicAccessPerStage[0] = AddArraySlots(
+        pipe->VS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Vertex], numSlots,
+        editedBlob[uint32_t(ShaderStage::Vertex)], pipeDesc.VS, directHeapAccess);
+    dynamicAccessPerStage[1] = AddArraySlots(
+        pipe->HS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Hull], numSlots,
+        editedBlob[uint32_t(ShaderStage::Hull)], pipeDesc.HS, directHeapAccess);
+    dynamicAccessPerStage[2] = AddArraySlots(
+        pipe->DS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Domain], numSlots,
+        editedBlob[uint32_t(ShaderStage::Domain)], pipeDesc.DS, directHeapAccess);
+    dynamicAccessPerStage[3] = AddArraySlots(
+        pipe->GS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Geometry], numSlots,
+        editedBlob[uint32_t(ShaderStage::Geometry)], pipeDesc.GS, directHeapAccess);
+    dynamicAccessPerStage[4] = AddArraySlots(
+        pipe->PS(), space, maxDescriptors, slots[(uint32_t)ShaderStage::Pixel], numSlots,
+        editedBlob[uint32_t(ShaderStage::Pixel)], pipeDesc.PS, directHeapAccess);
   }
 
   // if numSlots wasn't increased, none of the resources were arrayed so we have nothing to do.
   // Silently return
   if(numSlots == numReservedSlots)
-    return;
+  {
+    return false;
+  }
+
+  numSlots = AlignUp16(numSlots);
 
   // need to be able to add a descriptor of our UAV without hitting the 64 DWORD limit
   if(modsig.dwordLength > 62)
@@ -1341,6 +1509,22 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
     param.Descriptor.ShaderRegister = 0;
   }
 
+  if(m_BindlessFeedback.PipeStatsHeap == NULL)
+  {
+    D3D12_QUERY_HEAP_DESC pipestatsQueryDesc;
+    pipestatsQueryDesc.Count = 1;
+    pipestatsQueryDesc.NodeMask = 1;
+    pipestatsQueryDesc.Type = D3D12_QUERY_HEAP_TYPE_PIPELINE_STATISTICS;
+    HRESULT hr = m_pDevice->CreateQueryHeap(&pipestatsQueryDesc, __uuidof(ID3D12QueryHeap),
+                                            (void **)&m_BindlessFeedback.PipeStatsHeap);
+    m_pDevice->CheckHRESULT(hr);
+    if(FAILED(hr))
+    {
+      RDCERR("Failed to create shader feedback pipeline query heap HRESULT: %s", ToStr(hr).c_str());
+      return false;
+    }
+  }
+
   if(m_BindlessFeedback.FeedbackBuffer == NULL ||
      m_BindlessFeedback.FeedbackBuffer->GetDesc().Width < numSlots * sizeof(uint32_t))
   {
@@ -1357,7 +1541,8 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
     desc.MipLevels = 1;
     desc.SampleDesc.Count = 1;
     desc.SampleDesc.Quality = 0;
-    desc.Width = numSlots * sizeof(uint32_t);
+    desc.Width = AlignUp<uint32_t>(numSlots * sizeof(uint32_t), 1024) +
+                 sizeof(D3D12_QUERY_DATA_PIPELINE_STATISTICS);
 
     D3D12_HEAP_PROPERTIES heapProps;
     heapProps.Type = D3D12_HEAP_TYPE_DEFAULT;
@@ -1367,13 +1552,13 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
     heapProps.VisibleNodeMask = 1;
 
     HRESULT hr = m_pDevice->CreateCommittedResource(
-        &heapProps, D3D12_HEAP_FLAG_NONE, &desc, D3D12_RESOURCE_STATE_UNORDERED_ACCESS, NULL,
+        &heapProps, D3D12_HEAP_FLAG_NONE, &desc, D3D12_RESOURCE_STATE_COMMON, NULL,
         __uuidof(ID3D12Resource), (void **)&m_BindlessFeedback.FeedbackBuffer);
 
     if(m_BindlessFeedback.FeedbackBuffer == NULL || FAILED(hr))
     {
       RDCERR("Couldn't create feedback buffer with %u slots: %s", numSlots, ToStr(hr).c_str());
-      return;
+      return false;
     }
 
     m_BindlessFeedback.FeedbackBuffer->SetName(L"m_BindlessFeedback.FeedbackBuffer");
@@ -1396,7 +1581,9 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
 
     ID3D12GraphicsCommandList *list = m_pDevice->GetNewList();
     if(!list)
-      return;
+      return false;
+
+    GetDebugManager()->SetDescriptorHeaps(list, true, false);
 
     UINT zeroes[4] = {0, 0, 0, 0};
     list->ClearUnorderedAccessViewUint(GetDebugManager()->GetGPUHandle(FEEDBACK_CLEAR_UAV),
@@ -1417,7 +1604,7 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
     if(annotatedSig == NULL || FAILED(hr))
     {
       RDCERR("Couldn't create feedback modified root signature: %s", ToStr(hr).c_str());
-      return;
+      return false;
     }
   }
 
@@ -1431,7 +1618,7 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
     {
       SAFE_RELEASE(annotatedSig);
       RDCERR("Couldn't create feedback modified pipeline: %s", ToStr(hr).c_str());
-      return;
+      return false;
     }
   }
 
@@ -1456,7 +1643,20 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
         D3D12RenderState::SignatureElement(eRootUAV, GetResID(m_BindlessFeedback.FeedbackBuffer), 0);
   }
 
-  m_pDevice->ReplayLog(0, eventId, eReplay_OnlyDraw);
+  {
+    D3D12StatCallback cb(m_pDevice, m_BindlessFeedback.PipeStatsHeap);
+
+    m_pDevice->ReplayLog(0, eventId, eReplay_OnlyDraw);
+
+    ID3D12GraphicsCommandList *list = m_pDevice->GetNewList();
+    if(!list)
+      return true;
+
+    list->ResolveQueryData(m_BindlessFeedback.PipeStatsHeap, D3D12_QUERY_TYPE_PIPELINE_STATISTICS,
+                           0, 1, m_BindlessFeedback.FeedbackBuffer, numSlots * sizeof(uint32_t));
+
+    list->Close();
+  }
 
   m_pDevice->ExecuteLists();
   m_pDevice->FlushLists();
@@ -1477,6 +1677,9 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
   {
     uint32_t *slotsData = (uint32_t *)results.data();
 
+    D3D12_QUERY_DATA_PIPELINE_STATISTICS *pipelinestats =
+        (D3D12_QUERY_DATA_PIPELINE_STATISTICS *)(slotsData + numSlots);
+
     if(slotsData[0] == magicFeedbackValue)
     {
       result.valid = true;
@@ -1524,7 +1727,7 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
                 visMask = (uint32_t)ShaderStageMask::Geometry;
                 break;
               case D3D12_SHADER_VISIBILITY_PIXEL: visMask = uint32_t(ShaderStageMask::Pixel); break;
-              default: RDCERR("Unexpected shader visibility %d", p.ShaderVisibility); return;
+              default: RDCERR("Unexpected shader visibility %d", p.ShaderVisibility); return true;
             }
 
             // set the key type
@@ -1640,6 +1843,18 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
 
       std::sort(result.used.begin(), result.used.end());
     }
+    else if((pipelinestats->VSInvocations == 0 || !dynamicAccessPerStage[0]) &&
+            (pipelinestats->HSInvocations == 0 || !dynamicAccessPerStage[1]) &&
+            (pipelinestats->DSInvocations == 0 || !dynamicAccessPerStage[2]) &&
+            (pipelinestats->GSInvocations == 0 || !dynamicAccessPerStage[3]) &&
+            (pipelinestats->PSInvocations == 0 || !dynamicAccessPerStage[4]) &&
+            (pipelinestats->CSInvocations == 0 || !dynamicAccessPerStage[5]))
+    {
+      RDCDEBUG(
+          "No results from shader feedback but no dynamically-accessing shaders were actually "
+          "invoked");
+      result.valid = true;
+    }
     else
     {
       RDCERR("Didn't get valid feedback identifier. Expected %08x got %08x", magicFeedbackValue,
@@ -1647,8 +1862,7 @@ void D3D12Replay::FetchShaderFeedback(uint32_t eventId)
     }
   }
 
-  // replay from the start as we may have corrupted state while fetching the above feedback.
-  m_pDevice->ReplayLog(0, eventId, eReplay_Full);
+  return true;
 }
 
 void D3D12Replay::ClearFeedbackCache()
