@@ -97,6 +97,7 @@ public:
     EGLDisplay display = data.egl_dpy;
     EGLContext ctx = data.egl_ctx;
     EGLSurface draw = data.egl_wnd;
+    EGLConfig config = data.egl_cfg;
 
     if(ctx && draw)
     {
@@ -113,10 +114,18 @@ public:
 
       bool isYFlipped = IsYFlipped(display, draw);
 
+      int multiSamples;
+      EGL.GetConfigAttrib(display, config, EGL_SAMPLES, &multiSamples);
+      if(multiSamples != 1 && multiSamples != 2 && multiSamples != 4 && multiSamples != 8)
+      {
+        multiSamples = 1;
+      }
+
       params.width = width;
       params.height = height;
       params.isSRGB = isSRGB;
       params.isYFlipped = isYFlipped;
+      params.multiSamples = multiSamples;
     }
   }
 
@@ -438,9 +447,16 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglMakeCurrent_renderdoc_hooked(EGLDisplay di
 {
   if(RenderDoc::Inst().IsReplayApp())
   {
-    if(!EGL.MakeCurrent)
+    if(!EGL.MakeCurrent || !EGL.GetProcAddress)
       EGL.PopulateForReplay();
 
+    // populate GL function pointers now in case linked functions are called
+    if(EGL.GetProcAddress)
+    {
+      GL.PopulateWithCallback(
+          [](const char *funcName) -> void * { return (void *)EGL.GetProcAddress(funcName); });
+    }
+
     return EGL.MakeCurrent(display, draw, read, ctx);
   }
 
@@ -458,12 +474,13 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglMakeCurrent_renderdoc_hooked(EGLDisplay di
     {
       eglhook.contexts.insert(ctx);
 
-      FetchEnabledExtensions();
-
-      // see gl_emulated.cpp
-      GL.EmulateUnsupportedFunctions();
-      GL.EmulateRequiredExtensions();
-      GL.DriverForEmulation(&eglhook.driver);
+      if(FetchEnabledExtensions())
+      {
+        // see gl_emulated.cpp
+        GL.EmulateUnsupportedFunctions();
+        GL.EmulateRequiredExtensions();
+        GL.DriverForEmulation(&eglhook.driver);
+      }
     }
 
     SurfaceConfig cfg = eglhook.windows[draw];
