@@ -324,6 +324,7 @@ struct GLPlatform
   virtual bool CanCreateGLContext() = 0;
   virtual bool CanCreateGLESContext() = 0;
   virtual bool PopulateForReplay() = 0;
+  virtual void SetDriverType(RDCDriver api) = 0;
   virtual RDResult InitialiseAPI(GLWindowingData &replayContext, RDCDriver api, bool debug) = 0;
   virtual void *GetReplayFunction(const char *funcname) = 0;
 };
@@ -349,6 +350,7 @@ class GLDummyPlatform : public GLPlatform
   virtual bool CanCreateGLContext() { return true; }
   virtual bool CanCreateGLESContext() { return true; }
   virtual bool PopulateForReplay() { return true; }
+  virtual void SetDriverType(RDCDriver api) {}
   virtual RDResult InitialiseAPI(GLWindowingData &replayContext, RDCDriver api, bool debug)
   {
     return ResultCode::Succeeded;
@@ -501,12 +503,13 @@ struct GLMarkerRegion
 // - i.e. change state through here, and track dirty bits.
 struct GLPushPopState
 {
-  bool enableBits[8];
+  bool enableBits[9];
   GLenum ClipOrigin, ClipDepth;
   GLenum EquationRGB, EquationAlpha;
   GLenum SourceRGB, SourceAlpha;
   GLenum DestinationRGB, DestinationAlpha;
   GLenum PolygonMode;
+  GLdouble bounds[2];
   GLfloat Viewportf[4];
   GLint Viewport[4];
   GLenum ActiveTexture;
@@ -878,7 +881,7 @@ enum class ShaderType;
 // fills out the extension supported array and the version-specific checks above
 void DoVendorChecks(GLPlatform &platform, GLWindowingData context);
 void GetContextVersion(bool &ctxGLES, int &ctxVersion);
-void FetchEnabledExtensions();
+bool FetchEnabledExtensions();
 void GetGLSLVersions(ShaderType &shaderType, int &glslVersion, int &glslBaseVer, int &glslCSVer);
 
 GLuint CreateShader(GLenum shaderType, const rdcstr &src);
