@@ -425,6 +425,16 @@ void GLReplay::InitDebugData()
           "#extension GL_ARB_shader_bit_encoding : require\n";
   }
 
+  vs = GenerateGLSLShader(
+      GetEmbeddedResource(glsl_blit_vert), shaderType, glslBaseVer,
+      "#extension GL_ARB_separate_shader_objects : require\n#define FORCE_IO_LOCATION 1");
+
+  // used to combine with custom shaders.
+  // this has to have explicit locations on the output even though we don't normally use that,
+  // because GL doesn't have a fallback to match by name, and custom shaders are expected to have an
+  // explicit location on the input
+  DebugData.texDisplayVertexShader = CreateShader(eGL_VERTEX_SHADER, vs);
+
   vs = GenerateGLSLShader(GetEmbeddedResource(glsl_blit_vert), shaderType, glslBaseVer);
 
   DebugData.fixedcolFragShaderSPIRV = DebugData.quadoverdrawFragShaderSPIRV = 0;
@@ -453,9 +463,6 @@ void GLReplay::InitDebugData()
     }
   }
 
-  // used to combine with custom shaders.
-  DebugData.texDisplayVertexShader = CreateShader(eGL_VERTEX_SHADER, vs);
-
   for(int i = 0; i < 3; i++)
   {
     rdcstr defines = rdcstr("#define SHADER_BASETYPE ") + ToStr(i) + "\n";
@@ -1620,14 +1627,20 @@ void GLReplay::PickPixel(ResourceId texture, uint32_t x, uint32_t y, const Subre
   texDisplay.xOffset = -float(x << sub.mip);
   texDisplay.yOffset = -float(y << sub.mip);
 
+  auto &texDetails = m_pDriver->m_Textures[texDisplay.resourceId];
+
+  uint32_t mipWidth = RDCMAX(1U, (uint32_t)texDetails.width >> sub.mip);
+  uint32_t mipHeight = RDCMAX(1U, (uint32_t)texDetails.height >> sub.mip);
+
+  texDisplay.xOffset = -(float(x) / float(mipWidth)) * texDetails.width;
+  texDisplay.yOffset = -(float(y) / float(mipHeight)) * texDetails.height;
+
   RenderTextureInternal(texDisplay, eTexDisplay_MipShift);
 
   drv.glReadPixels(0, 0, 1, 1, eGL_RGBA, eGL_FLOAT, (void *)pixel);
 
   if(!HasExt[ARB_gpu_shader5])
   {
-    auto &texDetails = m_pDriver->m_Textures[texDisplay.resourceId];
-
     if(IsSIntFormat(texDetails.internalFormat))
     {
       int32_t casted[4] = {
@@ -1647,8 +1660,6 @@ void GLReplay::PickPixel(ResourceId texture, uint32_t x, uint32_t y, const Subre
   }
 
   {
-    auto &texDetails = m_pDriver->m_Textures[texture];
-
     // need to read stencil separately as GL can't read both depth and stencil
     // at the same time.
     if(texDetails.internalFormat == eGL_DEPTH24_STENCIL8 ||
