@@ -23,47 +23,97 @@
  ******************************************************************************/
 
 #include "metal_device.h"
+#include "metal_buffer.h"
 #include "metal_command_queue.h"
 #include "metal_function.h"
-#include "metal_helpers_bridge.h"
 #include "metal_library.h"
 #include "metal_manager.h"
 #include "metal_render_pipeline_state.h"
 #include "metal_texture.h"
 
 WrappedMTLDevice::WrappedMTLDevice(MTL::Device *realMTLDevice, ResourceId objId)
-    : WrappedMTLObject(realMTLDevice, objId, this, GetStateRef())
+    : WrappedMTLObject(realMTLDevice, objId, this, GetStateRef()), m_Capturer(*this)
 {
   AllocateObjCBridge(this);
   m_Device = this;
+
+  if(RenderDoc::Inst().IsReplayApp())
+  {
+  }
+  else
+  {
+    m_State = CaptureState::BackgroundCapturing;
+  }
+
+  m_SectionVersion = MetalInitParams::CurrentVersion;
+
   threadSerialiserTLSSlot = Threading::AllocateTLSSlot();
 
   m_ResourceManager = new MetalResourceManager(m_State, this);
+
+  if(!RenderDoc::Inst().IsReplayApp())
+  {
+    m_FrameCaptureRecord = GetResourceManager()->AddResourceRecord(ResourceIDGen::GetNewUniqueID());
+    m_FrameCaptureRecord->DataInSerialiser = false;
+    m_FrameCaptureRecord->Length = 0;
+    m_FrameCaptureRecord->InternalResource = true;
+  }
+  else
+  {
+    m_FrameCaptureRecord = NULL;
+
+    ResourceIDGen::SetReplayResourceIDs();
+  }
+
   RDCASSERT(m_Device == this);
   GetResourceManager()->AddCurrentResource(objId, this);
-}
 
-WrappedMTLDevice *WrappedMTLDevice::MTLCreateSystemDefaultDevice(MTL::Device *realMTLDevice)
-{
-  MTLFixupForMetalDriverAssert();
-  MTLHookObjcMethods();
-  ResourceId objId = ResourceIDGen::GetNewUniqueID();
-  WrappedMTLDevice *wrappedMTLDevice = new WrappedMTLDevice(realMTLDevice, objId);
+  if(IsCaptureMode(m_State))
+  {
+    Chunk *chunk = NULL;
 
-  return wrappedMTLDevice;
+    {
+      CACHE_THREAD_SERIALISER();
+
+      SCOPED_SERIALISE_CHUNK(MetalChunk::MTLCreateSystemDefaultDevice);
+      Serialise_MTLCreateSystemDefaultDevice(ser);
+      chunk = scope.Get();
+    }
+
+    MetalResourceRecord *record = GetResourceManager()->AddResourceRecord(this);
+    record->AddChunk(chunk);
+  }
+  else
+  {
+    // TODO: implement RD MTL replay
+  }
+
+  RenderDoc::Inst().AddDeviceFrameCapturer(this, &m_Capturer);
+
+  m_mtlCommandQueue = Unwrap(this)->newCommandQueue();
+  FirstFrame();
 }
 
 IMP WrappedMTLDevice::g_real_CAMetalLayer_nextDrawable;
 uint64_t WrappedMTLDevice::g_nextDrawableTLSSlot;
 
-MTL::Drawable *hooked_CAMetalLayer_nextDrawable(id self, SEL _cmd)
+CA::MetalDrawable *hooked_CAMetalLayer_nextDrawable(id self, SEL _cmd)
 {
+  CA::MetalLayer *mtlLayer = (CA::MetalLayer *)self;
+  MTL::Device *mtlDevice = mtlLayer->device();
+  WrappedMTLDevice *device = GetWrapped(mtlDevice);
+  RDCASSERT(object_getClass(mtlDevice) == objc_getClass("ObjCBridgeMTLDevice"));
+  device->RegisterMetalLayer(mtlLayer);
+  mtlLayer->setFramebufferOnly(false);
+
   RDCASSERTEQUAL(Threading::GetTLSValue(WrappedMTLDevice::g_nextDrawableTLSSlot), 0);
   Threading::SetTLSValue(WrappedMTLDevice::g_nextDrawableTLSSlot, (void *)(uintptr_t) true);
-  MTL::Drawable *drawable =
-      ((MTL::Drawable * (*)(id, SEL))WrappedMTLDevice::g_real_CAMetalLayer_nextDrawable)(self, _cmd);
+  CA::MetalDrawable *caMtlDrawable =
+      ((CA::MetalDrawable * (*)(id, SEL))WrappedMTLDevice::g_real_CAMetalLayer_nextDrawable)(self,
+                                                                                             _cmd);
+  device->RegisterDrawableInfo(caMtlDrawable);
   Threading::SetTLSValue(WrappedMTLDevice::g_nextDrawableTLSSlot, (void *)(uintptr_t) false);
-  return drawable;
+  return caMtlDrawable;
 }
 
 void WrappedMTLDevice::MTLHookObjcMethods()
@@ -98,6 +148,30 @@ void WrappedMTLDevice::MTLFixupForMetalDriverAssert()
 
 // Serialised MTLDevice APIs
 
+template <typename SerialiserType>
+bool WrappedMTLDevice::Serialise_MTLCreateSystemDefaultDevice(SerialiserType &ser)
+{
+  SERIALISE_ELEMENT_LOCAL(Device, GetResID(this)).TypedAs("MTLDevice"_lit);
+
+  SERIALISE_CHECK_READ_ERRORS();
+
+  if(IsReplayingAndReading())
+  {
+    // TODO: implement RD MTL replay
+  }
+  return true;
+}
+
+WrappedMTLDevice *WrappedMTLDevice::MTLCreateSystemDefaultDevice(MTL::Device *realMTLDevice)
+{
+  MTLFixupForMetalDriverAssert();
+  MTLHookObjcMethods();
+  ResourceId objId = ResourceIDGen::GetNewUniqueID();
+  WrappedMTLDevice *wrappedMTLDevice = new WrappedMTLDevice(realMTLDevice, objId);
+
+  return wrappedMTLDevice;
+}
+
 template <typename SerialiserType>
 bool WrappedMTLDevice::Serialise_newCommandQueue(SerialiserType &ser, WrappedMTLCommandQueue *queue)
 {
@@ -108,7 +182,13 @@ bool WrappedMTLDevice::Serialise_newCommandQueue(SerialiserType &ser, WrappedMTL
 
   if(IsReplayingAndReading())
   {
-    // TODO: implement RD MTL replay
+    MTL::CommandQueue *realMTLCommandQueue = Unwrap(this)->newCommandQueue();
+    WrappedMTLCommandQueue *wrappedMTLCommandQueue;
+    GetResourceManager()->WrapResource(realMTLCommandQueue, wrappedMTLCommandQueue);
+    GetResourceManager()->AddLiveResource(CommandQueue, wrappedMTLCommandQueue);
+
+    AddResource(CommandQueue, ResourceType::Queue, "Queue");
+    DerivedResource(this, CommandQueue);
   }
   return true;
 }
@@ -128,6 +208,7 @@ WrappedMTLCommandQueue *WrappedMTLDevice::newCommandQueue()
       Serialise_newCommandQueue(ser, wrappedMTLCommandQueue);
       chunk = scope.Get();
     }
+
     MetalResourceRecord *record = GetResourceManager()->AddResourceRecord(wrappedMTLCommandQueue);
     record->AddChunk(chunk);
   }
@@ -145,7 +226,19 @@ bool WrappedMTLDevice::Serialise_newDefaultLibrary(SerialiserType &ser, WrappedM
   bytebuf data;
   if(ser.IsWriting())
   {
-    ObjC::Get_defaultLibraryData(data);
+    NS::String *defaultType = NS::String::string("default", NS::UTF8StringEncoding);
+    NS::String *metallibExt = NS::String::string("metallib", NS::UTF8StringEncoding);
+    NS::Bundle *mainAppBundle = NS::Bundle::mainBundle();
+    NS::String *defaultLibaryPath = mainAppBundle->pathForResource(defaultType, metallibExt);
+    NS::Data *fileData = NS::Data::dataWithContentsOfFile(defaultLibaryPath);
+    dispatch_data_t dispatchData =
+        dispatch_data_create(fileData->bytes(), fileData->length(), dispatch_get_main_queue(),
+                             DISPATCH_DATA_DESTRUCTOR_DEFAULT);
+    NS::Data *nsData = (NS::Data *)dispatchData;
+    data.assign((byte *)nsData->bytes(), nsData->length());
+    dispatch_release(dispatchData);
+    defaultType->release();
+    metallibExt->release();
   }
 
   SERIALISE_ELEMENT_LOCAL(Device, this);
@@ -156,7 +249,17 @@ bool WrappedMTLDevice::Serialise_newDefaultLibrary(SerialiserType &ser, WrappedM
 
   if(IsReplayingAndReading())
   {
-    // TODO: implement RD MTL replay
+    dispatch_data_t dispatchData = dispatch_data_create(
+        data.data(), data.size(), dispatch_get_main_queue(), DISPATCH_DATA_DESTRUCTOR_DEFAULT);
+    NS::Error *error;
+    MTL::Library *realMTLLibrary = Unwrap(this)->newLibrary(dispatchData, &error);
+    dispatch_release(dispatchData);
+
+    WrappedMTLLibrary *wrappedMTLLibrary;
+    GetResourceManager()->WrapResource(realMTLLibrary, wrappedMTLLibrary);
+    GetResourceManager()->AddLiveResource(Library, wrappedMTLLibrary);
+    AddResource(Library, ResourceType::Pool, "Library");
+    DerivedResource(this, Library);
   }
   return true;
 }
@@ -177,9 +280,9 @@ WrappedMTLLibrary *WrappedMTLDevice::newDefaultLibrary()
       Serialise_newDefaultLibrary(ser, wrappedMTLLibrary);
       chunk = scope.Get();
     }
+
     MetalResourceRecord *record = GetResourceManager()->AddResourceRecord(wrappedMTLLibrary);
     record->AddChunk(chunk);
-    GetResourceManager()->MarkResourceFrameReferenced(id, eFrameRef_Read);
   }
   else
   {
@@ -203,7 +306,13 @@ bool WrappedMTLDevice::Serialise_newLibraryWithSource(SerialiserType &ser,
 
   if(IsReplayingAndReading())
   {
-    // TODO: implement RD MTL replay
+    NS::Error *compileErrors = NULL;
+    MTL::Library *realMTLLibrary = Unwrap(this)->newLibrary(source, options, &compileErrors);
+    WrappedMTLLibrary *wrappedMTLLibrary;
+    GetResourceManager()->WrapResource(realMTLLibrary, wrappedMTLLibrary);
+    GetResourceManager()->AddLiveResource(Library, wrappedMTLLibrary);
+    AddResource(Library, ResourceType::Pool, "Library");
+    DerivedResource(this, Library);
   }
   return true;
 }
@@ -225,9 +334,9 @@ WrappedMTLLibrary *WrappedMTLDevice::newLibraryWithSource(NS::String *source,
       Serialise_newLibraryWithSource(ser, wrappedMTLLibrary, source, options, error);
       chunk = scope.Get();
     }
+
     MetalResourceRecord *record = GetResourceManager()->AddResourceRecord(wrappedMTLLibrary);
     record->AddChunk(chunk);
-    GetResourceManager()->MarkResourceFrameReferenced(id, eFrameRef_Read);
   }
   else
   {
@@ -237,6 +346,58 @@ WrappedMTLLibrary *WrappedMTLDevice::newLibraryWithSource(NS::String *source,
   return wrappedMTLLibrary;
 }
 
+template <typename SerialiserType>
+bool WrappedMTLDevice::Serialise_newBufferWithBytes(SerialiserType &ser, WrappedMTLBuffer *buffer,
+                                                    const void *pointer, NS::UInteger length,
+                                                    MTL::ResourceOptions options)
+{
+  SERIALISE_ELEMENT_LOCAL(Buffer, GetResID(buffer)).TypedAs("MTLBuffer"_lit);
+  bytebuf initialData;
+  if(pointer)
+  {
+    initialData.assign((byte *)pointer, length);
+  }
+  SERIALISE_ELEMENT(initialData);
+  SERIALISE_ELEMENT(length).Important();
+  SERIALISE_ELEMENT(options);
+
+  SERIALISE_CHECK_READ_ERRORS();
+
+  // TODO: implement RD MTL replay
+  if(IsReplayingAndReading())
+  {
+    MTL::Buffer *realMTLBuffer;
+    if(initialData.isEmpty())
+    {
+      realMTLBuffer = Unwrap(this)->newBuffer(length, options);
+    }
+    else
+    {
+      RDCASSERT(initialData.size() == length);
+      realMTLBuffer = Unwrap(this)->newBuffer(initialData.data(), initialData.size(), options);
+    }
+    WrappedMTLBuffer *wrappedMTLBuffer;
+    GetResourceManager()->WrapResource(realMTLBuffer, wrappedMTLBuffer);
+    GetResourceManager()->AddLiveResource(Buffer, wrappedMTLBuffer);
+
+    AddResource(Buffer, ResourceType::Buffer, "Buffer");
+    DerivedResource(this, Buffer);
+  }
+  return true;
+}
+
+WrappedMTLBuffer *WrappedMTLDevice::newBufferWithBytes(const void *pointer, NS::UInteger length,
+                                                       MTL::ResourceOptions options)
+{
+  return Common_NewBuffer(true, pointer, length, options);
+}
+
+WrappedMTLBuffer *WrappedMTLDevice::newBufferWithLength(NS::UInteger length,
+                                                        MTL::ResourceOptions options)
+{
+  return Common_NewBuffer(false, NULL, length, options);
+}
+
 template <typename SerialiserType>
 bool WrappedMTLDevice::Serialise_newRenderPipelineStateWithDescriptor(
     SerialiserType &ser, WrappedMTLRenderPipelineState *pipelineState,
@@ -251,6 +412,18 @@ bool WrappedMTLDevice::Serialise_newRenderPipelineStateWithDescriptor(
   // TODO: implement RD MTL replay
   if(IsReplayingAndReading())
   {
+    ResourceId liveID;
+
+    MTL::RenderPipelineDescriptor *mtlDescriptor(descriptor);
+    MTL::RenderPipelineState *realMTLRenderPipelineState =
+        Unwrap(this)->newRenderPipelineState(mtlDescriptor, error);
+    mtlDescriptor->release();
+    WrappedMTLRenderPipelineState *wrappedMTLRenderPipelineState;
+    liveID = GetResourceManager()->WrapResource(realMTLRenderPipelineState,
+                                                wrappedMTLRenderPipelineState);
+    GetResourceManager()->AddLiveResource(RenderPipelineState, wrappedMTLRenderPipelineState);
+    AddResource(RenderPipelineState, ResourceType::PipelineState, "Pipeline State");
+    DerivedResource(this, RenderPipelineState);
   }
   return true;
 }
@@ -310,6 +483,20 @@ bool WrappedMTLDevice::Serialise_newTextureWithDescriptor(SerialiserType &ser,
 
   if(IsReplayingAndReading())
   {
+    // Ensure the created textures can be read by a shader
+    // Metal driver will treat TextureUsageUnknown as all options
+    if(descriptor.usage != MTL::TextureUsageUnknown)
+      descriptor.usage = (MTL::TextureUsage)(descriptor.usage | MTL::TextureUsageShaderRead);
+
+    MTL::TextureDescriptor *mtlDescriptor(descriptor);
+    MTL::Texture *realMTLTexture = Unwrap(this)->newTexture(mtlDescriptor);
+    mtlDescriptor->release();
+    WrappedMTLTexture *wrappedMTLTexture;
+    ResourceId liveID = GetResourceManager()->WrapResource(realMTLTexture, wrappedMTLTexture);
+    GetResourceManager()->AddLiveResource(Texture, wrappedMTLTexture);
+
+    AddResource(Texture, ResourceType::Texture, "Texture");
+    DerivedResource(this, Texture);
   }
   return true;
 }
@@ -468,6 +655,12 @@ WrappedMTLTexture *WrappedMTLDevice::Common_NewTexture(RDMTL::TextureDescriptor
 {
   MTL::Texture *realMTLTexture;
   MTL::TextureDescriptor *realDescriptor(descriptor);
+  // Ensure the created textures can be read by a shader
+  // Metal driver will treat TextureUsageUnknown as all options
+  MTL::TextureUsage usage = realDescriptor->usage();
+  if(usage != MTL::TextureUsageUnknown)
+    realDescriptor->setUsage((MTL::TextureUsage)(usage | MTL::TextureUsageShaderRead));
+
   SERIALISE_TIME_CALL(realMTLTexture = !ioSurfaceTexture ? Unwrap(this)->newTexture(realDescriptor)
                                                          : Unwrap(this)->newTexture(
                                                                realDescriptor, iosurface, plane));
@@ -492,14 +685,61 @@ WrappedMTLTexture *WrappedMTLDevice::Common_NewTexture(RDMTL::TextureDescriptor
     if(IsCaptureMode(m_State))
     {
       {
-        SCOPED_LOCK(m_PotentialBackBuffersLock);
-        m_PotentialBackBuffers.insert(wrappedMTLTexture);
+        SCOPED_LOCK(m_CapturePotentialBackBuffersLock);
+        m_CapturePotentialBackBuffers.insert(wrappedMTLTexture);
       }
     }
   }
   return wrappedMTLTexture;
 }
 
+WrappedMTLBuffer *WrappedMTLDevice::Common_NewBuffer(bool withBytes, const void *pointer,
+                                                     NS::UInteger length,
+                                                     MTL::ResourceOptions options)
+{
+  MTL::Buffer *realMTLBuffer;
+  SERIALISE_TIME_CALL(realMTLBuffer = withBytes ? Unwrap(this)->newBuffer(pointer, length, options)
+                                                : Unwrap(this)->newBuffer(length, options));
+
+  WrappedMTLBuffer *wrappedMTLBuffer;
+  ResourceId id = GetResourceManager()->WrapResource(realMTLBuffer, wrappedMTLBuffer);
+  if(IsCaptureMode(m_State))
+  {
+    Chunk *chunk = NULL;
+    {
+      CACHE_THREAD_SERIALISER();
+      SCOPED_SERIALISE_CHUNK(withBytes ? MetalChunk::MTLDevice_newBufferWithBytes
+                                       : MetalChunk::MTLDevice_newBufferWithLength);
+      Serialise_newBufferWithBytes(ser, wrappedMTLBuffer, pointer, length, options);
+      chunk = scope.Get();
+    }
+
+    MetalResourceRecord *record = GetResourceManager()->AddResourceRecord(wrappedMTLBuffer);
+    record->AddChunk(chunk);
+
+    MTL::StorageMode mode = realMTLBuffer->storageMode();
+    record->bufInfo = new MetalBufferInfo(mode);
+
+    // Create CPU side tracking info for CPU shared buffers
+    if(mode == MTL::StorageModeShared)
+    {
+      record->bufInfo->data = (byte *)realMTLBuffer->contents();
+      record->bufInfo->length = realMTLBuffer->length();
+    }
+    // Snapshot GPU only buffers
+    else if(mode == MTL::StorageModePrivate)
+    {
+      GetResourceManager()->MarkDirtyResource(id);
+    }
+  }
+  else
+  {
+    // TODO: implement RD MTL replay
+  }
+  return wrappedMTLBuffer;
+}
+
+INSTANTIATE_FUNCTION_SERIALISED(WrappedMTLDevice, bool, MTLCreateSystemDefaultDevice);
 INSTANTIATE_FUNCTION_WITH_RETURN_SERIALISED(WrappedMTLDevice, WrappedMTLCommandQueue *,
                                             newCommandQueue);
 INSTANTIATE_FUNCTION_WITH_RETURN_SERIALISED(WrappedMTLDevice, WrappedMTLLibrary *, newDefaultLibrary);
@@ -514,3 +754,6 @@ INSTANTIATE_FUNCTION_WITH_RETURN_SERIALISED(WrappedMTLDevice,
 INSTANTIATE_FUNCTION_WITH_RETURN_SERIALISED(WrappedMTLDevice, WrappedMTLTexture *,
                                             newTextureWithDescriptor,
                                             RDMTL::TextureDescriptor &descriptor);
+INSTANTIATE_FUNCTION_WITH_RETURN_SERIALISED(WrappedMTLDevice, WrappedMTLBuffer *,
+                                            newBufferWithBytes, const void *pointer,
+                                            NS::UInteger length, MTL::ResourceOptions options);
