@@ -458,6 +458,34 @@ D3D_PRIMITIVE_TOPOLOGY DXBCContainer::GetOutputTopology()
   return m_OutputTopology;
 }
 
+D3D_PRIMITIVE_TOPOLOGY DXBCContainer::GetOutputTopology(const void *ByteCode, size_t ByteCodeLength)
+{
+  const FileHeader *header = (const FileHeader *)ByteCode;
+
+  const byte *data = (const byte *)ByteCode;    // just for convenience
+
+  if(header->fourcc != FOURCC_DXBC)
+    return D3D_PRIMITIVE_TOPOLOGY_UNDEFINED;
+
+  if(header->fileLength != (uint32_t)ByteCodeLength)
+    return D3D_PRIMITIVE_TOPOLOGY_UNDEFINED;
+
+  const uint32_t *chunkOffsets = (const uint32_t *)(header + 1);    // right after the header
+
+  for(uint32_t chunkIdx = 0; chunkIdx < header->numChunks; chunkIdx++)
+  {
+    const uint32_t *fourcc = (const uint32_t *)(data + chunkOffsets[chunkIdx]);
+    const uint32_t *chunkSize = (const uint32_t *)(fourcc + 1);
+
+    const byte *chunkContents = (const byte *)(chunkSize + 1);
+
+    if(*fourcc == FOURCC_SHEX || *fourcc == FOURCC_SHDR)
+      return DXBCBytecode::Program::GetOutputTopology(chunkContents, *chunkSize);
+  }
+
+  return D3D_PRIMITIVE_TOPOLOGY_UNDEFINED;
+}
+
 const rdcstr &DXBCContainer::GetDisassembly()
 {
   if(m_Disassembly.empty())
@@ -565,13 +593,15 @@ void DXBCContainer::FillTraceLineInfo(ShaderDebugTrace &trace) const
 {
   if(m_DXBCByteCode)
   {
-    trace.lineInfo.resize(m_DXBCByteCode->GetNumInstructions());
+    trace.instInfo.resize(m_DXBCByteCode->GetNumInstructions());
     for(size_t i = 0; i < m_DXBCByteCode->GetNumInstructions(); i++)
     {
       const DXBCBytecode::Operation &op = m_DXBCByteCode->GetInstruction(i);
 
+      trace.instInfo[i].instruction = (uint32_t)i;
+
       if(m_DebugInfo)
-        m_DebugInfo->GetLineInfo(i, op.offset, trace.lineInfo[i]);
+        m_DebugInfo->GetLineInfo(i, op.offset, trace.instInfo[i].lineInfo);
 
       // we add some number of lines for the header we added with shader hash, debug name, etc on
       // top of what the bytecode disassembler did
@@ -587,38 +617,14 @@ void DXBCContainer::FillTraceLineInfo(ShaderDebugTrace &trace) const
         extraLines += (uint32_t)Bits::CountOnes((uint32_t)m_GlobalFlags) + 2;
 
       if(op.line > 0)
-        trace.lineInfo[i].disassemblyLine = extraLines + op.line;
+        trace.instInfo[i].lineInfo.disassemblyLine = extraLines + op.line;
+
+      if(m_DebugInfo)
+        m_DebugInfo->GetLocals(this, i, op.offset, trace.instInfo[i].sourceVars);
     }
   }
 }
 
-void DXBCContainer::FillStateInstructionInfo(ShaderDebugState &state) const
-{
-  uint32_t instruction = state.nextInstruction;
-
-  uintptr_t offset = 0;
-
-  state.sourceVars.clear();
-
-  if(m_DXBCByteCode)
-  {
-    if(instruction < m_DXBCByteCode->GetNumInstructions())
-      offset = m_DXBCByteCode->GetInstruction(instruction).offset;
-
-    if(m_DebugInfo)
-      m_DebugInfo->GetLocals(this, instruction, offset, state.sourceVars);
-  }
-
-  if(m_DebugInfo)
-  {
-    m_DebugInfo->GetCallstack(instruction, offset, state.callstack);
-  }
-  else
-  {
-    state.callstack.clear();
-  }
-}
-
 void DXBCContainer::StripChunk(bytebuf &ByteCode, uint32_t fourcc)
 {
   FileHeader *header = (FileHeader *)ByteCode.data();
