@@ -187,7 +187,6 @@ public:
 
   const rdcstr &GetDisassembly();
   void FillTraceLineInfo(ShaderDebugTrace &trace) const;
-  void FillStateInstructionInfo(ShaderDebugState &state) const;
 
   static void StripChunk(bytebuf &ByteCode, uint32_t fourcc);
   static void ReplaceChunk(bytebuf &ByteCode, uint32_t fourcc, const byte *replacement, size_t size);
@@ -220,6 +219,7 @@ public:
   static bool CheckForDebugInfo(const void *ByteCode, size_t ByteCodeLength);
   static bool CheckForDXIL(const void *ByteCode, size_t ByteCodeLength);
   static rdcstr GetDebugBinaryPath(const void *ByteCode, size_t ByteCodeLength);
+  static D3D_PRIMITIVE_TOPOLOGY GetOutputTopology(const void *ByteCode, size_t ByteCodeLength);
 
 private:
   void TryFetchSeparateDebugInfo(bytebuf &byteCode, const rdcstr &debugInfoPath);
