@@ -299,6 +299,8 @@ void MakeShaderReflection(DXBC::DXBCContainer *dxbc, ShaderReflection *refl,
 
     refl->debugInfo.files = dxbc->GetDebugInfo()->Files;
 
+    dxbc->GetDebugInfo()->GetLineInfo(~0U, ~0U, refl->debugInfo.entryLocation);
+
     rdcstr entry = dxbc->GetDebugInfo()->GetEntryFunction();
     if(entry.empty())
       entry = "main";
@@ -342,6 +344,12 @@ void MakeShaderReflection(DXBC::DXBCContainer *dxbc, ShaderReflection *refl,
   }
 
   refl->encoding = ShaderEncoding::DXBC;
+  refl->debugInfo.compiler = KnownShaderTool::fxc;
+  if(dxbc->GetDXILByteCode())
+  {
+    refl->encoding = ShaderEncoding::DXIL;
+    refl->debugInfo.compiler = KnownShaderTool::dxcDXIL;
+  }
   refl->rawBytes = dxbc->GetShaderBlob();
 
   refl->dispatchThreadsDimension[0] = dxbc->GetReflection()->DispatchThreadsDimension[0];
