@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2020 Baldur Karlsson
+ * Copyright (c) 2020-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,71 +24,27 @@
 
 #include "dxil_debuginfo.h"
 #include "common/formatting.h"
+#include "llvm_common.h"
 #include "llvm_decoder.h"
 
 namespace DXIL
 {
-enum class MetaDataRecord : uint32_t
-{
-  STRING_OLD = 1,
-  VALUE = 2,
-  NODE = 3,
-  NAME = 4,
-  DISTINCT_NODE = 5,
-  KIND = 6,
-  LOCATION = 7,
-  OLD_NODE = 8,
-  OLD_FN_NODE = 9,
-  NAMED_NODE = 10,
-  ATTACHMENT = 11,
-  GENERIC_DEBUG = 12,
-  SUBRANGE = 13,
-  ENUMERATOR = 14,
-  BASIC_TYPE = 15,
-  FILE = 16,
-  DERIVED_TYPE = 17,
-  COMPOSITE_TYPE = 18,
-  SUBROUTINE_TYPE = 19,
-  COMPILE_UNIT = 20,
-  SUBPROGRAM = 21,
-  LEXICAL_BLOCK = 22,
-  LEXICAL_BLOCK_FILE = 23,
-  NAMESPACE = 24,
-  TEMPLATE_TYPE = 25,
-  TEMPLATE_VALUE = 26,
-  GLOBAL_VAR = 27,
-  LOCAL_VAR = 28,
-  EXPRESSION = 29,
-  OBJC_PROPERTY = 30,
-  IMPORTED_ENTITY = 31,
-  MODULE = 32,
-  MACRO = 33,
-  MACRO_FILE = 34,
-  STRINGS = 35,
-  GLOBAL_DECL_ATTACHMENT = 36,
-  GLOBAL_VAR_EXPR = 37,
-  INDEX_OFFSET = 38,
-  INDEX = 39,
-  LABEL = 40,
-  COMMON_BLOCK = 44,
-};
-
 bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Metadata &meta)
 {
-  MetaDataRecord id = (MetaDataRecord)metaRecord.id;
+  LLVMBC::MetaDataRecord id = (LLVMBC::MetaDataRecord)metaRecord.id;
 
-  auto getNonNullMeta = [this](uint64_t id) { return &m_Metadata[size_t(id)]; };
-  auto getMeta = [this](uint64_t id) { return id ? &m_Metadata[size_t(id - 1)] : NULL; };
-  auto getMetaString = [this](uint64_t id) { return id ? &m_Metadata[size_t(id - 1)].str : NULL; };
+#define getNonNullMeta(id) &m_Metadata[size_t(id)]
+#define getMeta(id) (id ? &m_Metadata[size_t(id - 1)] : NULL)
+#define getMetaString(id) (id ? &m_Metadata[size_t(id - 1)].str : NULL)
 
-  if(id == MetaDataRecord::FILE)
+  if(id == LLVMBC::MetaDataRecord::FILE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
     meta.dwarf = new DIFile(getMeta(metaRecord.ops[1]), getMeta(metaRecord.ops[2]));
     meta.children = {getMeta(metaRecord.ops[1]), getMeta(metaRecord.ops[2])};
   }
-  else if(id == MetaDataRecord::COMPILE_UNIT)
+  else if(id == LLVMBC::MetaDataRecord::COMPILE_UNIT)
   {
     // should be at least 14 parameters
     RDCASSERT(metaRecord.ops.size() >= 14);
@@ -107,7 +63,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
                      getMeta(metaRecord.ops[10]), getMeta(metaRecord.ops[11]),
                      getMeta(metaRecord.ops[12]), getMeta(metaRecord.ops[13])};
   }
-  else if(id == MetaDataRecord::BASIC_TYPE)
+  else if(id == LLVMBC::MetaDataRecord::BASIC_TYPE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -115,7 +71,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
         new DIBasicType(DW_TAG(metaRecord.ops[1]), getMetaString(metaRecord.ops[2]),
                         metaRecord.ops[3], metaRecord.ops[4], DW_ENCODING(metaRecord.ops[5]));
   }
-  else if(id == MetaDataRecord::DERIVED_TYPE)
+  else if(id == LLVMBC::MetaDataRecord::DERIVED_TYPE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -128,7 +84,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
     meta.children = {getMeta(metaRecord.ops[3]), getMeta(metaRecord.ops[5]),
                      getMeta(metaRecord.ops[6]), getMeta(metaRecord.ops[11])};
   }
-  else if(id == MetaDataRecord::COMPOSITE_TYPE)
+  else if(id == LLVMBC::MetaDataRecord::COMPOSITE_TYPE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -143,7 +99,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
                      getMeta(metaRecord.ops[6]), getMeta(metaRecord.ops[11]),
                      getMeta(metaRecord.ops[14])};
   }
-  else if(id == MetaDataRecord::TEMPLATE_TYPE)
+  else if(id == LLVMBC::MetaDataRecord::TEMPLATE_TYPE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -152,7 +108,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
 
     meta.children = {getMeta(metaRecord.ops[2])};
   }
-  else if(id == MetaDataRecord::TEMPLATE_VALUE)
+  else if(id == LLVMBC::MetaDataRecord::TEMPLATE_VALUE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -162,7 +118,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
 
     meta.children = {getMeta(metaRecord.ops[3]), getMeta(metaRecord.ops[4])};
   }
-  else if(id == MetaDataRecord::SUBPROGRAM)
+  else if(id == LLVMBC::MetaDataRecord::SUBPROGRAM)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -179,7 +135,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
                      getMeta(metaRecord.ops[14]), getMeta(metaRecord.ops[15]),
                      getMeta(metaRecord.ops[16]), getMeta(metaRecord.ops[17])};
   }
-  else if(id == MetaDataRecord::SUBROUTINE_TYPE)
+  else if(id == LLVMBC::MetaDataRecord::SUBROUTINE_TYPE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -187,7 +143,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
 
     meta.children = {getMeta(metaRecord.ops[2])};
   }
-  else if(id == MetaDataRecord::GLOBAL_VAR)
+  else if(id == LLVMBC::MetaDataRecord::GLOBAL_VAR)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -210,7 +166,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
       RDCERR("Unsupported version of global variable metadata");
     }
   }
-  else if(id == MetaDataRecord::LOCATION)
+  else if(id == LLVMBC::MetaDataRecord::LOCATION)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -222,7 +178,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
 
     meta.children = {getNonNullMeta(metaRecord.ops[3]), getMeta(metaRecord.ops[4])};
   }
-  else if(id == MetaDataRecord::LOCAL_VAR)
+  else if(id == LLVMBC::MetaDataRecord::LOCAL_VAR)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -234,7 +190,7 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
     meta.children = {getMeta(metaRecord.ops[2]), getMeta(metaRecord.ops[4]),
                      getMeta(metaRecord.ops[6])};
   }
-  else if(id == MetaDataRecord::LEXICAL_BLOCK)
+  else if(id == LLVMBC::MetaDataRecord::LEXICAL_BLOCK)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
@@ -243,13 +199,13 @@ bool Program::ParseDebugMetaRecord(const LLVMBC::BlockOrRecord &metaRecord, Meta
 
     meta.children = {getMeta(metaRecord.ops[1]), getMeta(metaRecord.ops[2])};
   }
-  else if(id == MetaDataRecord::SUBRANGE)
+  else if(id == LLVMBC::MetaDataRecord::SUBRANGE)
   {
     meta.isDistinct = (metaRecord.ops[0] & 0x1);
 
     meta.dwarf = new DISubrange(metaRecord.ops[1], LLVMBC::BitReader::svbr(metaRecord.ops[2]));
   }
-  else if(id == MetaDataRecord::EXPRESSION)
+  else if(id == LLVMBC::MetaDataRecord::EXPRESSION)
   {
     DIExpression *expr = new DIExpression;
 
