@@ -143,12 +143,17 @@ struct StackFrame
   // allocated storage for locals
   rdcarray<ShaderVariable> locals;
 
+  // list of Ids we created, either variables/function parameters in this function, or IDs created
+  // in this function. When we return from this frame they will be emptied.
+  // This prevents a use-after-free with ShaderVariableChanges if we re-enter the same function
+  // and want to show the previous value of an id
+  rdcarray<Id> idsCreated;
+
   // as a hack for scoping without proper debug info, we track locals from their first use
   rdcarray<Id> localsUsed;
 
   // the thread's live list before the function was entered
   rdcarray<Id> live;
-  rdcarray<SourceVariableMapping> sourceVars;
 
   // the last block we were in and the current block, for OpPhis
   Id lastBlock, curBlock;
@@ -183,7 +188,7 @@ struct ThreadState
   ShaderVariable CalcDeriv(DerivDir dir, DerivType type, const rdcarray<ThreadState> &workgroup,
                            Id val);
 
-  void FillCallstack(ShaderDebugState &state);
+  void FillCallstack(rdcarray<Id> &funcs);
 
   bool Finished() const;
 
@@ -219,8 +224,6 @@ struct ThreadState
 
   std::map<Id, uint32_t> lastWrite;
 
-  rdcarray<SourceVariableMapping> sourceVars;
-
   // index in the pixel quad
   uint32_t workgroupIndex;
   bool helperInvocation;
@@ -250,6 +253,58 @@ enum class DebugScope
   Block,
 };
 
+struct TypeData
+{
+  VarType type = VarType::Unknown;
+  uint32_t vecSize = 0, matSize = 0;
+  bool colMajorMat = false;
+
+  Id baseType;
+  uint32_t arrayDimension = 0;
+  rdcarray<rdcpair<rdcstr, Id>> structMembers;
+};
+
+struct LocalMapping
+{
+  bool operator<(const LocalMapping &o) const
+  {
+    if(sourceVar != o.sourceVar)
+      return sourceVar < o.sourceVar;
+    if(indexes != o.indexes)
+      return indexes < o.indexes;
+    return debugVar < o.debugVar;
+  }
+
+  bool isSourceSupersetOf(const LocalMapping &o) const
+  {
+    // this mapping is a superset of the other if:
+
+    // it's the same source var
+    if(sourceVar != o.sourceVar)
+      return false;
+
+    // it contains the same or fewer indices
+    if(o.indexes.size() < indexes.size())
+      return false;
+
+    // the common prefix of indexes is identical
+    for(size_t i = 0; i < indexes.size(); i++)
+      if(indexes[i] != o.indexes[i])
+        return false;
+
+    // if all those conditions are true, we either map to the same index (indexes size is the same -
+    // likely case) or else we cover a whole sub-tree where the other only covers a leaf (other has
+    // more indexes - unlikely but possible)
+    return true;
+  }
+
+  uint32_t instIndex;
+  Id sourceVar;
+  Id debugVar;
+  bool isDeclare;
+  rdcarray<uint32_t> indexes;
+};
+
 struct ScopeData
 {
   DebugScope type;
@@ -262,6 +317,22 @@ struct ScopeData
   rdcstr name;
 
   rdcarray<Id> locals;
+
+  rdcarray<LocalMapping> localMappings;
+
+  bool HasAncestor(const ScopeData *check) const
+  {
+    const ScopeData *cur = this;
+
+    while(cur)
+    {
+      if(cur == check)
+        return true;
+      cur = cur->parent;
+    }
+
+    return false;
+  }
 };
 
 struct InlineData
@@ -274,11 +345,11 @@ struct LocalData
 {
   rdcstr name;
   ScopeData *scope;
-
-  Id curId;
+  TypeData *type;
 };
 
-typedef rdcpair<Id, Id> LocalMapping;
+Id ParseRawName(const rdcstr &name);
+rdcstr GetRawName(Id id);
 
 class Debugger : public Processor, public ShaderDebugger
 {
@@ -302,28 +373,26 @@ public:
   const Decorations &GetDecorations(Id typeId);
   bool IsDebugExtInstSet(Id id) const;
   bool HasDebugInfo() const { return m_DebugInfo.valid; }
-  rdcstr GetRawName(Id id) const;
+  bool InDebugScope(uint32_t inst) const;
   rdcstr GetHumanName(Id id);
-  void AddSourceVars(rdcarray<SourceVariableMapping> &sourceVars, const ShaderVariable &var, Id id);
   void AllocateVariable(Id id, Id typeId, ShaderVariable &outVar);
 
   ShaderVariable ReadFromPointer(const ShaderVariable &v) const;
   ShaderVariable GetPointerValue(const ShaderVariable &v) const;
   uint64_t GetPointerByteOffset(const ShaderVariable &ptr) const;
   DebugAPIWrapper::TextureType GetTextureType(const ShaderVariable &img) const;
-  ShaderVariable MakePointerVariable(Id id, const ShaderVariable *v, uint32_t scalar0 = ~0U,
-                                     uint32_t scalar1 = ~0U) const;
+  ShaderVariable MakePointerVariable(Id id, const ShaderVariable *v, uint8_t scalar0 = 0xff,
+                                     uint8_t scalar1 = 0xff) const;
   Id GetPointerBaseId(const ShaderVariable &v) const;
   bool IsOpaquePointer(const ShaderVariable &v) const;
   bool ArePointersAndEqual(const ShaderVariable &a, const ShaderVariable &b) const;
-  void WriteThroughPointer(const ShaderVariable &ptr, const ShaderVariable &val);
+  void WriteThroughPointer(ShaderVariable &ptr, const ShaderVariable &val);
   ShaderVariable MakeCompositePointer(const ShaderVariable &base, Id id, rdcarray<uint32_t> &indices);
 
   DebugAPIWrapper *GetAPIWrapper() { return apiWrapper; }
   uint32_t GetNumInstructions() { return (uint32_t)instructionOffsets.size(); }
   GlobalState GetGlobal() { return global; }
   const rdcarray<Id> &GetLiveGlobals() { return liveGlobals; }
-  const rdcarray<SourceVariableMapping> &GetGlobalSourceVars();
   ThreadState &GetActiveLane() { return workgroup[activeLaneIndex]; }
   const ThreadState &GetActiveLane() const { return workgroup[activeLaneIndex]; }
 private:
@@ -341,10 +410,12 @@ private:
                                            uint64_t, const rdcstr &)>
                             callback) const;
 
-  static Id ParseRawName(const rdcstr &name);
-
   void MakeSignatureNames(const rdcarray<SPIRVInterfaceAccess> &sigList, rdcarray<rdcstr> &sigNames);
 
+  void FillCallstack(ThreadState &thread, ShaderDebugState &state);
+  void FillDebugSourceVars(rdcarray<InstructionSourceInfo> &instInfo);
+  void FillDefaultSourceVars(rdcarray<InstructionSourceInfo> &instInfo);
+
   /////////////////////////////////////////////////////////
   // debug data
 
@@ -374,17 +445,16 @@ private:
   rdcarray<MemberName> memberNames;
   std::map<ShaderEntryPoint, Id> entryLookup;
 
-  SparseIdMap<size_t> idDeathOffset;
+  DenseIdMap<rdcpair<size_t, size_t>> idLiveRange;
 
   SparseIdMap<size_t> m_Files;
   LineColumnInfo m_CurLineCol;
-  std::map<size_t, LineColumnInfo> m_LineColInfo;
+  rdcarray<InstructionSourceInfo> m_InstInfo;
 
   SparseIdMap<uint32_t> labelInstruction;
 
   // the live mutable global variables, to initialise a stack frame's live list
   rdcarray<Id> liveGlobals;
-  rdcarray<SourceVariableMapping> globalSourceVars;
 
   struct Function
   {
@@ -406,11 +476,14 @@ private:
   {
     bool valid = false;
 
+    SparseIdMap<TypeData> types;
     SparseIdMap<ScopeData> scopes;
     SparseIdMap<InlineData> inlined;
     ScopeData *curScope = NULL;
     InlineData *curInline = NULL;
 
+    rdcarray<rdcpair<const ScopeData *, LocalMapping>> pendingMappings;
+
     rdcarray<Id> globals;
     rdcarray<Id> constants;
 
@@ -419,10 +492,15 @@ private:
     SparseIdMap<int32_t> sources;
     SparseIdMap<rdcstr> filenames;
 
+    SparseIdMap<Id> funcToDebugFunc;
+
     std::map<size_t, ScopeData *> lineScope;
     std::map<size_t, InlineData *> lineInline;
-    std::map<size_t, LocalMapping> localMappings;
+
+    rdcarray<LocalMapping> activeLocalMappings;
   } m_DebugInfo;
+
+  const ScopeData *GetScope(size_t offset) const;
 };
 
 // this does a 'safe' value assignment, by doing parallel depth-first iteration of both variables
