@@ -60,7 +60,8 @@ void FillSpecConstantVariables(ResourceId shader, const SPIRVPatchData &patchDat
 }
 
 void AddXFBAnnotations(const ShaderReflection &refl, const SPIRVPatchData &patchData,
-                       const char *entryName, rdcarray<uint32_t> &modSpirv, uint32_t &xfbStride)
+                       uint32_t rastStream, const char *entryName, rdcarray<uint32_t> &modSpirv,
+                       uint32_t &xfbStride)
 {
   rdcspv::Editor editor(modSpirv);
 
@@ -104,7 +105,8 @@ void AddXFBAnnotations(const ShaderReflection &refl, const SPIRVPatchData &patch
         rdcspv::OpDecorate decorate(it);
 
         if(decorate.decoration == rdcspv::Decoration::XfbBuffer ||
-           decorate.decoration == rdcspv::Decoration::XfbStride)
+           decorate.decoration == rdcspv::Decoration::XfbStride ||
+           decorate.decoration == rdcspv::Decoration::Stream)
         {
           editor.Remove(it);
         }
@@ -164,6 +166,10 @@ void AddXFBAnnotations(const ShaderReflection &refl, const SPIRVPatchData &patch
 
   for(size_t i = 0; i < outsig.size(); i++)
   {
+    // ignore anything from the non-rasterized stream
+    if(outsig[i].stream != rastStream)
+      continue;
+
     if(outpatch[i].isArraySubsequentElement)
     {
       // do not patch anything as we only patch the base array, but reserve space in the stride
@@ -190,6 +196,10 @@ void AddXFBAnnotations(const ShaderReflection &refl, const SPIRVPatchData &patch
 
   for(size_t i = 0; i < outpatch.size(); i++)
   {
+    // ignore anything from the non-rasterized stream
+    if(outsig[i].stream != rastStream)
+      continue;
+
     if(outpatch[i].ID && !outpatch[i].isArraySubsequentElement &&
        vars.find(outpatch[i].ID) == vars.end())
     {
@@ -200,6 +210,32 @@ void AddXFBAnnotations(const ShaderReflection &refl, const SPIRVPatchData &patch
       vars.insert(outpatch[i].ID);
     }
   }
+
+  // if the rasterized stream isn't 0 we need to patch any stream emits to emit 0 instead of
+  // rastStream, and drop any that used to go to other streams
+  if(hasXFB && rastStream != 0)
+  {
+    rdcspv::Id newStream = editor.AddConstantImmediate<uint32_t>(0U);
+
+    for(rdcspv::Iter it = editor.Begin(rdcspv::Section::Functions);
+        it < editor.End(rdcspv::Section::Functions); ++it)
+    {
+      if(it.opcode() == rdcspv::Op::EmitStreamVertex || it.opcode() == rdcspv::Op::EndStreamPrimitive)
+      {
+        // same format for both
+        rdcspv::OpEmitStreamVertex emit(it);
+
+        ShaderVariable stream = editor.EvaluateConstant(emit.stream, {});
+
+        // if this was emitting to our stream, still emit but to the new stream index. If it wasn't,
+        // drop it entirely
+        if(stream.value.u32v[0] == rastStream)
+          it.word(1) = newStream.value();
+        else
+          editor.Remove(it);
+      }
+    }
+  }
 }
 
 const int32_t INVALID_BIND = -INT_MAX;
@@ -397,7 +433,11 @@ StructSizes CalculateStructProps(uint32_t emptyStructSize, const ShaderConstant
 
       if(i + 1 == c.type.members.size())
       {
-        ret.scalarSize = AlignUp(m.byteOffset + member.scalarSize, ret.scalarAlign);
+        // scalar struct sizes are NOT padded up to the multiple of their alignment. It is allowed
+        // in scalar packing for an outside variable to 'sit in' the padding at the end of a struct
+        // due to alignment. This rule is more tight than even C packing, which normally has
+        // sizeof(struct) be a multiple of its alignment
+        ret.scalarSize = m.byteOffset + member.scalarSize;
         ret.baseSize = AlignUp(m.byteOffset + member.baseSize, ret.baseAlign);
         ret.extendedSize = AlignUp16(m.byteOffset + member.extendedSize);
       }
@@ -576,10 +616,28 @@ void Reflector::RegisterOp(Iter it)
       {
         sources.back().contents += strings[dbg.arg<Id>(0)];
       }
+      else if(dbg.inst == ShaderDbg::Function)
+      {
+        LineColumnInfo &info = debugFuncToLocation[dbg.result];
+
+        info.fileIndex = (int32_t)debugSources[dbg.arg<Id>(2)];
+        info.lineStart = info.lineEnd = EvaluateConstant(dbg.arg<Id>(3), {}).value.u32v[0];
+      }
+      else if(dbg.inst == ShaderDbg::FunctionDefinition)
+      {
+        funcToDebugFunc[dbg.arg<Id>(1)] = dbg.arg<Id>(0);
+      }
       else if(dbg.inst == ShaderDbg::CompilationUnit)
       {
         sources[debugSources[dbg.arg<Id>(2)]].lang =
             (SourceLanguage)EvaluateConstant(dbg.arg<Id>(3), {}).value.u32v[0];
+
+        compUnitToFileIndex[dbg.result] = debugSources[dbg.arg<Id>(2)];
+      }
+      else if(dbg.inst == ShaderDbg::EntryPoint)
+      {
+        debugFuncToBaseFile[dbg.arg<Id>(0)] = compUnitToFileIndex[dbg.arg<Id>(1)];
+        debugFuncToCmdLine[dbg.arg<Id>(0)] = strings[dbg.arg<Id>(3)];
       }
       else if(dbg.inst == ShaderDbg::GlobalVariable)
       {
@@ -814,9 +872,6 @@ void Reflector::MakeReflection(const GraphicsAPI sourceAPI, const ShaderStage st
         reflection.dispatchThreadsDimension[2] = 0;
   }
 
-  if(!cmdline.empty())
-    reflection.debugInfo.compileFlags.flags = {{"@cmdline", cmdline}};
-
   for(size_t i = 0; i < sources.size(); i++)
   {
     switch(sources[i].lang)
@@ -833,7 +888,38 @@ void Reflector::MakeReflection(const GraphicsAPI sourceAPI, const ShaderStage st
       case SourceLanguage::Max: break;
     }
 
-    reflection.debugInfo.files.push_back({sources[i].name, sources[i].contents});
+    if(!sources[i].name.empty() || !sources[i].contents.empty())
+      reflection.debugInfo.files.push_back({sources[i].name, sources[i].contents});
+  }
+
+  switch(m_Generator)
+  {
+    case Generator::GlslangReferenceFrontEnd:
+    case Generator::ShadercoverGlslang:
+      reflection.debugInfo.compiler = reflection.debugInfo.encoding == ShaderEncoding::HLSL
+                                          ? KnownShaderTool::glslangValidatorHLSL
+                                          : KnownShaderTool::glslangValidatorGLSL;
+      break;
+    case Generator::SPIRVToolsAssembler:
+      reflection.debugInfo.compiler = KnownShaderTool::spirv_as;
+      break;
+    case Generator::spiregg: reflection.debugInfo.compiler = KnownShaderTool::dxcSPIRV; break;
+    default: reflection.debugInfo.compiler = KnownShaderTool::Unknown; break;
+  }
+
+  if(!cmdline.empty())
+    reflection.debugInfo.compileFlags.flags = {{"@cmdline", cmdline}};
+
+  {
+    auto it = funcToDebugFunc.find(entry->id);
+    if(it != funcToDebugFunc.end())
+    {
+      reflection.debugInfo.entryLocation = debugFuncToLocation[it->second];
+      if(debugFuncToCmdLine.find(it->second) != debugFuncToCmdLine.end())
+        reflection.debugInfo.compileFlags.flags = {{"@cmdline", debugFuncToCmdLine[it->second]}};
+      if(debugFuncToBaseFile.find(it->second) != debugFuncToBaseFile.end())
+        reflection.debugInfo.editBaseFile = (int32_t)debugFuncToBaseFile[it->second];
+    }
   }
 
   PreprocessLineDirectives(reflection.debugInfo.files);
@@ -992,6 +1078,8 @@ void Reflector::MakeReflection(const GraphicsAPI sourceAPI, const ShaderStage st
         {
           const std::set<uint32_t> &usedchildren = usedStructChildren[global.id];
 
+          size_t oldSigSize = sigarray.size();
+
           for(uint32_t i = 0; i < (uint32_t)baseType.children.size(); i++)
           {
             // skip this member if it's in a builtin struct but has no builtin decoration
@@ -1019,6 +1107,18 @@ void Reflector::MakeReflection(const GraphicsAPI sourceAPI, const ShaderStage st
                                   baseType.children[i].decorations, sigarray, patchData, specInfo);
           }
 
+          // apply stream decoration from a parent struct into newly-added members
+          for(const DecorationAndParamData &d : decorations[global.id].others)
+          {
+            if(d.value == Decoration::Stream)
+            {
+              for(size_t idx = oldSigSize; idx < sigarray.size(); idx++)
+              {
+                sigarray[idx].stream = d.stream;
+              }
+            }
+          }
+
           // move on now, we've processed this global struct
           continue;
         }
@@ -1239,6 +1339,7 @@ void Reflector::MakeReflection(const GraphicsAPI sourceAPI, const ShaderStage st
             if(cblock.name.empty())
               cblock.name = StringFormat::Fmt("uniforms%u", global.id.value());
             cblock.bufferBacked = !pushConst;
+            cblock.inlineDataBytes = pushConst;
 
             MakeConstantBlockVariables(effectiveStorage, *varType, 0, 0, cblock.variables,
                                        pointerTypes, specInfo);
@@ -1289,6 +1390,7 @@ void Reflector::MakeReflection(const GraphicsAPI sourceAPI, const ShaderStage st
   {
     specblock.name = "Specialization Constants";
     specblock.bufferBacked = false;
+    specblock.inlineDataBytes = true;
     specblock.compileConstants = true;
     specblock.byteSize = 0;
 
@@ -1308,6 +1410,7 @@ void Reflector::MakeReflection(const GraphicsAPI sourceAPI, const ShaderStage st
   {
     globalsblock.name = "$Globals";
     globalsblock.bufferBacked = false;
+    globalsblock.inlineDataBytes = false;
     globalsblock.byteSize = (uint32_t)globalsblock.variables.size();
     globalsblock.bindPoint = (int)cblocks.size();
 
@@ -1547,8 +1650,11 @@ void Reflector::MakeConstantBlockVariables(rdcspv::StorageClass storage, const D
   cblock.resize(structType.children.size());
   for(size_t i = 0; i < structType.children.size(); i++)
   {
+    rdcstr name = structType.children[i].name;
+    if(name.empty())
+      name = StringFormat::Fmt("_child%zu", i);
     MakeConstantBlockVariable(cblock[i], pointerTypes, storage,
-                              dataTypes[structType.children[i].type], structType.children[i].name,
+                              dataTypes[structType.children[i].type], name,
                               structType.children[i].decorations, specInfo);
   }
 
@@ -1626,6 +1732,21 @@ void Reflector::MakeConstantBlockVariables(rdcspv::StorageClass storage, const D
   }
 }
 
+void Reflector::ApplyMatrixByteStride(const DataType &type, uint8_t matrixByteStride,
+                                      rdcarray<ShaderConstant> &members) const
+{
+  const DataType &inner = dataTypes[type.InnerType()];
+
+  for(ShaderConstant &m : members)
+  {
+    if(m.type.matrixByteStride == 0)
+      m.type.matrixByteStride = matrixByteStride;
+
+    if(inner.type == DataType::ArrayType)
+      ApplyMatrixByteStride(inner, matrixByteStride, m.type.members);
+  }
+}
+
 void Reflector::MakeConstantBlockVariable(ShaderConstant &outConst,
                                           SparseIdMap<uint16_t> &pointerTypes,
                                           rdcspv::StorageClass storage, const DataType &type,
@@ -1727,6 +1848,11 @@ void Reflector::MakeConstantBlockVariable(ShaderConstant &outConst,
 
     if(curType->type == DataType::ArrayType)
     {
+      // matrix byte stride is only applied on the root variable, so as we recurse down the type
+      // tree for multi-dimensional arrays of matrices we need to propagate down the stride
+      if(outConst.type.matrixByteStride != 0)
+        ApplyMatrixByteStride(*curType, outConst.type.matrixByteStride, outConst.type.members);
+
       outConst.type.name = type.name;
 
       // if the inner type is an array, it will be expanded in our members list. So don't also
@@ -1813,6 +1939,8 @@ void Reflector::AddSignatureParameter(const bool isInput, const ShaderStage stag
       // push the member-index access chain value
       patch.accessChain.push_back(0U);
 
+      size_t oldSigSize = sigarray.size();
+
       for(size_t c = 0; c < varType->children.size(); c++)
       {
         rdcstr childName = varName;
@@ -1833,6 +1961,18 @@ void Reflector::AddSignatureParameter(const bool isInput, const ShaderStage stag
         patch.accessChain.back()++;
       }
 
+      // apply stream decoration from a parent struct into newly-added members
+      for(const DecorationAndParamData &d : varDecorations.others)
+      {
+        if(d.value == Decoration::Stream)
+        {
+          for(size_t idx = oldSigSize; idx < sigarray.size(); idx++)
+          {
+            sigarray[idx].stream = d.stream;
+          }
+        }
+      }
+
       // pop the member-index access chain value
       patch.accessChain.pop_back();
 
@@ -1872,8 +2012,12 @@ void Reflector::AddSignatureParameter(const bool isInput, const ShaderStage stag
   sig.regChannelMask = sig.channelUsedMask = (1 << sig.compCount) - 1;
 
   for(const DecorationAndParamData &d : varDecorations.others)
+  {
     if(d.value == Decoration::Component)
       sig.regChannelMask <<= d.component;
+    if(d.value == Decoration::Stream)
+      sig.stream = d.stream;
+  }
 
   sig.channelUsedMask = sig.regChannelMask;
 
