@@ -107,6 +107,8 @@ private:
   rdcstr StringiseConstant(rdcspv::Id id) const;
   void CheckDebuggable(bool &debuggable, rdcstr &debugStatus) const;
 
+  void ApplyMatrixByteStride(const DataType &type, uint8_t matrixByteStride,
+                             rdcarray<ShaderConstant> &members) const;
   void MakeConstantBlockVariables(rdcspv::StorageClass storage, const DataType &structType,
                                   uint32_t arraySize, uint32_t arrayByteStride,
                                   rdcarray<ShaderConstant> &cblock,
@@ -127,6 +129,11 @@ private:
   DenseIdMap<rdcstr> strings;
   rdcarray<SourceFile> sources;
   SparseIdMap<size_t> debugSources;
+  SparseIdMap<size_t> compUnitToFileIndex;
+  SparseIdMap<size_t> debugFuncToBaseFile;
+  SparseIdMap<rdcstr> debugFuncToCmdLine;
+  SparseIdMap<LineColumnInfo> debugFuncToLocation;
+  SparseIdMap<Id> funcToDebugFunc;
 
   Id curBlock;
   std::set<Id> loopBlocks;
@@ -150,4 +157,5 @@ void FillSpecConstantVariables(ResourceId shader, const SPIRVPatchData &patchDat
 
 // common function used by any API that utilises SPIR-V
 void AddXFBAnnotations(const ShaderReflection &refl, const SPIRVPatchData &patchData,
-                       const char *entryName, rdcarray<uint32_t> &modSpirv, uint32_t &xfbStride);
+                       uint32_t rastStream, const char *entryName, rdcarray<uint32_t> &modSpirv,
+                       uint32_t &xfbStride);
