@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,12 +31,11 @@
 #include "vk_resources.h"
 
 #include <stdint.h>
-#include <vector>
 
 void CheckSubresourceRanges(const ImageState &state, bool expectAspectsSplit,
                             bool expectLevelsSplit, bool expectLayersSplit, bool expectDepthSplit)
 {
-  std::vector<VkImageAspectFlags> splitAspects;
+  rdcarray<VkImageAspectFlags> splitAspects;
   if(expectAspectsSplit)
   {
     for(auto it = ImageAspectFlagIter::begin(state.GetImageInfo().Aspects());
