@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,6 +22,8 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
+#define AMD_GPUPERFAPI_SKIP_VULKAN_INCLUDE 1
+
 #include <algorithm>
 #include <iterator>
 #include "vk_core.h"
@@ -29,7 +31,7 @@
 #include "vk_resources.h"
 
 #include "driver/ihv/amd/amd_counters.h"
-#include "driver/ihv/amd/official/GPUPerfAPI/Include/GPUPerfAPI-VK.h"
+#include "driver/ihv/amd/official/GPUPerfAPI/Include/gpu_perf_api_vk.h"
 #include "strings/string_utils.h"
 
 static uint32_t FromKHRCounter(GPUCounter counterID)
@@ -328,16 +330,16 @@ CounterDescription VulkanReplay::DescribeCounter(GPUCounter counterID)
   return desc;
 }
 
-struct VulkanAMDDrawCallback : public VulkanDrawcallCallback
+struct VulkanAMDActionCallback : public VulkanActionCallback
 {
-  VulkanAMDDrawCallback(WrappedVulkan *dev, VulkanReplay *rp, uint32_t &sampleIndex,
-                        rdcarray<uint32_t> &eventIDs)
+  VulkanAMDActionCallback(WrappedVulkan *dev, VulkanReplay *rp, uint32_t &sampleIndex,
+                          rdcarray<uint32_t> &eventIDs)
       : m_pDriver(dev), m_pReplay(rp), m_pSampleId(&sampleIndex), m_pEventIds(&eventIDs)
   {
-    m_pDriver->SetDrawcallCB(this);
+    m_pDriver->SetActionCB(this);
   }
 
-  virtual ~VulkanAMDDrawCallback() { m_pDriver->SetDrawcallCB(NULL); }
+  virtual ~VulkanAMDActionCallback() { m_pDriver->SetActionCB(NULL); }
   void PreDraw(uint32_t eid, VkCommandBuffer cmd) override
   {
     m_pEventIds->push_back(eid);
@@ -382,21 +384,21 @@ struct VulkanAMDDrawCallback : public VulkanDrawcallCallback
   void PreDispatch(uint32_t eid, VkCommandBuffer cmd) override { PreDraw(eid, cmd); }
   bool PostDispatch(uint32_t eid, VkCommandBuffer cmd) override { return PostDraw(eid, cmd); }
   void PostRedispatch(uint32_t eid, VkCommandBuffer cmd) override { PostRedraw(eid, cmd); }
-  void PreMisc(uint32_t eid, DrawFlags flags, VkCommandBuffer cmd) override
+  void PreMisc(uint32_t eid, ActionFlags flags, VkCommandBuffer cmd) override
   {
-    if(flags & DrawFlags::PassBoundary)
+    if(flags & ActionFlags::PassBoundary)
       return;
     PreDraw(eid, cmd);
   }
-  bool PostMisc(uint32_t eid, DrawFlags flags, VkCommandBuffer cmd) override
+  bool PostMisc(uint32_t eid, ActionFlags flags, VkCommandBuffer cmd) override
   {
-    if(flags & DrawFlags::PassBoundary)
+    if(flags & ActionFlags::PassBoundary)
       return false;
     return PostDraw(eid, cmd);
   }
-  void PostRemisc(uint32_t eid, DrawFlags flags, VkCommandBuffer cmd) override
+  void PostRemisc(uint32_t eid, ActionFlags flags, VkCommandBuffer cmd) override
   {
-    if(flags & DrawFlags::PassBoundary)
+    if(flags & ActionFlags::PassBoundary)
       return;
     PostRedraw(eid, cmd);
   }
@@ -406,6 +408,7 @@ struct VulkanAMDDrawCallback : public VulkanDrawcallCallback
     m_AliasEvents.push_back(make_rdcpair(primary, alias));
   }
   bool SplitSecondary() override { return false; }
+  bool ForceLoadRPs() override { return false; }
   void PreCmdExecute(uint32_t baseEid, uint32_t secondaryFirst, uint32_t secondaryLast,
                      VkCommandBuffer cmd) override
   {
@@ -432,7 +435,7 @@ void VulkanReplay::FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex,
 {
   uint32_t maxEID = m_pDriver->GetMaxEID();
 
-  m_pAMDDrawCallback = new VulkanAMDDrawCallback(m_pDriver, this, *sampleIndex, *eventIDs);
+  m_pAMDActionCallback = new VulkanAMDActionCallback(m_pDriver, this, *sampleIndex, *eventIDs);
 
   // replay the events to perform all the queries
   m_pDriver->ReplayLog(*eventStartID, maxEID, eReplay_Full);
@@ -440,8 +443,8 @@ void VulkanReplay::FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex,
 
 rdcarray<CounterResult> VulkanReplay::FetchCountersAMD(const rdcarray<GPUCounter> &counters)
 {
-  GPA_vkContextOpenInfo context = {Unwrap(m_pDriver->GetInstance()),
-                                   Unwrap(m_pDriver->GetPhysDev()), Unwrap(m_pDriver->GetDev())};
+  GpaVkContextOpenInfo context = {Unwrap(m_pDriver->GetInstance()), Unwrap(m_pDriver->GetPhysDev()),
+                                  Unwrap(m_pDriver->GetDev())};
 
   if(!m_pAMDCounters->BeginMeasurementMode(AMDCounters::ApiType::Vk, (void *)&context))
   {
@@ -488,13 +491,13 @@ rdcarray<CounterResult> VulkanReplay::FetchCountersAMD(const rdcarray<GPUCounter
   rdcarray<CounterResult> ret =
       m_pAMDCounters->GetCounterData(sessionID, sampleIndex, eventIDs, counters);
 
-  for(size_t i = 0; i < m_pAMDDrawCallback->m_AliasEvents.size(); i++)
+  for(size_t i = 0; i < m_pAMDActionCallback->m_AliasEvents.size(); i++)
   {
     for(size_t c = 0; c < counters.size(); c++)
     {
       CounterResult search;
       search.counter = counters[c];
-      search.eventId = m_pAMDDrawCallback->m_AliasEvents[i].first;
+      search.eventId = m_pAMDActionCallback->m_AliasEvents[i].first;
 
       // find the result we're aliasing
       int32_t idx = ret.indexOf(search);
@@ -502,7 +505,7 @@ rdcarray<CounterResult> VulkanReplay::FetchCountersAMD(const rdcarray<GPUCounter
       {
         // duplicate the result and append
         CounterResult aliased = ret[idx];
-        aliased.eventId = m_pAMDDrawCallback->m_AliasEvents[i].second;
+        aliased.eventId = m_pAMDActionCallback->m_AliasEvents[i].second;
         ret.push_back(aliased);
       }
       else
@@ -513,7 +516,7 @@ rdcarray<CounterResult> VulkanReplay::FetchCountersAMD(const rdcarray<GPUCounter
     }
   }
 
-  SAFE_DELETE(m_pAMDDrawCallback);
+  SAFE_DELETE(m_pAMDActionCallback);
 
   // sort so that the alias results appear in the right places
   std::sort(ret.begin(), ret.end());
@@ -523,14 +526,14 @@ rdcarray<CounterResult> VulkanReplay::FetchCountersAMD(const rdcarray<GPUCounter
   return ret;
 }
 
-struct VulkanKHRCallback : public VulkanDrawcallCallback
+struct VulkanKHRCallback : public VulkanActionCallback
 {
   VulkanKHRCallback(WrappedVulkan *vk, VulkanReplay *rp, VkQueryPool qp)
       : m_pDriver(vk), m_pReplay(rp), m_QueryPool(qp)
   {
-    m_pDriver->SetDrawcallCB(this);
+    m_pDriver->SetActionCB(this);
   }
-  ~VulkanKHRCallback() { m_pDriver->SetDrawcallCB(NULL); }
+  ~VulkanKHRCallback() { m_pDriver->SetActionCB(NULL); }
   void PreDraw(uint32_t eid, VkCommandBuffer cmd) override
   {
     ObjDisp(cmd)->CmdBeginQuery(Unwrap(cmd), m_QueryPool, (uint32_t)m_Results.size(), 0);
@@ -548,12 +551,12 @@ struct VulkanKHRCallback : public VulkanDrawcallCallback
   void PreDispatch(uint32_t eid, VkCommandBuffer cmd) override { PreDraw(eid, cmd); }
   bool PostDispatch(uint32_t eid, VkCommandBuffer cmd) override { return PostDraw(eid, cmd); }
   void PostRedispatch(uint32_t eid, VkCommandBuffer cmd) override { PostRedraw(eid, cmd); }
-  void PreMisc(uint32_t eid, DrawFlags flags, VkCommandBuffer cmd) override { PreDraw(eid, cmd); }
-  bool PostMisc(uint32_t eid, DrawFlags flags, VkCommandBuffer cmd) override
+  void PreMisc(uint32_t eid, ActionFlags flags, VkCommandBuffer cmd) override { PreDraw(eid, cmd); }
+  bool PostMisc(uint32_t eid, ActionFlags flags, VkCommandBuffer cmd) override
   {
     return PostDraw(eid, cmd);
   }
-  void PostRemisc(uint32_t eid, DrawFlags flags, VkCommandBuffer cmd) override
+  void PostRemisc(uint32_t eid, ActionFlags flags, VkCommandBuffer cmd) override
   {
     PostRedraw(eid, cmd);
   }
@@ -562,6 +565,7 @@ struct VulkanKHRCallback : public VulkanDrawcallCallback
     m_AliasEvents.push_back(std::make_pair(primary, alias));
   }
   bool SplitSecondary() override { return false; }
+  bool ForceLoadRPs() override { return false; }
   void PreCmdExecute(uint32_t baseEid, uint32_t secondaryFirst, uint32_t secondaryLast,
                      VkCommandBuffer cmd) override
   {
@@ -614,21 +618,24 @@ rdcarray<CounterResult> VulkanReplay::FetchCountersKHR(const rdcarray<GPUCounter
 
   VkQueryPool queryPool;
   vkr = ObjDisp(dev)->CreateQueryPool(Unwrap(dev), &queryPoolCreateInfo, NULL, &queryPool);
-  RDCASSERTEQUAL(vkr, VK_SUCCESS);
+  CheckVkResult(vkr);
 
   // Reset query pool
   VkCommandBuffer cmd = m_pDriver->GetNextCmd();
 
+  if(cmd == VK_NULL_HANDLE)
+    return {};
+
   VkCommandBufferBeginInfo beginInfo = {VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO, NULL,
                                         VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT};
 
   vkr = ObjDisp(dev)->BeginCommandBuffer(Unwrap(cmd), &beginInfo);
-  RDCASSERTEQUAL(vkr, VK_SUCCESS);
+  CheckVkResult(vkr);
 
   ObjDisp(dev)->CmdResetQueryPool(Unwrap(cmd), queryPool, 0, maxEID);
 
   vkr = ObjDisp(dev)->EndCommandBuffer(Unwrap(cmd));
-  RDCASSERTEQUAL(vkr, VK_SUCCESS);
+  CheckVkResult(vkr);
 
   m_pDriver->SubmitCmds();
 
@@ -654,7 +661,7 @@ rdcarray<CounterResult> VulkanReplay::FetchCountersKHR(const rdcarray<GPUCounter
       Unwrap(dev), queryPool, 0, (uint32_t)cb.m_Results.size(),
       sizeof(VkPerformanceCounterResultKHR) * perfResults.size(), &perfResults[0],
       sizeof(VkPerformanceCounterResultKHR) * counters.size(), VK_QUERY_RESULT_WAIT_BIT);
-  RDCASSERTEQUAL(vkr, VK_SUCCESS);
+  CheckVkResult(vkr);
 
   ObjDisp(dev)->DestroyQueryPool(Unwrap(dev), queryPool, NULL);
 
@@ -709,7 +716,7 @@ rdcarray<CounterResult> VulkanReplay::FetchCountersKHR(const rdcarray<GPUCounter
   return ret;
 }
 
-struct VulkanGPUTimerCallback : public VulkanDrawcallCallback
+struct VulkanGPUTimerCallback : public VulkanActionCallback
 {
   VulkanGPUTimerCallback(WrappedVulkan *vk, VulkanReplay *rp, VkQueryPool tsqp, VkQueryPool occqp,
                          VkQueryPool psqp)
@@ -719,9 +726,9 @@ struct VulkanGPUTimerCallback : public VulkanDrawcallCallback
         m_OcclusionQueryPool(occqp),
         m_PipeStatsQueryPool(psqp)
   {
-    m_pDriver->SetDrawcallCB(this);
+    m_pDriver->SetActionCB(this);
   }
-  ~VulkanGPUTimerCallback() { m_pDriver->SetDrawcallCB(NULL); }
+  ~VulkanGPUTimerCallback() { m_pDriver->SetActionCB(NULL); }
   void PreDraw(uint32_t eid, VkCommandBuffer cmd) override
   {
     if(m_OcclusionQueryPool != VK_NULL_HANDLE)
@@ -750,21 +757,21 @@ struct VulkanGPUTimerCallback : public VulkanDrawcallCallback
   void PreDispatch(uint32_t eid, VkCommandBuffer cmd) override { PreDraw(eid, cmd); }
   bool PostDispatch(uint32_t eid, VkCommandBuffer cmd) override { return PostDraw(eid, cmd); }
   void PostRedispatch(uint32_t eid, VkCommandBuffer cmd) override { PostRedraw(eid, cmd); }
-  void PreMisc(uint32_t eid, DrawFlags flags, VkCommandBuffer cmd) override
+  void PreMisc(uint32_t eid, ActionFlags flags, VkCommandBuffer cmd) override
   {
-    if(flags & DrawFlags::PassBoundary)
+    if(flags & ActionFlags::PassBoundary)
       return;
     PreDraw(eid, cmd);
   }
-  bool PostMisc(uint32_t eid, DrawFlags flags, VkCommandBuffer cmd) override
+  bool PostMisc(uint32_t eid, ActionFlags flags, VkCommandBuffer cmd) override
   {
-    if(flags & DrawFlags::PassBoundary)
+    if(flags & ActionFlags::PassBoundary)
       return false;
     return PostDraw(eid, cmd);
   }
-  void PostRemisc(uint32_t eid, DrawFlags flags, VkCommandBuffer cmd) override
+  void PostRemisc(uint32_t eid, ActionFlags flags, VkCommandBuffer cmd) override
   {
-    if(flags & DrawFlags::PassBoundary)
+    if(flags & ActionFlags::PassBoundary)
       return;
     PostRedraw(eid, cmd);
   }
@@ -773,6 +780,7 @@ struct VulkanGPUTimerCallback : public VulkanDrawcallCallback
     m_AliasEvents.push_back(make_rdcpair(primary, alias));
   }
   bool SplitSecondary() override { return false; }
+  bool ForceLoadRPs() override { return false; }
   void PreCmdExecute(uint32_t baseEid, uint32_t secondaryFirst, uint32_t secondaryLast,
                      VkCommandBuffer cmd) override
   {
@@ -857,7 +865,7 @@ rdcarray<CounterResult> VulkanReplay::FetchCounters(const rdcarray<GPUCounter> &
   VkQueryPool timeStampPool;
   VkResult vkr =
       ObjDisp(dev)->CreateQueryPool(Unwrap(dev), &timeStampPoolCreateInfo, NULL, &timeStampPool);
-  RDCASSERTEQUAL(vkr, VK_SUCCESS);
+  CheckVkResult(vkr);
 
   bool occlNeeded = false;
   bool statsNeeded = false;
@@ -886,23 +894,26 @@ rdcarray<CounterResult> VulkanReplay::FetchCounters(const rdcarray<GPUCounter> &
   if(availableFeatures.occlusionQueryPrecise && occlNeeded)
   {
     vkr = ObjDisp(dev)->CreateQueryPool(Unwrap(dev), &occlusionPoolCreateInfo, NULL, &occlusionPool);
-    RDCASSERTEQUAL(vkr, VK_SUCCESS);
+    CheckVkResult(vkr);
   }
 
   VkQueryPool pipeStatsPool = VK_NULL_HANDLE;
   if(availableFeatures.pipelineStatisticsQuery && statsNeeded)
   {
     vkr = ObjDisp(dev)->CreateQueryPool(Unwrap(dev), &pipeStatsPoolCreateInfo, NULL, &pipeStatsPool);
-    RDCASSERTEQUAL(vkr, VK_SUCCESS);
+    CheckVkResult(vkr);
   }
 
   VkCommandBuffer cmd = m_pDriver->GetNextCmd();
 
+  if(cmd == VK_NULL_HANDLE)
+    return {};
+
   VkCommandBufferBeginInfo beginInfo = {VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO, NULL,
                                         VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT};
 
   vkr = ObjDisp(dev)->BeginCommandBuffer(Unwrap(cmd), &beginInfo);
-  RDCASSERTEQUAL(vkr, VK_SUCCESS);
+  CheckVkResult(vkr);
 
   ObjDisp(dev)->CmdResetQueryPool(Unwrap(cmd), timeStampPool, 0, maxEID * 2);
   if(occlusionPool != VK_NULL_HANDLE)
@@ -911,7 +922,7 @@ rdcarray<CounterResult> VulkanReplay::FetchCounters(const rdcarray<GPUCounter> &
     ObjDisp(dev)->CmdResetQueryPool(Unwrap(cmd), pipeStatsPool, 0, maxEID);
 
   vkr = ObjDisp(dev)->EndCommandBuffer(Unwrap(cmd));
-  RDCASSERTEQUAL(vkr, VK_SUCCESS);
+  CheckVkResult(vkr);
 
 #if ENABLED(SINGLE_FLUSH_VALIDATE)
   m_pDriver->SubmitCmds();
@@ -929,7 +940,7 @@ rdcarray<CounterResult> VulkanReplay::FetchCounters(const rdcarray<GPUCounter> &
       Unwrap(dev), timeStampPool, 0, (uint32_t)m_TimeStampData.size(),
       sizeof(uint64_t) * m_TimeStampData.size(), &m_TimeStampData[0], sizeof(uint64_t),
       VK_QUERY_RESULT_64_BIT | VK_QUERY_RESULT_WAIT_BIT);
-  RDCASSERTEQUAL(vkr, VK_SUCCESS);
+  CheckVkResult(vkr);
 
   ObjDisp(dev)->DestroyQueryPool(Unwrap(dev), timeStampPool, NULL);
 
@@ -941,7 +952,7 @@ rdcarray<CounterResult> VulkanReplay::FetchCounters(const rdcarray<GPUCounter> &
         Unwrap(dev), occlusionPool, 0, (uint32_t)m_OcclusionData.size(),
         sizeof(uint64_t) * m_OcclusionData.size(), &m_OcclusionData[0], sizeof(uint64_t),
         VK_QUERY_RESULT_64_BIT | VK_QUERY_RESULT_WAIT_BIT);
-    RDCASSERTEQUAL(vkr, VK_SUCCESS);
+    CheckVkResult(vkr);
 
     ObjDisp(dev)->DestroyQueryPool(Unwrap(dev), occlusionPool, NULL);
   }
@@ -954,7 +965,7 @@ rdcarray<CounterResult> VulkanReplay::FetchCounters(const rdcarray<GPUCounter> &
         Unwrap(dev), pipeStatsPool, 0, (uint32_t)cb.m_Results.size(),
         sizeof(uint64_t) * m_PipeStatsData.size(), &m_PipeStatsData[0], sizeof(uint64_t) * 11,
         VK_QUERY_RESULT_64_BIT | VK_QUERY_RESULT_WAIT_BIT);
-    RDCASSERTEQUAL(vkr, VK_SUCCESS);
+    CheckVkResult(vkr);
 
     ObjDisp(dev)->DestroyQueryPool(Unwrap(dev), pipeStatsPool, NULL);
   }
