@@ -71,6 +71,37 @@ enum VulkanDynamicStateIndex
   VkDynamicLogicOpEXT,
   VkDynamicPrimRestart,
   VkDynamicColorWriteEXT,
+  VkDynamicTessDomainOriginEXT,
+  VkDynamicDepthClampEnableEXT,
+  VkDynamicPolygonModeEXT,
+  VkDynamicRasterizationSamplesEXT,
+  VkDynamicSampleMaskEXT,
+  VkDynamicAlphaToCoverageEXT,
+  VkDynamicAlphaToOneEXT,
+  VkDynamicLogicOpEnableEXT,
+  VkDynamicColorBlendEnableEXT,
+  VkDynamicColorBlendEquationEXT,
+  VkDynamicColorWriteMaskEXT,
+  VkDynamicRasterizationStreamEXT,
+  VkDynamicConservativeRastModeEXT,
+  VkDynamicOverstimationSizeEXT,
+  VkDynamicDepthClipEnableEXT,
+  VkDynamicSampleLocationsEnableEXT,
+  VkDynamicStateColorBlendAdvancedEXT,
+  VkDynamicProvokingVertexModeEXT,
+  VkDynamicLineRastModeEXT,
+  VkDynamicLineStippleEnableEXT,
+  VkDynamicDepthClipNegativeOneEXT,
+  VkDynamicViewportWScalingEXT,
+  VkDynamicViewportSwizzleEXT,
+  VkDynamicCoverageToColorEnableEXT,
+  VkDynamicCoverageToColorLocationEXT,
+  VkDynamicCoverageModulationModeEXT,
+  VkDynamicCoverageModulationTableEnableEXT,
+  VkDynamicCoverageModulationTableEXT,
+  VkDynamicShadingRateImageEnableEXT,
+  VkDynamicRepresentativeFragTestEXT,
+  VkDynamicCoverageReductionModeEXT,
   VkDynamicCount,
 };
 
@@ -91,7 +122,7 @@ struct DescSetLayout
     // some elements could be untouched. We set stageFlags to 0 so the UI ignores these
     // elements
     Binding()
-        : descriptorType(VK_DESCRIPTOR_TYPE_MAX_ENUM),
+        : layoutDescType(VK_DESCRIPTOR_TYPE_MAX_ENUM),
           elemOffset(0),
           descriptorCount(0),
           stageFlags(0),
@@ -101,7 +132,7 @@ struct DescSetLayout
     }
     // move the immutable sampler
     Binding(Binding &&b)
-        : descriptorType(b.descriptorType),
+        : layoutDescType(b.layoutDescType),
           elemOffset(b.elemOffset),
           descriptorCount(b.descriptorCount),
           stageFlags(b.stageFlags),
@@ -112,7 +143,7 @@ struct DescSetLayout
     }
     // Copy the immutable sampler
     Binding(const Binding &b)
-        : descriptorType(b.descriptorType),
+        : layoutDescType(b.layoutDescType),
           elemOffset(b.elemOffset),
           descriptorCount(b.descriptorCount),
           stageFlags(b.stageFlags),
@@ -130,7 +161,7 @@ struct DescSetLayout
       if(this == &b)
         return *this;
 
-      descriptorType = b.descriptorType;
+      layoutDescType = b.layoutDescType;
       elemOffset = b.elemOffset;
       descriptorCount = b.descriptorCount;
       stageFlags = b.stageFlags;
@@ -144,7 +175,9 @@ struct DescSetLayout
       return *this;
     }
     ~Binding() { SAFE_DELETE_ARRAY(immutableSampler); }
-    VkDescriptorType descriptorType;
+    // this is the layout-declared type, but since it may be mutable in most cases this is not used
+    // - only push descriptors use this
+    VkDescriptorType layoutDescType;
     uint32_t elemOffset;
     uint32_t descriptorCount;
     VkShaderStageFlags stageFlags : 31;
@@ -153,6 +186,9 @@ struct DescSetLayout
   };
   rdcarray<Binding> bindings;
 
+  // parallel array to bindings, with a bitmask of mutable types
+  rdcarray<uint64_t> mutableBitmasks;
+
   uint32_t totalElems;
   uint32_t dynamicCount;
   VkDescriptorSetLayoutCreateFlags flags;
@@ -163,10 +199,16 @@ struct DescSetLayout
   // the cummulative stageFlags for all bindings in this layout
   VkShaderStageFlags anyStageFlags;
 
-  bool operator==(const DescSetLayout &other) const;
-  bool operator!=(const DescSetLayout &other) const { return !(*this == other); }
+  bool isCompatible(const DescSetLayout &other) const;
 };
 
+bool IsValid(bool allowNULLDescriptors, const VkWriteDescriptorSet &write, uint32_t arrayElement);
+bool CreateDescriptorWritesForSlotData(WrappedVulkan *vk, rdcarray<VkWriteDescriptorSet> &writes,
+                                       VkDescriptorBufferInfo *&writeScratch,
+                                       const DescriptorSetSlot *slots, uint32_t descriptorCount,
+                                       VkDescriptorSet set, uint32_t dstBind,
+                                       const DescSetLayout::Binding &layoutBind);
+
 struct DescUpdateTemplateApplication
 {
   rdcarray<VkDescriptorBufferInfo> bufInfo;
@@ -429,6 +471,9 @@ struct VulkanCreationInfo
     // VkPipelineFragmentShadingRateCreateInfoKHR
     VkExtent2D shadingRate;
     VkFragmentShadingRateCombinerOpKHR shadingRateCombiners[2];
+
+    // VkPipelineViewportDepthClipControlCreateInfoEXT
+    bool negativeOneToOne;
   };
   std::unordered_map<ResourceId, Pipeline> m_Pipeline;
 
@@ -479,6 +524,7 @@ struct VulkanCreationInfo
       int32_t depthstencilResolveAttachment;
       int32_t fragmentDensityAttachment;
       int32_t shadingRateAttachment;
+      VkSampleCountFlagBits tileOnlyMSAASampleCount;
 
       rdcarray<VkImageLayout> inputLayouts;
       rdcarray<VkImageLayout> inputStencilLayouts;
@@ -491,6 +537,9 @@ struct VulkanCreationInfo
       VkExtent2D shadingRateTexelSize;
 
       rdcarray<uint32_t> multiviews;
+
+      bool feedbackLoop;
+      bool tileOnlyMSAAEnable;
     };
     rdcarray<Subpass> subpasses;
 
@@ -558,12 +607,14 @@ struct VulkanCreationInfo
   struct Buffer
   {
     void Init(VulkanResourceManager *resourceMan, VulkanCreationInfo &info,
-              const VkBufferCreateInfo *pCreateInfo);
+              const VkBufferCreateInfo *pCreateInfo, VkMemoryRequirements origMrq);
 
     VkBufferUsageFlags usage;
     uint64_t size;
     uint64_t gpuAddress;
     bool external;
+
+    VkMemoryRequirements mrq;
   };
   std::unordered_map<ResourceId, Buffer> m_Buffer;
 
@@ -582,7 +633,7 @@ struct VulkanCreationInfo
   struct Image
   {
     void Init(VulkanResourceManager *resourceMan, VulkanCreationInfo &info,
-              const VkImageCreateInfo *pCreateInfo);
+              const VkImageCreateInfo *pCreateInfo, VkMemoryRequirements origMrq);
 
     VkImageType type;
     VkFormat format;
@@ -594,6 +645,8 @@ struct VulkanCreationInfo
     bool external;
     bool cube;
     TextureCategory creationFlags;
+
+    VkMemoryRequirements mrq;
   };
   std::unordered_map<ResourceId, Image> m_Image;
 
@@ -689,6 +742,7 @@ struct VulkanCreationInfo
 
     uint32_t maxSets;
     rdcarray<VkDescriptorPoolSize> poolSizes;
+    rdcarray<uint64_t> mutableBitmasks;
 
     void CreateOverflow(VkDevice device, VulkanResourceManager *resourceMan);
 
