@@ -197,7 +197,7 @@ VKMeshDisplayPipelines VulkanDebugManager::CacheMeshDisplayPipelines(VkPipelineL
       false,
   };
 
-  ia.primitiveRestartEnable = primary.allowRestart && SupportsRestart(primary.topology);
+  ia.primitiveRestartEnable = primary.allowRestart;
 
   VkRect2D scissor = {{0, 0}, {16384, 16384}};
 
@@ -796,6 +796,7 @@ void VulkanReplay::RenderMesh(uint32_t eventId, const rdcarray<MeshFormat> &seco
   }
 
   MeshFormat helper;
+  helper.allowRestart = false;
   helper.indexByteStride = 2;
   helper.topology = Topology::LineList;
 
