@@ -643,16 +643,23 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_vertex_attribute_divisor)
   {
     vertexDivisor.pVertexBindingDivisors = vibindDivisors;
-    vertexDivisor.vertexBindingDivisorCount = vi.vertexBindingDescriptionCount;
+    vertexDivisor.vertexBindingDivisorCount = 0;
 
     for(uint32_t i = 0; i < vi.vertexBindingDescriptionCount; i++)
     {
-      vibindDivisors[i].binding = i;
-      vibindDivisors[i].divisor = pipeInfo.vertexBindings[i].instanceDivisor;
+      if(pipeInfo.vertexBindings[i].perInstance)
+      {
+        uint32_t instIdx = vertexDivisor.vertexBindingDivisorCount++;
+        vibindDivisors[instIdx].binding = pipeInfo.vertexBindings[i].vbufferBinding;
+        vibindDivisors[instIdx].divisor = pipeInfo.vertexBindings[i].instanceDivisor;
+      }
     }
 
-    vertexDivisor.pNext = vi.pNext;
-    vi.pNext = &vertexDivisor;
+    if(vertexDivisor.vertexBindingDivisorCount > 0)
+    {
+      vertexDivisor.pNext = vi.pNext;
+      vi.pNext = &vertexDivisor;
+    }
   }
 
   RDCASSERT(ARRAY_COUNT(viattr) >= pipeInfo.vertexAttrs.size());
@@ -661,6 +668,8 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   static VkPipelineInputAssemblyStateCreateInfo ia = {
       VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO};
 
+  ia.pNext = NULL;
+
   ia.topology = pipeInfo.topology;
   ia.primitiveRestartEnable = pipeInfo.primitiveRestartEnable;
 
@@ -685,10 +694,11 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
 
   static VkPipelineViewportStateCreateInfo vp = {
       VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO};
-
   static VkViewport views[32] = {};
   static VkRect2D scissors[32] = {};
 
+  vp.pNext = NULL;
+
   memcpy(views, &pipeInfo.viewports[0], pipeInfo.viewports.size() * sizeof(VkViewport));
 
   vp.pViewports = &views[0];
@@ -702,6 +712,18 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   RDCASSERT(ARRAY_COUNT(views) >= pipeInfo.viewports.size());
   RDCASSERT(ARRAY_COUNT(scissors) >= pipeInfo.scissors.size());
 
+  static VkPipelineViewportDepthClipControlCreateInfoEXT depthClipControl = {
+      VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_DEPTH_CLIP_CONTROL_CREATE_INFO_EXT,
+  };
+
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_depth_clip_control)
+  {
+    depthClipControl.negativeOneToOne = pipeInfo.negativeOneToOne;
+
+    depthClipControl.pNext = vp.pNext;
+    vp.pNext = &depthClipControl;
+  }
+
   static VkPipelineRasterizationStateCreateInfo rs = {
       VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO,
   };
@@ -789,9 +811,11 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
       VK_STRUCTURE_TYPE_PIPELINE_SAMPLE_LOCATIONS_STATE_CREATE_INFO_EXT,
   };
 
-  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_sample_locations)
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_sample_locations &&
+     pipeInfo.sampleLocations.enabled)
   {
     sampleLoc.sampleLocationsEnable = pipeInfo.sampleLocations.enabled;
+    sampleLoc.sampleLocationsInfo.sType = VK_STRUCTURE_TYPE_SAMPLE_LOCATIONS_INFO_EXT;
     sampleLoc.sampleLocationsInfo.sampleLocationGridSize = pipeInfo.sampleLocations.gridSize;
     sampleLoc.sampleLocationsInfo.sampleLocationsPerPixel = pipeInfo.rasterizationSamples;
     sampleLoc.sampleLocationsInfo.sampleLocationsCount =
@@ -805,6 +829,8 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   static VkPipelineDepthStencilStateCreateInfo ds = {
       VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO};
 
+  ds.pNext = NULL;
+
   ds.depthTestEnable = pipeInfo.depthTestEnable;
   ds.depthWriteEnable = pipeInfo.depthWriteEnable;
   ds.depthCompareOp = pipeInfo.depthCompareOp;
@@ -818,6 +844,8 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   static VkPipelineColorBlendStateCreateInfo cb = {
       VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO};
 
+  cb.pNext = NULL;
+
   cb.logicOpEnable = pipeInfo.logicOpEnable;
   cb.logicOp = pipeInfo.logicOp;
   memcpy(cb.blendConstants, pipeInfo.blendConst, sizeof(cb.blendConstants));
@@ -845,6 +873,8 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
 
   static VkPipelineDynamicStateCreateInfo dyn = {VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO};
 
+  dyn.pNext = NULL;
+
   dyn.dynamicStateCount = 0;
   dyn.pDynamicStates = dynSt;
 
@@ -963,6 +993,8 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   // never create derivatives
   ret.flags &= ~VK_PIPELINE_CREATE_DERIVATIVE_BIT;
 
+  ret.flags &= ~VK_PIPELINE_CREATE_LIBRARY_BIT_KHR;
+
   pipeCreateInfo = ret;
 }
 
