@@ -357,7 +357,8 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
 
   rdcspv::Id bufferAddressConst, ssboVar, uint32ptrtype;
 
-  if(usesMultiview && (stage == ShaderStage::Pixel || stage == ShaderStage::Vertex))
+  if(usesMultiview &&
+     (stage == ShaderStage::Pixel || stage == ShaderStage::Vertex || stage == ShaderStage::Geometry))
   {
     editor.AddCapability(rdcspv::Capability::MultiView);
     editor.AddExtension("SPV_KHR_multiview");
@@ -514,7 +515,8 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
     rdcarray<rdcspv::Id> idxs;
 
     auto fetchOrAddGlobalInput = [&editor, &idxs, &refl, &patchData, &locationGather, &newGlobals](
-        const char *name, ShaderBuiltin builtin, rdcspv::BuiltIn spvBuiltin, rdcspv::Id varType) {
+        const char *name, ShaderBuiltin builtin, rdcspv::BuiltIn spvBuiltin, rdcspv::Id varType,
+        bool integer) {
       rdcspv::Id ret;
 
       rdcspv::Id ptrType = editor.DeclareType(rdcspv::Pointer(varType, rdcspv::StorageClass::Input));
@@ -569,6 +571,10 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
             rdcspv::OpVariable(ptrType, editor.MakeId(), rdcspv::StorageClass::Input));
         editor.AddDecoration(rdcspv::OpDecorate(
             rdocGlobalVar, rdcspv::DecorationParam<rdcspv::Decoration::BuiltIn>(spvBuiltin)));
+        // Fragment shader inputs that are signed or unsigned integers, integer vectors, or any
+        // double-precision floating-point type must be decorated with Flat.
+        if(integer && refl.stage == ShaderStage::Pixel)
+          editor.AddDecoration(rdcspv::OpDecorate(rdocGlobalVar, rdcspv::Decoration::Flat));
 
         newGlobals.push_back(rdocGlobalVar);
 
@@ -587,7 +593,7 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
     {
       // the location for compute is easy, it's just the global invocation
       location = fetchOrAddGlobalInput("rdoc_invocation", ShaderBuiltin::DispatchThreadIndex,
-                                       rdcspv::BuiltIn::GlobalInvocationId, uvec3Type);
+                                       rdcspv::BuiltIn::GlobalInvocationId, uvec3Type, true);
     }
     else if(stage == ShaderStage::Vertex || stage == ShaderStage::Pixel)
     {
@@ -598,7 +604,7 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
       if(editor.HasCapability(rdcspv::Capability::MultiView))
       {
         view = fetchOrAddGlobalInput("rdoc_viewIndex", ShaderBuiltin::ViewportIndex,
-                                     rdcspv::BuiltIn::ViewIndex, uint32Type);
+                                     rdcspv::BuiltIn::ViewIndex, uint32Type, true);
       }
       else
       {
@@ -608,9 +614,9 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
       if(stage == ShaderStage::Vertex)
       {
         rdcspv::Id vtx = fetchOrAddGlobalInput("rdoc_vertexIndex", ShaderBuiltin::VertexIndex,
-                                               rdcspv::BuiltIn::VertexIndex, uint32Type);
+                                               rdcspv::BuiltIn::VertexIndex, uint32Type, true);
         rdcspv::Id inst = fetchOrAddGlobalInput("rdoc_instanceIndex", ShaderBuiltin::InstanceIndex,
-                                                rdcspv::BuiltIn::InstanceIndex, uint32Type);
+                                                rdcspv::BuiltIn::InstanceIndex, uint32Type, true);
 
         location = locationGather.add(
             rdcspv::OpCompositeConstruct(uvec3Type, editor.MakeId(), {vtx, inst, view}));
@@ -621,7 +627,7 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
         rdcspv::Id float4Type = editor.DeclareType(rdcspv::Vector(rdcspv::scalar<float>(), 4));
 
         rdcspv::Id coord = fetchOrAddGlobalInput("rdoc_fragCoord", ShaderBuiltin::Position,
-                                                 rdcspv::BuiltIn::FragCoord, float4Type);
+                                                 rdcspv::BuiltIn::FragCoord, float4Type, false);
 
         // grab just the xy
         coord = locationGather.add(
@@ -651,7 +657,7 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
              refl.inputSignature[i].systemValue == ShaderBuiltin::MSAASamplePosition)
           {
             samp = fetchOrAddGlobalInput("rdoc_sampleIndex", ShaderBuiltin::MSAASampleIndex,
-                                         rdcspv::BuiltIn::SampleId, uint32Type);
+                                         rdcspv::BuiltIn::SampleId, uint32Type, true);
           }
         }
 
@@ -672,7 +678,7 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
         if(usePrimitiveID)
         {
           prim = fetchOrAddGlobalInput("rdoc_primitiveIndex", ShaderBuiltin::PrimitiveIndex,
-                                       rdcspv::BuiltIn::PrimitiveId, uint32Type);
+                                       rdcspv::BuiltIn::PrimitiveId, uint32Type, true);
         }
         else
         {
@@ -683,6 +689,28 @@ void AnnotateShader(const ShaderReflection &refl, const SPIRVPatchData &patchDat
             rdcspv::OpCompositeConstruct(uvec3Type, editor.MakeId(), {coord, view, prim}));
       }
     }
+    else if(stage == ShaderStage::Geometry)
+    {
+      rdcspv::Id prim = fetchOrAddGlobalInput("rdoc_primitiveIndex", ShaderBuiltin::PrimitiveIndex,
+                                              rdcspv::BuiltIn::PrimitiveId, uint32Type, true);
+
+      rdcspv::Id view;
+
+      // only search for the view index is the multiview capability is declared, otherwise it's
+      // invalid and we just set 0. Valid for both Vertex and Pixel shaders
+      if(editor.HasCapability(rdcspv::Capability::MultiView))
+      {
+        view = fetchOrAddGlobalInput("rdoc_viewIndex", ShaderBuiltin::ViewportIndex,
+                                     rdcspv::BuiltIn::ViewIndex, uint32Type, true);
+      }
+      else
+      {
+        view = editor.AddConstantImmediate<uint32_t>(0U);
+      }
+
+      location = locationGather.add(
+          rdcspv::OpCompositeConstruct(uvec3Type, editor.MakeId(), {prim, view, zero}));
+    }
     else
     {
       RDCWARN("No identifier stored for %s stage", ToStr(stage).c_str());
@@ -1326,13 +1354,13 @@ void VulkanReplay::ClearFeedbackCache()
   m_BindlessFeedback.Usage.clear();
 }
 
-void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
+bool VulkanReplay::FetchShaderFeedback(uint32_t eventId)
 {
   if(m_BindlessFeedback.Usage.find(eventId) != m_BindlessFeedback.Usage.end())
-    return;
+    return false;
 
   if(!Vulkan_BindlessFeedback())
-    return;
+    return false;
 
   // create it here so we won't re-run any code if the event is re-selected. We'll mark it as valid
   // if it actually has any data in it later.
@@ -1354,14 +1382,21 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
   const ActionDescription *action = m_pDriver->GetAction(eventId);
 
   if(action == NULL || !(action->flags & (ActionFlags::Dispatch | ActionFlags::Drawcall)))
-    return;
+  {
+    // deliberately show no bindings as used for non-draws
+    result.valid = true;
+    return false;
+  }
 
   result.compute = bool(action->flags & ActionFlags::Dispatch);
 
   const VulkanStatePipeline &pipe = result.compute ? state.compute : state.graphics;
 
   if(pipe.pipeline == ResourceId())
-    return;
+  {
+    result.valid = true;
+    return false;
+  }
 
   const VulkanCreationInfo::Pipeline &pipeInfo = creationInfo.m_Pipeline[pipe.pipeline];
 
@@ -1441,12 +1476,12 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
         const DescSetLayout::Binding &bindData = layout.bindings[binding];
 
         // skip empty bindings
-        if(bindData.descriptorType == VK_DESCRIPTOR_TYPE_MAX_ENUM)
+        if(bindData.layoutDescType == VK_DESCRIPTOR_TYPE_MAX_ENUM)
           continue;
 
         // only process array bindings
         if(bindData.descriptorCount > 1 &&
-           bindData.descriptorType != VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK)
+           bindData.layoutDescType != VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK)
         {
           key.binding = (uint32_t)binding;
 
@@ -1465,7 +1500,9 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
 
   // if we don't have any array descriptors or printf's to feedback then just return now
   if(offsetMap.empty() && !usesPrintf)
-    return;
+  {
+    return false;
+  }
 
   if(!result.compute)
   {
@@ -1473,13 +1510,10 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
     if(!m_pDriver->GetDeviceEnabledFeatures().vertexPipelineStoresAndAtomics &&
        !m_pDriver->GetDeviceEnabledFeatures().fragmentStoresAndAtomics)
     {
-      return;
+      return false;
     }
   }
 
-  // replay from the start without this action, as we want a clean state to do printfs from.
-  m_pDriver->ReplayLog(0, eventId, eReplay_WithoutDraw);
-
   // we go through the driver for all these creations since they need to be properly
   // registered in order to be put in the partial replay state. Our patched shader is valid so we
   // don't need to replay after doing the feedback execute
@@ -1539,7 +1573,7 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
 
     // if the pool failed due to limits, it will be NULL so bail now
     if(descpool == VK_NULL_HANDLE)
-      return;
+      return false;
 
     // create pipeline layout with new descriptor set layouts
     {
@@ -1732,13 +1766,13 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
   }
 
   modifiedstate.subpassContents = VK_SUBPASS_CONTENTS_INLINE;
-  modifiedstate.dynamicRendering.flags &= VK_RENDERING_CONTENTS_SECONDARY_COMMAND_BUFFERS_BIT;
+  modifiedstate.dynamicRendering.flags &= ~VK_RENDERING_CONTENTS_SECONDARY_COMMAND_BUFFERS_BIT;
 
   {
     VkCommandBuffer cmd = m_pDriver->GetNextCmd();
 
     if(cmd == VK_NULL_HANDLE)
-      return;
+      return false;
 
     VkCommandBufferBeginInfo beginInfo = {VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO, NULL,
                                           VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT};
@@ -1900,6 +1934,11 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
           msg.location.vertex.instance = location[1] - action->instanceOffset;
           msg.location.vertex.view = location[2];
         }
+        else if(stage == ShaderStage::Geometry)
+        {
+          msg.location.geometry.primitive = location[0];
+          msg.location.geometry.view = location[1];
+        }
         else
         {
           msg.location.pixel.x = location[0] >> 16U;
@@ -1952,6 +1991,8 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
   for(size_t i = 0; i < ARRAY_COUNT(modules); i++)
     if(modules[i] != VK_NULL_HANDLE)
       m_pDriver->vkDestroyShaderModule(dev, modules[i], NULL);
+
+  return true;
 }
 
 #if ENABLED(ENABLE_UNIT_TESTS)
