@@ -29,6 +29,109 @@
 #include "vk_info.h"
 #include "vk_resources.h"
 
+namespace
+{
+struct RenderingInfoStructs
+{
+  VkRenderingInfo info;
+
+  rdcarray<VkRenderingAttachmentInfo> color;
+  VkRenderingAttachmentInfo depth;
+  VkRenderingAttachmentInfo stencil;
+
+  VkRenderingFragmentDensityMapAttachmentInfoEXT fragmentDensity;
+  VkRenderingFragmentShadingRateAttachmentInfoKHR shadingRate;
+  VkMultisampledRenderToSingleSampledInfoEXT tileOnlyMSAA;
+};
+
+void setupRenderingInfo(const VulkanRenderState::DynamicRendering &dynamicRendering,
+                        RenderingInfoStructs *structs, VkRenderingFlags flags,
+                        const VkRect2D &renderArea)
+{
+  VkRenderingInfo *info = &structs->info;
+
+  *info = {};
+  info->sType = VK_STRUCTURE_TYPE_RENDERING_INFO;
+  info->flags = flags;
+  info->layerCount = dynamicRendering.layerCount;
+  info->renderArea = renderArea;
+  info->viewMask = dynamicRendering.viewMask;
+
+  structs->depth = dynamicRendering.depth;
+  info->pDepthAttachment = &structs->depth;
+  if(structs->depth.imageLayout == VK_IMAGE_LAYOUT_UNDEFINED)
+    info->pDepthAttachment = NULL;
+  structs->stencil = dynamicRendering.stencil;
+  info->pStencilAttachment = &structs->stencil;
+  if(structs->stencil.imageLayout == VK_IMAGE_LAYOUT_UNDEFINED)
+    info->pStencilAttachment = NULL;
+
+  structs->color = dynamicRendering.color;
+
+  info->colorAttachmentCount = (uint32_t)structs->color.size();
+  info->pColorAttachments = structs->color.data();
+
+  // patch the load/store actions and unwrap
+  for(uint32_t i = 0; i < (uint32_t)structs->color.size() + 2; i++)
+  {
+    VkRenderingAttachmentInfo *att = (VkRenderingAttachmentInfo *)info->pColorAttachments + i;
+
+    if(i == info->colorAttachmentCount)
+      att = (VkRenderingAttachmentInfo *)info->pDepthAttachment;
+    else if(i == info->colorAttachmentCount + 1)
+      att = (VkRenderingAttachmentInfo *)info->pStencilAttachment;
+
+    if(!att)
+      continue;
+
+    if(att->loadOp != VK_ATTACHMENT_LOAD_OP_NONE_EXT)
+      att->loadOp = VK_ATTACHMENT_LOAD_OP_LOAD;
+
+    if(att->storeOp != VK_ATTACHMENT_STORE_OP_NONE)
+      att->storeOp = VK_ATTACHMENT_STORE_OP_STORE;
+
+    att->imageView = Unwrap(att->imageView);
+    att->resolveImageView = Unwrap(att->resolveImageView);
+  }
+
+  structs->fragmentDensity = {
+      VK_STRUCTURE_TYPE_RENDERING_FRAGMENT_DENSITY_MAP_ATTACHMENT_INFO_EXT, NULL,
+      dynamicRendering.fragmentDensityView, dynamicRendering.fragmentDensityLayout,
+  };
+
+  if(dynamicRendering.fragmentDensityView != VK_NULL_HANDLE)
+  {
+    structs->fragmentDensity.pNext = info->pNext;
+    info->pNext = &structs->fragmentDensity;
+  }
+
+  structs->shadingRate = {
+      VK_STRUCTURE_TYPE_RENDERING_FRAGMENT_SHADING_RATE_ATTACHMENT_INFO_KHR,
+      NULL,
+      dynamicRendering.shadingRateView,
+      dynamicRendering.shadingRateLayout,
+      dynamicRendering.shadingRateTexelSize,
+  };
+
+  if(dynamicRendering.shadingRateView != VK_NULL_HANDLE)
+  {
+    structs->shadingRate.pNext = info->pNext;
+    info->pNext = &structs->shadingRate;
+  }
+
+  structs->tileOnlyMSAA = {
+      VK_STRUCTURE_TYPE_MULTISAMPLED_RENDER_TO_SINGLE_SAMPLED_INFO_EXT, NULL,
+      dynamicRendering.tileOnlyMSAAEnable, dynamicRendering.tileOnlyMSAASampleCount,
+  };
+
+  if(dynamicRendering.tileOnlyMSAAEnable)
+  {
+    structs->tileOnlyMSAA.pNext = info->pNext;
+    info->pNext = &structs->tileOnlyMSAA;
+  }
+}
+}    // namespace
+
 VulkanRenderState::VulkanRenderState()
 {
   RDCEraseEl(ibuffer);
@@ -39,65 +142,24 @@ void VulkanRenderState::BeginRenderPassAndApplyState(WrappedVulkan *vk, VkComman
 {
   if(dynamicRendering.active)
   {
-    VkRenderingInfo info = {};
-    info.sType = VK_STRUCTURE_TYPE_RENDERING_INFO;
-
     // for action callbacks that want to stop the renderpass, do something, then start it with
     // original state, we need to preserve the suspending flag instead of removing it. For other
     // uses, we remove both flags as we're just doing a manual start/stop and we're not in a
     // suspended pass
+    VkRenderingFlags flags = dynamicRendering.flags;
     if(obeySuspending)
     {
-      info.flags = dynamicRendering.flags & ~VK_RENDERING_RESUMING_BIT;
+      flags &= ~VK_RENDERING_RESUMING_BIT;
     }
     else
     {
-      info.flags =
-          dynamicRendering.flags & ~(VK_RENDERING_RESUMING_BIT | VK_RENDERING_SUSPENDING_BIT);
+      flags &= ~(VK_RENDERING_RESUMING_BIT | VK_RENDERING_SUSPENDING_BIT);
     }
 
-    info.layerCount = dynamicRendering.layerCount;
-    info.renderArea = renderArea;
-    info.viewMask = dynamicRendering.viewMask;
+    RenderingInfoStructs structs;
+    setupRenderingInfo(dynamicRendering, &structs, flags, renderArea);
 
-    VkRenderingAttachmentInfo depth = dynamicRendering.depth;
-    info.pDepthAttachment = &depth;
-    if(depth.imageLayout == VK_IMAGE_LAYOUT_UNDEFINED)
-      info.pDepthAttachment = NULL;
-    VkRenderingAttachmentInfo stencil = dynamicRendering.depth;
-    info.pStencilAttachment = &stencil;
-    if(stencil.imageLayout == VK_IMAGE_LAYOUT_UNDEFINED)
-      info.pStencilAttachment = NULL;
-
-    rdcarray<VkRenderingAttachmentInfo> color = dynamicRendering.color;
-
-    info.colorAttachmentCount = (uint32_t)color.size();
-    info.pColorAttachments = color.data();
-
-    // patch the load/store actions and unwrap
-    for(uint32_t i = 0; i < (uint32_t)color.size() + 2; i++)
-    {
-      VkRenderingAttachmentInfo *att = (VkRenderingAttachmentInfo *)info.pColorAttachments + i;
-
-      if(i == info.colorAttachmentCount)
-        att = (VkRenderingAttachmentInfo *)info.pDepthAttachment;
-      else if(i == info.colorAttachmentCount + 1)
-        att = (VkRenderingAttachmentInfo *)info.pStencilAttachment;
-
-      if(!att)
-        continue;
-
-      if(att->loadOp != VK_ATTACHMENT_LOAD_OP_NONE_EXT)
-        att->loadOp = VK_ATTACHMENT_LOAD_OP_LOAD;
-
-      if(att->storeOp != VK_ATTACHMENT_STORE_OP_NONE)
-        att->storeOp = VK_ATTACHMENT_STORE_OP_STORE;
-
-      att->imageView = Unwrap(att->imageView);
-      att->resolveImageView = Unwrap(att->resolveImageView);
-    }
-
-    ObjDisp(cmd)->CmdBeginRendering(Unwrap(cmd), &info);
+    ObjDisp(cmd)->CmdBeginRendering(Unwrap(cmd), &structs.info);
   }
   else
   {
@@ -178,55 +240,14 @@ void VulkanRenderState::FinishSuspendedRenderPass(VkCommandBuffer cmd)
 {
   if(dynamicRendering.active && dynamicRendering.suspended)
   {
-    VkRenderingInfo info = {};
-    info.sType = VK_STRUCTURE_TYPE_RENDERING_INFO;
-
     // still resume the existing pass, but don't suspend again after that
-    info.flags = dynamicRendering.flags & ~VK_RENDERING_SUSPENDING_BIT;
+    const VkRenderingFlags flags = dynamicRendering.flags & ~VK_RENDERING_SUSPENDING_BIT;
 
-    info.layerCount = dynamicRendering.layerCount;
-    info.renderArea = renderArea;
-    info.viewMask = dynamicRendering.viewMask;
-
-    VkRenderingAttachmentInfo depth = dynamicRendering.depth;
-    info.pDepthAttachment = &depth;
-    if(depth.imageLayout == VK_IMAGE_LAYOUT_UNDEFINED)
-      info.pDepthAttachment = NULL;
-    VkRenderingAttachmentInfo stencil = dynamicRendering.depth;
-    info.pStencilAttachment = &stencil;
-    if(stencil.imageLayout == VK_IMAGE_LAYOUT_UNDEFINED)
-      info.pStencilAttachment = NULL;
-
-    rdcarray<VkRenderingAttachmentInfo> color = dynamicRendering.color;
-
-    info.colorAttachmentCount = (uint32_t)color.size();
-    info.pColorAttachments = color.data();
-
-    // patch the load/store actions and unwrap
-    for(uint32_t i = 0; i < (uint32_t)color.size() + 2; i++)
-    {
-      VkRenderingAttachmentInfo *att = (VkRenderingAttachmentInfo *)info.pColorAttachments + i;
-
-      if(i == info.colorAttachmentCount)
-        att = (VkRenderingAttachmentInfo *)info.pDepthAttachment;
-      else if(i == info.colorAttachmentCount + 1)
-        att = (VkRenderingAttachmentInfo *)info.pStencilAttachment;
-
-      if(!att)
-        continue;
-
-      if(att->loadOp != VK_ATTACHMENT_LOAD_OP_NONE_EXT)
-        att->loadOp = VK_ATTACHMENT_LOAD_OP_LOAD;
-
-      if(att->storeOp != VK_ATTACHMENT_STORE_OP_NONE)
-        att->storeOp = VK_ATTACHMENT_STORE_OP_STORE;
-
-      att->imageView = Unwrap(att->imageView);
-      att->resolveImageView = Unwrap(att->resolveImageView);
-    }
+    RenderingInfoStructs structs;
+    setupRenderingInfo(dynamicRendering, &structs, flags, renderArea);
 
     // do nothing, just resume and then end without suspending
-    ObjDisp(cmd)->CmdBeginRendering(Unwrap(cmd), &info);
+    ObjDisp(cmd)->CmdBeginRendering(Unwrap(cmd), &structs.info);
     ObjDisp(cmd)->CmdEndRendering(Unwrap(cmd));
   }
 }
@@ -384,15 +405,21 @@ void VulkanRenderState::BindPipeline(WrappedVulkan *vk, VkCommandBuffer cmd,
     {
       if(dynamicStates[VkDynamicDepthBiasEnable])
         ObjDisp(cmd)->CmdSetDepthBiasEnableEXT(Unwrap(cmd), depthBiasEnable);
-      if(dynamicStates[VkDynamicLogicOpEXT])
-        ObjDisp(cmd)->CmdSetLogicOpEXT(Unwrap(cmd), logicOp);
-      if(dynamicStates[VkDynamicControlPointsEXT])
-        ObjDisp(cmd)->CmdSetPatchControlPointsEXT(Unwrap(cmd), patchControlPoints);
       if(dynamicStates[VkDynamicPrimRestart])
         ObjDisp(cmd)->CmdSetPrimitiveRestartEnableEXT(Unwrap(cmd), primRestartEnable);
       if(dynamicStates[VkDynamicRastDiscard])
         ObjDisp(cmd)->CmdSetRasterizerDiscardEnableEXT(Unwrap(cmd), rastDiscardEnable);
     }
+    if(vk->ExtendedDynamicState2Logic())
+    {
+      if(dynamicStates[VkDynamicLogicOpEXT])
+        ObjDisp(cmd)->CmdSetLogicOpEXT(Unwrap(cmd), logicOp);
+    }
+    if(vk->ExtendedDynamicState2CPs())
+    {
+      if(dynamicStates[VkDynamicControlPointsEXT])
+        ObjDisp(cmd)->CmdSetPatchControlPointsEXT(Unwrap(cmd), patchControlPoints);
+    }
 
     if(dynamicStates[VkDynamicLineWidth])
       ObjDisp(cmd)->CmdSetLineWidth(Unwrap(cmd), lineWidth);
@@ -611,7 +638,7 @@ void VulkanRenderState::BindDescriptorSetsForPipeline(WrappedVulkan *vk, VkComma
         const DescSetLayout &createdDescLayout =
             vk->GetDebugManager()->GetDescSetLayout(createdDescSetLayoutId);
 
-        if(descLayout != createdDescLayout)
+        if(!descLayout.isCompatible(createdDescLayout))
         {
           // this set is incompatible, don't rebind it. Assume the application knows the shader
           // doesn't need this set, and the binding is just stale
@@ -703,9 +730,13 @@ void VulkanRenderState::BindDescriptorSetsWithoutPipeline(WrappedVulkan *vk, VkC
           // behaviour to use it anyway. If this binding *should* be valid, it will still be
           // valid at the end.
 
+          const DescSetLayout &iDescLayout =
+              vk->GetDebugManager()->GetDescSetLayout(iPipeLayout.descSetLayouts[i]);
+          const DescSetLayout &refDescLayout =
+              vk->GetDebugManager()->GetDescSetLayout(refPipeLayout.descSetLayouts[i]);
+
           if(iPipeLayout.descSetLayouts[i] != refPipeLayout.descSetLayouts[i] &&
-             vk->GetDebugManager()->GetDescSetLayout(iPipeLayout.descSetLayouts[i]) !=
-                 vk->GetDebugManager()->GetDescSetLayout(refPipeLayout.descSetLayouts[i]))
+             !iDescLayout.isCompatible(refDescLayout))
           {
             // set is incompatible, don't rebind it
             continue;
@@ -726,9 +757,13 @@ void VulkanRenderState::BindDescriptorSetsWithoutPipeline(WrappedVulkan *vk, VkC
             if(j >= refPipeLayout.descSetLayouts.size())
               break;
 
+            const DescSetLayout &iDescLayout =
+                vk->GetDebugManager()->GetDescSetLayout(iPipeLayout.descSetLayouts[i]);
+            const DescSetLayout &refDescLayout =
+                vk->GetDebugManager()->GetDescSetLayout(refPipeLayout.descSetLayouts[i]);
+
             if(iPipeLayout.descSetLayouts[j] != refPipeLayout.descSetLayouts[j] &&
-               vk->GetDebugManager()->GetDescSetLayout(iPipeLayout.descSetLayouts[j]) !=
-                   vk->GetDebugManager()->GetDescSetLayout(refPipeLayout.descSetLayouts[j]))
+               !iDescLayout.isCompatible(refDescLayout))
             {
               compatible = false;
               break;
@@ -815,19 +850,22 @@ void VulkanRenderState::BindDescriptorSet(WrappedVulkan *vk, const DescSetLayout
 
     VkWriteDescriptorSet push = {VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET};
 
+    VulkanResourceManager *rm = vk->GetResourceManager();
+
     for(size_t b = 0; b < descLayout.bindings.size(); b++)
     {
-      const DescSetLayout::Binding &bind = descLayout.bindings[b];
+      const DescSetLayout::Binding &layoutBind = descLayout.bindings[b];
 
       // skip if this binding isn't used
-      if(bind.descriptorType == VK_DESCRIPTOR_TYPE_MAX_ENUM)
+      if(layoutBind.layoutDescType == VK_DESCRIPTOR_TYPE_MAX_ENUM)
         continue;
 
       // push.dstSet; // unused for push descriptors
       push.dstBinding = (uint32_t)b;
       push.dstArrayElement = 0;
-      push.descriptorType = bind.descriptorType;
-      push.descriptorCount = bind.descriptorCount;
+      // we can use the layout type here, since push descriptors are not allowed to be mutable
+      push.descriptorType = layoutBind.layoutDescType;
+      push.descriptorCount = layoutBind.descriptorCount;
 
       const DescriptorSetSlot *slots = setInfo.data.binds[b];
 
@@ -837,8 +875,7 @@ void VulkanRenderState::BindDescriptorSet(WrappedVulkan *vk, const DescSetLayout
         VkBufferView *dst = new VkBufferView[push.descriptorCount];
 
         for(uint32_t a = 0; a < push.descriptorCount; a++)
-          dst[a] = Unwrap(
-              vk->GetResourceManager()->GetCurrentHandle<VkBufferView>(slots[a].texelBufferView));
+          dst[a] = Unwrap(rm->GetCurrentHandle<VkBufferView>(slots[a].resource));
 
         push.pTexelBufferView = dst;
         allocBufViewWrites.push_back(dst);
@@ -853,12 +890,9 @@ void VulkanRenderState::BindDescriptorSet(WrappedVulkan *vk, const DescSetLayout
 
         for(uint32_t a = 0; a < push.descriptorCount; a++)
         {
-          const DescriptorSetSlotImageInfo &src = slots[a].imageInfo;
-
-          dst[a].imageLayout = src.imageLayout;
-          dst[a].sampler = Unwrap(vk->GetResourceManager()->GetCurrentHandle<VkSampler>(src.sampler));
-          dst[a].imageView =
-              Unwrap(vk->GetResourceManager()->GetCurrentHandle<VkImageView>(src.imageView));
+          dst[a].imageLayout = convert(slots[a].imageLayout);
+          dst[a].sampler = Unwrap(rm->GetCurrentHandle<VkSampler>(slots[a].sampler));
+          dst[a].imageView = Unwrap(rm->GetCurrentHandle<VkImageView>(slots[a].resource));
         }
 
         push.pImageInfo = dst;
@@ -870,11 +904,11 @@ void VulkanRenderState::BindDescriptorSet(WrappedVulkan *vk, const DescSetLayout
         VkWriteDescriptorSetInlineUniformBlock *inlineWrite = allocInlineWrites.back();
         inlineWrite->sType = VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_INLINE_UNIFORM_BLOCK;
         inlineWrite->pNext = NULL;
-        inlineWrite->dataSize = bind.descriptorCount;
-        inlineWrite->pData = setInfo.data.inlineBytes.data() + slots[0].inlineOffset;
+        inlineWrite->dataSize = layoutBind.descriptorCount;
+        inlineWrite->pData = setInfo.data.inlineBytes.data() + slots[0].offset;
 
         push.pNext = inlineWrite;
-        push.descriptorCount = bind.descriptorCount;
+        push.descriptorCount = layoutBind.descriptorCount;
         writes.push_back(push);
 
         // skip validity checks
@@ -886,11 +920,9 @@ void VulkanRenderState::BindDescriptorSet(WrappedVulkan *vk, const DescSetLayout
 
         for(uint32_t a = 0; a < push.descriptorCount; a++)
         {
-          const DescriptorSetSlotBufferInfo &src = slots[a].bufferInfo;
-
-          dst[a].offset = src.offset;
-          dst[a].range = src.range;
-          dst[a].buffer = Unwrap(vk->GetResourceManager()->GetCurrentHandle<VkBuffer>(src.buffer));
+          dst[a].offset = slots[a].offset;
+          dst[a].range = slots[a].GetRange();
+          dst[a].buffer = Unwrap(rm->GetCurrentHandle<VkBuffer>(slots[a].resource));
         }
 
         push.pBufferInfo = dst;
@@ -900,7 +932,7 @@ void VulkanRenderState::BindDescriptorSet(WrappedVulkan *vk, const DescSetLayout
       // start with no descriptors
       push.descriptorCount = 0;
 
-      for(uint32_t w = 0; w < bind.descriptorCount; w++)
+      for(uint32_t w = 0; w < layoutBind.descriptorCount; w++)
       {
         // if this push is valid, we increment the descriptor count and continue
         if(IsValid(vk->NULLDescriptorsAllowed(), push, w - push.dstArrayElement))
