@@ -259,6 +259,22 @@ void WrappedVulkan::ReplayQueueSubmit(VkQueue queue, VkSubmitInfo2 submitInfo, r
     // we're adding multiple events, need to increment ourselves
     m_RootEventID++;
 
+    if(submitInfo.commandBufferInfoCount == 0)
+    {
+      rdcstr name = StringFormat::Fmt("=> %s: No Command Buffers", basename.c_str());
+
+      ActionDescription action;
+      action.customName = name;
+      action.flags |= ActionFlags::CommandBufferBoundary | ActionFlags::PassBoundary;
+      AddEvent();
+
+      m_RootEvents.back().chunkIndex = APIEvent::NoChunk;
+      m_Events.back().chunkIndex = APIEvent::NoChunk;
+
+      AddAction(action);
+      m_RootEventID++;
+    }
+
     for(uint32_t c = 0; c < submitInfo.commandBufferInfoCount; c++)
     {
       ResourceId cmd = GetResourceManager()->GetOriginalID(
@@ -363,6 +379,13 @@ void WrappedVulkan::ReplayQueueSubmit(VkQueue queue, VkSubmitInfo2 submitInfo, r
     // account for the queue submit event
     m_RootEventID++;
 
+    if(submitInfo.commandBufferInfoCount == 0)
+    {
+      // account for the "No Command Buffers" virtual label
+      m_RootEventID++;
+      m_RootActionID++;
+    }
+
     uint32_t startEID = m_RootEventID;
 
     // advance m_CurEventID to match the events added when reading
@@ -843,6 +866,7 @@ void WrappedVulkan::CaptureQueueSubmit(VkQueue queue,
 
   std::set<rdcpair<ResourceId, VkResourceRecord *>> capDescriptors;
   std::set<rdcpair<ResourceId, VkResourceRecord *>> descriptorSets;
+  std::set<VkResourceRecord *> cmdsWithReferences;
 
   // pull in any copy sources, conservatively
   if(capframe)
@@ -859,8 +883,6 @@ void WrappedVulkan::CaptureQueueSubmit(VkQueue queue,
 
     VkResourceRecord *record = GetRecord(commandBuffers[i]);
 
-    UpdateImageStates(record->bakedCommands->cmdInfo->imageStates);
-
     if(Vulkan_Debug_VerboseCommandRecording())
     {
       RDCLOG("vkQueueSubmit() to queue %s, cmd %zu of %zu: %s baked to %s",
@@ -882,12 +904,20 @@ void WrappedVulkan::CaptureQueueSubmit(VkQueue queue,
           it != record->bakedCommands->cmdInfo->sparse.end(); ++it)
         GetResourceManager()->MarkSparseMapReferenced(*it);
 
-      // pull in frame refs from this baked command buffer
-      record->bakedCommands->AddResourceReferences(GetResourceManager());
+      // can't pull in frame refs here, we need to do these last, since they are disjoint from
+      // descriptor references and we have to apply those first to be conservative.
+      // For example say a buffer is referenced in a descriptor for read, then completely
+      // overwritten in a later CmdFillBuffer. The fill will be listed in the normal references
+      // here, the descriptor reference is lazy tracked and only added below when we handle those.
+      // Since we don't know the ordering, we do it in the most conservative order - descriptor
+      // references can never be CompleteWrite, they're either read-only or read-before-write for
+      // storage descriptors.
+      // record->bakedCommands->AddResourceReferences(GetResourceManager());
+      // GetResourceManager()->MergeReferencedMemory(record->bakedCommands->cmdInfo->memFrameRefs);
+      // UpdateImageStates(record->bakedCommands->cmdInfo->imageStates);
+      cmdsWithReferences.insert(record->bakedCommands);
       record->bakedCommands->AddReferencedIDs(refdIDs);
 
-      GetResourceManager()->MergeReferencedMemory(record->bakedCommands->cmdInfo->memFrameRefs);
-
       // ref the parent command buffer's alloc record, this will pull in the cmd buffer pool
       GetResourceManager()->MarkResourceFrameReferenced(
           record->cmdInfo->allocRecord->GetResourceID(), eFrameRef_Read);
@@ -897,10 +927,15 @@ void WrappedVulkan::CaptureQueueSubmit(VkQueue queue,
       for(size_t sub = 0; sub < subcmds.size(); sub++)
       {
         VkResourceRecord *bakedSubcmds = subcmds[sub]->bakedCommands;
-        bakedSubcmds->AddResourceReferences(GetResourceManager());
+
+        // cannot add references here until after we've done descriptor sets later, see comment
+        // above
+        // bakedSubcmds->AddResourceReferences(GetResourceManager());
+        // GetResourceManager()->MergeReferencedMemory(bakedSubcmds->cmdInfo->memFrameRefs);
+        // UpdateImageStates(bakedSubcmds->cmdInfo->imageStates);
+        cmdsWithReferences.insert(bakedSubcmds);
+
         bakedSubcmds->AddReferencedIDs(refdIDs);
-        UpdateImageStates(bakedSubcmds->cmdInfo->imageStates);
-        GetResourceManager()->MergeReferencedMemory(bakedSubcmds->cmdInfo->memFrameRefs);
         GetResourceManager()->MarkResourceFrameReferenced(
             subcmds[sub]->cmdInfo->allocRecord->GetResourceID(), eFrameRef_Read);
 
@@ -939,9 +974,13 @@ void WrappedVulkan::CaptureQueueSubmit(VkQueue queue,
       VkResourceRecord *record = GetRecord(commandBuffers[i]);
 
       record->bakedCommands->AddResourceReferences(GetResourceManager());
+      UpdateImageStates(record->bakedCommands->cmdInfo->imageStates);
 
       for(VkResourceRecord *sub : record->bakedCommands->cmdInfo->subcmds)
+      {
         sub->bakedCommands->AddResourceReferences(GetResourceManager());
+        UpdateImageStates(sub->bakedCommands->cmdInfo->imageStates);
+      }
     }
 
     // every 20 submits clean background references, in case the application isn't presenting.
@@ -970,19 +1009,17 @@ void WrappedVulkan::CaptureQueueSubmit(VkQueue queue,
       {
         const DescSetLayout::Binding &bind = layout->bindings[b];
 
-        // skip empty bindings
-        if(bind.descriptorType == VK_DESCRIPTOR_TYPE_MAX_ENUM ||
-           bind.descriptorType == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK)
+        // skip empty bindings or inline uniform blocks
+        if(bind.layoutDescType == VK_DESCRIPTOR_TYPE_MAX_ENUM ||
+           bind.layoutDescType == VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK)
           continue;
 
         uint32_t count = bind.descriptorCount;
         if(bind.variableSize)
           count = setrecord->descInfo->data.variableDescriptorCount;
 
-        FrameRefType ref = GetRefType(bind.descriptorType);
-
         for(uint32_t a = 0; a < count; a++)
-          setrecord->descInfo->data.binds[b][a].AccumulateBindRefs(refs, rm, ref);
+          setrecord->descInfo->data.binds[b][a].AccumulateBindRefs(refs, rm);
       }
 
       for(auto refit = refs.bindFrameRefs.begin(); refit != refs.bindFrameRefs.end(); ++refit)
@@ -1009,6 +1046,15 @@ void WrappedVulkan::CaptureQueueSubmit(VkQueue queue,
       }
     }
 
+    // now we can insert frame references from command buffers, to have a conservative ordering vs.
+    // descriptor references since we don't know which happened first
+    for(auto it = cmdsWithReferences.begin(); it != cmdsWithReferences.end(); ++it)
+    {
+      (*it)->AddResourceReferences(GetResourceManager());
+      GetResourceManager()->MergeReferencedMemory((*it)->cmdInfo->memFrameRefs);
+      UpdateImageStates((*it)->cmdInfo->imageStates);
+    }
+
     GetResourceManager()->MarkResourceFrameReferenced(GetResID(queue), eFrameRef_Read);
 
     if(fence != VK_NULL_HANDLE)
@@ -1337,7 +1383,7 @@ VkResult WrappedVulkan::vkQueueSubmit(VkQueue queue, uint32_t submitCount,
 
   if(beginCapture)
   {
-    RenderDoc::Inst().StartFrameCapture(LayerDisp(m_Instance), NULL);
+    RenderDoc::Inst().StartFrameCapture(DeviceOwnedWindow(LayerDisp(m_Instance), NULL));
   }
 
   {
@@ -1391,13 +1437,13 @@ VkResult WrappedVulkan::vkQueueSubmit(VkQueue queue, uint32_t submitCount,
 
   if(endCapture)
   {
-    RenderDoc::Inst().EndFrameCapture(LayerDisp(m_Instance), NULL);
+    RenderDoc::Inst().EndFrameCapture(DeviceOwnedWindow(LayerDisp(m_Instance), NULL));
   }
 
   if(present)
   {
     AdvanceFrame();
-    Present(LayerDisp(m_Instance), NULL);
+    Present(DeviceOwnedWindow(LayerDisp(m_Instance), NULL));
   }
 
   return ret;
@@ -1505,7 +1551,7 @@ VkResult WrappedVulkan::vkQueueSubmit2(VkQueue queue, uint32_t submitCount,
 
   if(beginCapture)
   {
-    RenderDoc::Inst().StartFrameCapture(LayerDisp(m_Instance), NULL);
+    RenderDoc::Inst().StartFrameCapture(DeviceOwnedWindow(LayerDisp(m_Instance), NULL));
   }
 
   {
@@ -1559,13 +1605,13 @@ VkResult WrappedVulkan::vkQueueSubmit2(VkQueue queue, uint32_t submitCount,
 
   if(endCapture)
   {
-    RenderDoc::Inst().EndFrameCapture(LayerDisp(m_Instance), NULL);
+    RenderDoc::Inst().EndFrameCapture(DeviceOwnedWindow(LayerDisp(m_Instance), NULL));
   }
 
   if(present)
   {
     AdvanceFrame();
-    Present(LayerDisp(m_Instance), NULL);
+    Present(DeviceOwnedWindow(LayerDisp(m_Instance), NULL));
   }
 
   return ret;
@@ -1831,26 +1877,30 @@ VkResult WrappedVulkan::vkQueueBindSparse(VkQueue queue, uint32_t bindInfoCount,
   // update our internal page tables
   if(IsCaptureMode(m_State))
   {
+    std::set<ResourceId> memories;
     for(uint32_t i = 0; i < bindInfoCount; i++)
     {
       for(uint32_t buf = 0; buf < pBindInfo[i].bufferBindCount; buf++)
       {
         const VkSparseBufferMemoryBindInfo &bind = pBindInfo[i].pBufferBinds[buf];
-        GetRecord(bind.buffer)->resInfo->Update(bind.bindCount, bind.pBinds);
+        GetRecord(bind.buffer)->resInfo->Update(bind.bindCount, bind.pBinds, memories);
       }
 
       for(uint32_t op = 0; op < pBindInfo[i].imageOpaqueBindCount; op++)
       {
         const VkSparseImageOpaqueMemoryBindInfo &bind = pBindInfo[i].pImageOpaqueBinds[op];
-        GetRecord(bind.image)->resInfo->Update(bind.bindCount, bind.pBinds);
+        GetRecord(bind.image)->resInfo->Update(bind.bindCount, bind.pBinds, memories);
       }
 
       for(uint32_t op = 0; op < pBindInfo[i].imageBindCount; op++)
       {
         const VkSparseImageMemoryBindInfo &bind = pBindInfo[i].pImageBinds[op];
-        GetRecord(bind.image)->resInfo->Update(bind.bindCount, bind.pBinds);
+        GetRecord(bind.image)->resInfo->Update(bind.bindCount, bind.pBinds, memories);
       }
     }
+
+    for(ResourceId id : memories)
+      GetResourceManager()->MarkDirtyResource(id);
   }
 
   return ret;
