@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2021 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -46,9 +46,8 @@
  * careful the GPU never waits on an event that will never become set, or the GPU
  * will lock up.
  *
- * For now the implementation is simple, conservative and inefficient. We keep
- * events Set always, never replaying any Reset (CPU or GPU). This means any
- * wait will always succeed on the GPU.
+ * For now the implementation is simple, conservative and inefficient. We ignore
+ * the real events and don't set/reset them.
  *
  * On the CPU-side with GetEventStatus we do another hard sync with
  * DeviceWaitIdle.
@@ -123,7 +122,8 @@ bool WrappedVulkan::Serialise_vkCreateFence(SerialiserType &ser, VkDevice device
 
     if(ret != VK_SUCCESS)
     {
-      RDCERR("Failed on resource serialise-creation, VkResult: %s", ToStr(ret).c_str());
+      SET_ERROR_RESULT(m_FailedReplayResult, ResultCode::APIReplayFailed,
+                       "Failed creating fence, VkResult: %s", ToStr(ret).c_str());
       return false;
     }
     else
@@ -347,11 +347,11 @@ bool WrappedVulkan::Serialise_vkCreateEvent(SerialiserType &ser, VkDevice device
     VkResult ret = ObjDisp(device)->CreateEvent(Unwrap(device), &CreateInfo, NULL, &ev);
 
     // see top of this file for current event/fence handling
-    ObjDisp(device)->SetEvent(Unwrap(device), ev);
 
     if(ret != VK_SUCCESS)
     {
-      RDCERR("Failed on resource serialise-creation, VkResult: %s", ToStr(ret).c_str());
+      SET_ERROR_RESULT(m_FailedReplayResult, ResultCode::APIReplayFailed,
+                       "Failed creating event, VkResult: %s", ToStr(ret).c_str());
       return false;
     }
     else
@@ -557,7 +557,8 @@ bool WrappedVulkan::Serialise_vkCreateSemaphore(SerialiserType &ser, VkDevice de
 
     if(ret != VK_SUCCESS)
     {
-      RDCERR("Failed on resource serialise-creation, VkResult: %s", ToStr(ret).c_str());
+      SET_ERROR_RESULT(m_FailedReplayResult, ResultCode::APIReplayFailed,
+                       "Failed creating semaphore, VkResult: %s", ToStr(ret).c_str());
       return false;
     }
     else
@@ -1171,8 +1172,8 @@ bool WrappedVulkan::Serialise_vkCmdSetEvent2(SerialiserType &ser, VkCommandBuffe
         commandBuffer = VK_NULL_HANDLE;
     }
 
-    if(commandBuffer != VK_NULL_HANDLE)
-      ObjDisp(commandBuffer)->CmdSetEvent2(Unwrap(commandBuffer), Unwrap(event), &DependencyInfo);
+    // if(commandBuffer != VK_NULL_HANDLE)
+    //  ObjDisp(commandBuffer)->CmdSetEvent2(Unwrap(commandBuffer), Unwrap(event), &DependencyInfo);
   }
 
   return true;
@@ -1183,18 +1184,11 @@ void WrappedVulkan::vkCmdSetEvent2(VkCommandBuffer commandBuffer, VkEvent event,
 {
   SCOPED_DBG_SINK();
 
-  VkDependencyInfo unwrappedInfo = *pDependencyInfo;
-
-  byte *tempMem = GetTempMemory(GetNextPatchSize(&unwrappedInfo));
-
-  {
-    VkBaseInStructure dummy = {};
-    dummy.pNext = (const VkBaseInStructure *)&unwrappedInfo;
-    UnwrapNextChain(m_State, "VkDependencyInfo", tempMem, &dummy);
-  }
+  byte *tempMem = GetTempMemory(GetNextPatchSize(pDependencyInfo));
+  VkDependencyInfo *unwrappedInfo = UnwrapStructAndChain(m_State, tempMem, pDependencyInfo);
 
   SERIALISE_TIME_CALL(
-      ObjDisp(commandBuffer)->CmdSetEvent2(Unwrap(commandBuffer), Unwrap(event), &unwrappedInfo));
+      ObjDisp(commandBuffer)->CmdSetEvent2(Unwrap(commandBuffer), Unwrap(event), unwrappedInfo));
 
   if(IsCaptureMode(m_State))
   {
