@@ -316,6 +316,8 @@ FILE *fopen(const rdcstr &filename, FileMode mode);
 size_t fread(void *buf, size_t elementSize, size_t count, FILE *f);
 size_t fwrite(const void *buf, size_t elementSize, size_t count, FILE *f);
 
+bool IsUntrustedFile(const rdcstr &filename);
+
 bool exists(const rdcstr &filename);
 
 rdcstr ErrorString();
@@ -464,7 +466,6 @@ void Shutdown();
 namespace OSUtility
 {
 inline void ForceCrash();
-inline void DebugBreak();
 bool DebuggerPresent();
 enum
 {
