@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -42,7 +42,7 @@ static BOOL add_hooks()
   // this instance is being used for a shell extension.
   if(f == "dllhost.exe" || f == "explorer.exe")
   {
-#ifndef _RELEASE
+#if ENABLED(RDOC_RELEASE)
     OutputDebugStringA(
         "Detecting shell process! Disabling hooking in dllhost.exe or explorer.exe\n");
 #endif
@@ -58,6 +58,8 @@ static BOOL add_hooks()
 
     RenderDoc::Inst().Initialise();
 
+    LibraryHooks::ReplayInitialise();
+
     return true;
   }
 
