@@ -1913,8 +1913,8 @@ bool ReplayController::FatalErrorCheck()
 
   if(m_FatalError != ResultCode::Succeeded)
   {
-    RDCLOG("Fatal error detected: %s (%s)", ToStr(m_FatalError.code).c_str(),
-           m_FatalError.message.c_str());
+    RDCLOG("Fatal error detected: %s (%s) at event %u", ToStr(m_FatalError.code).c_str(),
+           m_FatalError.message.c_str(), m_EventID);
 
     IReplayDriver *old = m_pDevice;
 
@@ -2153,7 +2153,7 @@ RDResult ReplayController::PostCreateInit(IReplayDriver *device, RDCFile *rdc)
   m_pDevice->SetPipelineStates(&m_D3D11PipelineState, &m_D3D12PipelineState, &m_GLPipelineState,
                                &m_VulkanPipelineState);
 
-  GCNISA::GetTargets(m_APIProps.pipelineType, m_GCNTargets);
+  GCNISA::GetTargets(m_APIProps.pipelineType, m_pDevice->GetDriverInfo(), m_GCNTargets);
 
   if(result != ResultCode::Succeeded)
     return result;
