@@ -202,6 +202,15 @@ void ReplayOutput::SetTextureDisplay(const TextureDisplay &o)
   m_CustomDirty = true;
   m_RenderData.texDisplay = o;
   m_MainOutput.dirty = true;
+
+  m_TextureDim = {0, 0};
+  for(size_t t = 0; t < m_pController->m_Textures.size(); t++)
+  {
+    if(m_pController->m_Textures[t].resourceId == m_RenderData.texDisplay.resourceId)
+    {
+      m_TextureDim = {m_pController->m_Textures[t].width, m_pController->m_Textures[t].height};
+    }
+  }
 }
 
 void ReplayOutput::SetMeshDisplay(const MeshDisplay &o)
@@ -341,6 +350,9 @@ void ReplayOutput::ClearThumbnails()
   for(size_t i = 0; i < m_Thumbnails.size(); i++)
     m_pDevice->DestroyOutputWindow(m_Thumbnails[i].outputID);
 
+  for(auto it = m_ThumbnailGenerators.begin(); it != m_ThumbnailGenerators.end(); ++it)
+    m_pDevice->DestroyOutputWindow(it->second);
+
   m_Thumbnails.clear();
 }
 
@@ -362,6 +374,125 @@ ResultDetails ReplayOutput::SetPixelContext(WindowingData window)
   return RDResult();
 }
 
+bytebuf ReplayOutput::DrawThumbnail(int32_t width, int32_t height, ResourceId textureId,
+                                    const Subresource &sub, CompType typeCast)
+{
+  bytebuf ret;
+
+  uint64_t key = (uint64_t(width) << 32) | height;
+  int idx = -1;
+  uint64_t outputID = 0;
+  for(int i = 0; i < m_ThumbnailGenerators.count(); i++)
+  {
+    if(m_ThumbnailGenerators[i].first == key)
+    {
+      idx = i;
+      outputID = m_ThumbnailGenerators[i].second;
+      break;
+    }
+  }
+
+  if(idx < 0)
+  {
+    // resize oldest generator if we have hit the max
+    if(m_ThumbnailGenerators.size() == MaxThumbnailGenerators)
+    {
+      outputID = m_ThumbnailGenerators.back().second;
+      m_pDevice->SetOutputWindowDimensions(outputID, width, height);
+      m_ThumbnailGenerators.pop_back();
+    }
+    else
+    {
+      outputID = m_pDevice->MakeOutputWindow(CreateHeadlessWindowingData(width, height), false);
+    }
+  }
+  else
+  {
+    // remove the found one, so it can get inserted into the front
+    m_ThumbnailGenerators.erase(idx);
+  }
+  // make this the most recent generator, so the oldest one will be kicked out
+  m_ThumbnailGenerators.insert(0, {key, outputID});
+
+  bool depthMode = false;
+
+  for(size_t t = 0; t < m_pController->m_Textures.size(); t++)
+  {
+    if(m_pController->m_Textures[t].resourceId == textureId)
+    {
+      depthMode = (m_pController->m_Textures[t].creationFlags & TextureCategory::DepthTarget) ||
+                  (m_pController->m_Textures[t].format.compType == CompType::Depth);
+      break;
+    }
+  }
+
+  if(textureId == ResourceId())
+  {
+    m_pDevice->BindOutputWindow(outputID, false);
+
+    FloatVector dark = RenderDoc::Inst().DarkCheckerboardColor();
+    FloatVector light = RenderDoc::Inst().LightCheckerboardColor();
+
+    FloatVector dark2;
+    dark2.x = light.x;
+    dark2.y = dark.y;
+    dark2.z = dark.z;
+    dark2.w = 1.0f;
+    FloatVector light2;
+    light2.x = dark.x;
+    light2.y = light.y;
+    light2.z = light.z;
+    light2.w = 1.0f;
+    m_pDevice->RenderCheckerboard(dark2, light2);
+    m_pController->FatalErrorCheck();
+
+    m_pDevice->FlipOutputWindow(outputID);
+    m_pController->FatalErrorCheck();
+  }
+  else
+  {
+    m_pDevice->BindOutputWindow(outputID, false);
+    m_pDevice->ClearOutputWindowColor(outputID, FloatVector());
+
+    TextureDisplay disp;
+
+    disp.red = disp.green = disp.blue = true;
+    disp.alpha = false;
+    disp.hdrMultiplier = -1.0f;
+    disp.linearDisplayAsGamma = true;
+    disp.flipY = false;
+    disp.subresource = sub;
+    disp.subresource.sample = 0;
+    disp.customShaderId = ResourceId();
+    disp.resourceId = m_pDevice->GetLiveID(textureId);
+    disp.typeCast = typeCast;
+    disp.scale = -1.0f;
+    disp.rangeMin = 0.0f;
+    disp.rangeMax = 1.0f;
+    disp.xOffset = 0.0f;
+    disp.yOffset = 0.0f;
+    disp.rawOutput = false;
+    disp.overlay = DebugOverlay::NoOverlay;
+
+    if(typeCast == CompType::SNorm)
+      disp.rangeMin = -1.0f;
+
+    if(depthMode)
+      disp.green = disp.blue = false;
+
+    m_pDevice->RenderTexture(disp);
+    m_pController->FatalErrorCheck();
+
+    m_pDevice->FlipOutputWindow(outputID);
+    m_pController->FatalErrorCheck();
+  }
+
+  m_pDevice->GetOutputWindowData(outputID, ret);
+  m_pController->FatalErrorCheck();
+
+  return ret;
+}
+
 ResultDetails ReplayOutput::AddThumbnail(WindowingData window, ResourceId texID,
                                          const Subresource &sub, CompType typeCast)
 {
@@ -593,11 +724,25 @@ void ReplayOutput::DisplayContext()
   int x = (int)m_ContextX;
   int y = (int)m_ContextY;
 
-  x >>= disp.subresource.mip;
-  x <<= disp.subresource.mip;
+  if(m_TextureDim.first > 0 && m_TextureDim.second > 0)
+  {
+    rdcpair<uint32_t, uint32_t> mipDim = {RDCMAX(1U, m_TextureDim.first >> disp.subresource.mip),
+                                          RDCMAX(1U, m_TextureDim.second >> disp.subresource.mip)};
 
-  y >>= disp.subresource.mip;
-  y <<= disp.subresource.mip;
+    x = int((float(x) / float(m_TextureDim.first)) * mipDim.first);
+    x = int((float(x) / float(mipDim.first)) * m_TextureDim.first);
+
+    y = int((float(y) / float(m_TextureDim.second)) * mipDim.second);
+    y = int((float(y) / float(mipDim.second)) * m_TextureDim.second);
+  }
+  else
+  {
+    x >>= disp.subresource.mip;
+    x <<= disp.subresource.mip;
+
+    y >>= disp.subresource.mip;
+    y <<= disp.subresource.mip;
+  }
 
   disp.xOffset = -(float)x * disp.scale;
   disp.yOffset = -(float)y * disp.scale;
