@@ -107,9 +107,12 @@ StructuredBuffer<MyStruct> structrotest : register(t2);
 Texture2D<float> dimtex : register(t3);
 Texture2DMS<float> dimtexms : register(t4);
 Texture2D<float4> smiley : register(t5);
+Texture2D<int4> smileyint : register(t6);
+Texture2D<uint4> smileyuint : register(t7);
 
 RWByteAddressBuffer byterwtest : register(u1);
 RWStructuredBuffer<MyStruct> structrwtest : register(u2);
+RWBuffer<float4> typedrwtest : register(u3);
 
 Buffer<float> unboundsrv1 : register(t100);
 Texture2D<float> unboundsrv2 : register(t101);
@@ -243,7 +246,7 @@ float4 main(v2f IN) : SV_Target0
   // test reading/writing byte address data
 
   // mis-aligned loads
-  if(IN.tri == 35)
+  if(IN.tri == 35) // undefined-test
   {
     // use this to ensure the compiler doesn't know we're using fixed locations
     uint z = intval - IN.tri - 7;
@@ -289,7 +292,7 @@ float4 main(v2f IN) : SV_Target0
   }
 
   // mis-aligned store
-  if(IN.tri == 40)
+  if(IN.tri == 40) // undefined-test
   {
     // use this to ensure the compiler doesn't know we're using fixed locations
     uint z = intval - IN.tri - 7;
@@ -301,7 +304,7 @@ float4 main(v2f IN) : SV_Target0
     return asfloat(byterwtest.Load(z2+0).x);
   }
   // mis-aligned loads
-  if(IN.tri == 41)
+  if(IN.tri == 41) // undefined-test
   {
     // use this to ensure the compiler doesn't know we're using fixed locations
     uint z = intval - IN.tri - 7;
@@ -676,6 +679,67 @@ float4 main(v2f IN) : SV_Target0
   {
     return rgb_srv[0];
   }
+  if(IN.tri == 80)
+  {
+    uint z2 = uint(zero);
+    MyStruct read = structrwtest[z2+7];
+
+    return read.b.xyzw;
+  }
+  if(IN.tri == 81)
+  {
+    uint z2 = uint(zero);
+    MyStruct read = structrwtest[z2+7];
+
+    return read.b.zzyx;
+  }
+  if(IN.tri == 82)
+  {
+    uint z2 = uint(zero);
+    MyStruct read = structrwtest[z2+7];
+
+    return read.b.zwxy;
+  }
+  if(IN.tri == 83)
+  {
+    uint z2 = uint(zero);
+    MyStruct read = structrwtest[z2+7];
+
+    return read.b.wzwy;
+  }
+#ifdef TYPED_UAV_EXT
+  if(IN.tri == 84)
+  {
+    return typedrwtest[uint(zero)].xyzw;
+  }
+  if(IN.tri == 85)
+  {
+    return typedrwtest[uint(zero)].zzyx;
+  }
+  if(IN.tri == 86)
+  {
+    return typedrwtest[uint(zero)].zwxy;
+  }
+  if(IN.tri == 87)
+  {
+    return typedrwtest[uint(zero)].wzwy;
+  }
+#endif
+  if(IN.tri == 88)
+  {
+    float2 uv = posone * float2(0.55f, 0.48f);
+    return smiley.Sample(linearwrap, uv, int2(4, 3));
+  }
+  if(IN.tri == 89)
+  {
+    float2 uv = posone * float2(1.81f, 0.48f);
+    return smileyint.Load(int3(uv*16,0));
+  }
+  if(IN.tri == 90)
+  {
+    float2 uv = posone * float2(1.81f, 0.48f);
+    return smileyuint.Load(int3(uv*16,0));
+  }
 
   return float4(0.4f, 0.4f, 0.4f, 0.4f);
 }
@@ -849,6 +913,24 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
 
     const uint32_t numTests = atoi(pixel.c_str() + lastTest) + 1;
 
+    std::string undefined_tests = "Undefined tests:";
+
+    size_t undef = pixel.find("undefined-test");
+    while(undef != std::string::npos)
+    {
+      size_t testNumStart = pixel.rfind("IN.tri == ", undef);
+      testNumStart += sizeof("IN.tri == ") - 1;
+      size_t testNumEnd = pixel.find_first_not_of("0123456789", testNumStart);
+
+      undefined_tests += " ";
+      undefined_tests += pixel.substr(testNumStart, testNumEnd - testNumStart);
+
+      undef = pixel.find("undefined-test", undef + 1);
+    }
+
+    if(opts2.TypedUAVLoadAdditionalFormats)
+      common += "\n#define TYPED_UAV_EXT 1\n";
+
     ID3DBlobPtr vsblob = Compile(common + vertex, "main", "vs_5_0");
     ID3DBlobPtr psblob = Compile(common + pixel, "main", "ps_5_0");
 
@@ -938,17 +1020,22 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
     ID3D11UnorderedAccessViewPtr structuav =
         MakeUAV(structBuf2).Format(DXGI_FORMAT_UNKNOWN).FirstElement(3).NumElements(5);
 
+    ID3D11BufferPtr rgbuavBuf = MakeBuffer().UAV().Data(structdata);
+    ID3D11UnorderedAccessViewPtr typeuav = MakeUAV(rgbuavBuf).Format(DXGI_FORMAT_R32G32B32A32_FLOAT);
+
     Texture rgba8;
     LoadXPM(SmileyTexture, rgba8);
 
     ID3D11Texture2DPtr smiley =
-        MakeTexture(DXGI_FORMAT_R8G8B8A8_UNORM, rgba8.width, rgba8.height).SRV();
-    ID3D11ShaderResourceViewPtr smileysrv = MakeSRV(smiley);
+        MakeTexture(DXGI_FORMAT_R8G8B8A8_TYPELESS, rgba8.width, rgba8.height).SRV();
+    ID3D11ShaderResourceViewPtr smileysrv = MakeSRV(smiley).Format(DXGI_FORMAT_R8G8B8A8_UNORM);
+    ID3D11ShaderResourceViewPtr smileyintsrv = MakeSRV(smiley).Format(DXGI_FORMAT_R8G8B8A8_SINT);
+    ID3D11ShaderResourceViewPtr smileyuintsrv = MakeSRV(smiley).Format(DXGI_FORMAT_R8G8B8A8_UINT);
 
     ctx->UpdateSubresource(smiley, 0, NULL, rgba8.data.data(), rgba8.width * sizeof(uint32_t), 0);
 
     ID3D11ShaderResourceView *srvs[] = {
-        srv, rawsrv, structsrv, testSRV, msSRV, smileysrv,
+        srv, rawsrv, structsrv, testSRV, msSRV, smileysrv, smileyintsrv, smileyuintsrv,
     };
 
     ctx->PSSetShaderResources(0, ARRAY_COUNT(srvs), srvs);
@@ -992,10 +1079,12 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
       UINT zero[4] = {};
       ctx->ClearUnorderedAccessViewUint(rawuav, zero);
       ctx->ClearUnorderedAccessViewUint(structuav, zero);
-      ID3D11UnorderedAccessView *uavs[] = {rawuav, structuav};
-      ctx->OMSetRenderTargetsAndUnorderedAccessViews(1, &fltRT.GetInterfacePtr(), NULL, 1, 2, uavs,
+      ID3D11UnorderedAccessView *uavs[] = {rawuav, structuav, typeuav};
+      ctx->OMSetRenderTargetsAndUnorderedAccessViews(1, &fltRT.GetInterfacePtr(), NULL, 1, 3, uavs,
                                                      NULL);
 
+      setMarker(undefined_tests);
+
       setMarker("Main Test");
       ctx->DrawInstanced(3, numTests, 0, 0);
 
