@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019-2020 Baldur Karlsson
+ * Copyright (c) 2019-2022 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -219,10 +219,11 @@ float4 main(v2f IN) : SV_Target0
       vbData[i].col.y = float(i) / 200.0f;
     }
 
-    ID3D12ResourcePtr vb =
-        MakeBuffer().Data(vbData).Size(UINT((vbData.size() + 100) * sizeof(DefaultA2V)));
+    vbData.resize(vbData.size() + 100);
 
-    Vec4f instData[16] = {};
+    ID3D12ResourcePtr vb = MakeBuffer().Data(vbData).Size(UINT(vbData.size() * sizeof(DefaultA2V)));
+
+    Vec4f instData[256] = {};
     for(int i = 0; i < ARRAY_COUNT(instData); i++)
       instData[i] = Vec4f(-100.0f, -100.0f, -100.0f, -100.0f);
 
@@ -240,7 +241,7 @@ float4 main(v2f IN) : SV_Target0
     ID3D12ResourcePtr instvb = MakeBuffer().Data(instData).Size(4096);
 
     std::vector<uint16_t> idxData;
-    idxData.resize(100);
+    idxData.resize(2048);
 
     {
       idxData[0] = 0;
