@@ -156,6 +156,32 @@ D3D12_ROOT_PARAMETER1 cbvParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg
   return ret;
 }
 
+D3D12_ROOT_PARAMETER1 srvParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg)
+{
+  D3D12_ROOT_PARAMETER1 ret;
+
+  ret.ShaderVisibility = vis;
+  ret.ParameterType = D3D12_ROOT_PARAMETER_TYPE_SRV;
+  ret.Descriptor.RegisterSpace = space;
+  ret.Descriptor.ShaderRegister = reg;
+  ret.Descriptor.Flags = D3D12_ROOT_DESCRIPTOR_FLAG_NONE;
+
+  return ret;
+}
+
+D3D12_ROOT_PARAMETER1 uavParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg)
+{
+  D3D12_ROOT_PARAMETER1 ret;
+
+  ret.ShaderVisibility = vis;
+  ret.ParameterType = D3D12_ROOT_PARAMETER_TYPE_UAV;
+  ret.Descriptor.RegisterSpace = space;
+  ret.Descriptor.ShaderRegister = reg;
+  ret.Descriptor.Flags = D3D12_ROOT_DESCRIPTOR_FLAG_NONE;
+
+  return ret;
+}
+
 D3D12_ROOT_PARAMETER1 constParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg, UINT num)
 {
   D3D12_ROOT_PARAMETER1 ret;
@@ -177,6 +203,11 @@ D3D12_ROOT_PARAMETER1 tableParam(D3D12_SHADER_VISIBILITY vis, D3D12_DESCRIPTOR_R
   static D3D12_DESCRIPTOR_RANGE1 ranges[32] = {};
   static int rangeIdx = 0;
 
+  if(type == D3D12_DESCRIPTOR_RANGE_TYPE_SAMPLER)
+  {
+    flags &= ~D3D12_DESCRIPTOR_RANGE_FLAG_DATA_VOLATILE;
+  }
+
   D3D12_DESCRIPTOR_RANGE1 &range = ranges[rangeIdx];
   rangeIdx = (rangeIdx + 1) % ARRAY_COUNT(ranges);
 
@@ -337,6 +368,12 @@ D3D12TextureCreator &D3D12TextureCreator::DSV()
   return *this;
 }
 
+D3D12TextureCreator &D3D12TextureCreator::NoSRV()
+{
+  m_TexDesc.Flags |= D3D12_RESOURCE_FLAG_DENY_SHADER_RESOURCE;
+  return *this;
+}
+
 D3D12TextureCreator &D3D12TextureCreator::Upload()
 {
   m_HeapDesc.Type = D3D12_HEAP_TYPE_UPLOAD;
