@@ -160,6 +160,7 @@ public:
   D3D12TextureCreator &UAV();
   D3D12TextureCreator &RTV();
   D3D12TextureCreator &DSV();
+  D3D12TextureCreator &NoSRV();
 
   D3D12TextureCreator &Upload();
   D3D12TextureCreator &Readback();
@@ -264,6 +265,8 @@ private:
 };
 
 D3D12_ROOT_PARAMETER1 cbvParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg);
+D3D12_ROOT_PARAMETER1 srvParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg);
+D3D12_ROOT_PARAMETER1 uavParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg);
 
 D3D12_ROOT_PARAMETER1 constParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg, UINT num);
 
