@@ -156,6 +156,9 @@ ByteAddressBuffer byterotest : register(t1);
 StructuredBuffer<MyStruct> structrotest : register(t2);
 Texture2D<float> dimtex : register(t3);
 Texture2DMS<float> dimtexms : register(t4);
+Texture2D<float4> smiley : register(t5);
+Texture2D<int4> smileyint : register(t6);
+Texture2D<uint4> smileyuint : register(t7);
 RWByteAddressBuffer byterwtest : register(u1);
 RWStructuredBuffer<MyStruct> structrwtest : register(u2);
 
@@ -172,6 +175,10 @@ Buffer<float4> rgb_srv : register(t103);
 
 SamplerState linearclamp : register(s0);
 
+StructuredBuffer<MyStruct> rootsrv : register(t20);
+StructuredBuffer<MyStruct> appendsrv : register(t40);
+Texture2D<float> dimtex_edge : register(t41);
+
 float4 main(v2f IN) : SV_Target0
 {
   float  posinf = IN.oneVal/IN.zeroVal.x;
@@ -293,7 +300,7 @@ float4 main(v2f IN) : SV_Target0
   // test reading/writing byte address data
 
   // mis-aligned loads
-  if(IN.tri == 35)
+  if(IN.tri == 35) // undefined-test
   {
     // use this to ensure the compiler doesn't know we're using fixed locations
     uint z = intval - IN.tri - 7;
@@ -339,7 +346,7 @@ float4 main(v2f IN) : SV_Target0
   }
 
   // mis-aligned store
-  if(IN.tri == 40)
+  if(IN.tri == 40) // undefined-test
   {
     // use this to ensure the compiler doesn't know we're using fixed locations
     uint z = intval - IN.tri - 7;
@@ -351,7 +358,7 @@ float4 main(v2f IN) : SV_Target0
     return asfloat(byterwtest.Load(z2+0).x);
   }
   // mis-aligned loads
-  if(IN.tri == 41)
+  if(IN.tri == 41) // undefined-test
   {
     // use this to ensure the compiler doesn't know we're using fixed locations
     uint z = intval - IN.tri - 7;
@@ -700,6 +707,41 @@ float4 main(v2f IN) : SV_Target0
   {
     return rgb_srv[0];
   }
+  if(IN.tri == 77)
+  {
+    float2 uv = posone * float2(0.55f, 0.48f);
+    return smiley.Sample(linearclamp, uv, int2(4, 3));
+  }
+  if(IN.tri == 78)
+  {
+    uint z = intval - IN.tri - 7;
+
+    MyStruct read = rootsrv[z+0];
+
+    return float4(read.b.xyz, read.c);
+  }
+  if(IN.tri == 79)
+  {
+    uint z = intval - IN.tri - 7;
+
+    MyStruct read = appendsrv[z+0];
+
+    return float4(read.b.xyz, read.c);
+  }
+  if(IN.tri == 80)
+  {
+    // use this to ensure the compiler doesn't know we're using fixed mips
+    uint z = intval - IN.tri - 7;
+
+    uint width = 0, height = 0, numLevels = 0;
+    dimtex_edge.GetDimensions(z, width, height, numLevels);
+    return float4(max(1,width), max(1,height), numLevels, 0.0f);
+  }
+  if(IN.tri == 81)
+  {
+    float2 uv = posone * float2(0.55f, 0.48f);
+    return smileyint.Load(int3(uv*16,0));
+  }
 
   return float4(0.4f, 0.4f, 0.4f, 0.4f);
 }
@@ -747,6 +789,20 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
 
     const uint32_t numTests = atoi(pixel.c_str() + lastTest) + 1;
 
+    std::string undefined_tests = "Undefined tests:";
+
+    size_t undef = pixel.find("undefined-test");
+    while(undef != std::string::npos)
+    {
+      size_t testNumStart = pixel.rfind("IN.tri == ", undef);
+      testNumStart += sizeof("IN.tri == ") - 1;
+      size_t testNumEnd = pixel.find_first_not_of("0123456789", testNumStart);
+
+      undefined_tests += " ";
+      undefined_tests += pixel.substr(testNumStart, testNumEnd - testNumStart);
+
+      undef = pixel.find("undefined-test", undef + 1);
+    }
     ID3DBlobPtr vsblob = Compile(common + vertex, "main", "vs_5_0");
     ID3DBlobPtr psblob = Compile(common + pixel, "main", "ps_5_0");
 
@@ -774,12 +830,40 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
     staticSamp.AddressU = staticSamp.AddressV = staticSamp.AddressW = D3D12_TEXTURE_ADDRESS_MODE_WRAP;
     staticSamp.ShaderVisibility = D3D12_SHADER_VISIBILITY_PIXEL;
 
+    D3D12_DESCRIPTOR_RANGE1 multiRanges[3] = {
+        {
+            D3D12_DESCRIPTOR_RANGE_TYPE_SRV, 2, 30, 0,
+            D3D12_DESCRIPTOR_RANGE_FLAG_DESCRIPTORS_VOLATILE |
+                D3D12_DESCRIPTOR_RANGE_FLAG_DATA_VOLATILE,
+            30,
+        },
+        {
+            D3D12_DESCRIPTOR_RANGE_TYPE_UAV, 3, 32, 0,
+            D3D12_DESCRIPTOR_RANGE_FLAG_DESCRIPTORS_VOLATILE |
+                D3D12_DESCRIPTOR_RANGE_FLAG_DATA_VOLATILE,
+            D3D12_DESCRIPTOR_RANGE_OFFSET_APPEND,
+        },
+        {
+            D3D12_DESCRIPTOR_RANGE_TYPE_SRV, 2, 40, 0,
+            D3D12_DESCRIPTOR_RANGE_FLAG_DESCRIPTORS_VOLATILE |
+                D3D12_DESCRIPTOR_RANGE_FLAG_DATA_VOLATILE,
+            D3D12_DESCRIPTOR_RANGE_OFFSET_APPEND,
+        },
+    };
+    D3D12_ROOT_PARAMETER1 multiRangeParam;
+    multiRangeParam.ShaderVisibility = D3D12_SHADER_VISIBILITY_PIXEL;
+    multiRangeParam.ParameterType = D3D12_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE;
+    multiRangeParam.DescriptorTable.NumDescriptorRanges = ARRAY_COUNT(multiRanges);
+    multiRangeParam.DescriptorTable.pDescriptorRanges = multiRanges;
+
     ID3D12RootSignaturePtr sig = MakeSig(
         {
-            tableParam(D3D12_SHADER_VISIBILITY_PIXEL, D3D12_DESCRIPTOR_RANGE_TYPE_SRV, 0, 0, 5, 0),
-            tableParam(D3D12_SHADER_VISIBILITY_PIXEL, D3D12_DESCRIPTOR_RANGE_TYPE_UAV, 0, 1, 2, 5),
+            tableParam(D3D12_SHADER_VISIBILITY_PIXEL, D3D12_DESCRIPTOR_RANGE_TYPE_SRV, 0, 0, 8, 0),
+            tableParam(D3D12_SHADER_VISIBILITY_PIXEL, D3D12_DESCRIPTOR_RANGE_TYPE_UAV, 0, 1, 2, 10),
             tableParam(D3D12_SHADER_VISIBILITY_PIXEL, D3D12_DESCRIPTOR_RANGE_TYPE_SRV, 0, 100, 5, 20),
             tableParam(D3D12_SHADER_VISIBILITY_PIXEL, D3D12_DESCRIPTOR_RANGE_TYPE_UAV, 0, 4, 3, 30),
+            multiRangeParam, uavParam(D3D12_SHADER_VISIBILITY_PIXEL, 0, 21),
+            srvParam(D3D12_SHADER_VISIBILITY_PIXEL, 0, 20),
         },
         D3D12_ROOT_SIGNATURE_FLAG_ALLOW_INPUT_ASSEMBLER_INPUT_LAYOUT, 1, &staticSamp);
 
@@ -791,7 +875,7 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
                                          .RTVs({DXGI_FORMAT_R32G32B32A32_FLOAT});
 
     // Recompile the same PS with SM 5.1 to test shader debugging with the different bytecode
-    psblob = Compile(common + pixel, "main", "ps_5_1");
+    psblob = Compile(common + "\n#define SM_5_1 1\n" + pixel, "main", "ps_5_1");
     ID3D12PipelineStatePtr pso_5_1 = MakePSO()
                                          .RootSig(sig)
                                          .InputLayout(inputLayout)
@@ -805,7 +889,7 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
                                    .RTV()
                                    .InitialState(D3D12_RESOURCE_STATE_RENDER_TARGET);
     D3D12_CPU_DESCRIPTOR_HANDLE fltRTV = MakeRTV(fltTex).CreateCPU(0);
-    D3D12_GPU_DESCRIPTOR_HANDLE fltSRV = MakeSRV(fltTex).CreateGPU(7);
+    D3D12_GPU_DESCRIPTOR_HANDLE fltSRV = MakeSRV(fltTex).CreateGPU(8);
 
     float triWidth = 8.0f / float(texDim);
 
@@ -840,6 +924,20 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
     cpu.ptr += dev->GetDescriptorHandleIncrementSize(D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV) * 3;
     dev->CreateShaderResourceView(testTex, NULL, cpu);
 
+    {
+      cpu = m_CBVUAVSRV->GetCPUDescriptorHandleForHeapStart();
+      cpu.ptr += dev->GetDescriptorHandleIncrementSize(D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV) * 36;
+
+      D3D12_SHADER_RESOURCE_VIEW_DESC desc = {};
+      desc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
+      desc.ViewDimension = D3D12_SRV_DIMENSION_TEXTURE2DARRAY;
+      desc.Shader4ComponentMapping = D3D12_DEFAULT_SHADER_4_COMPONENT_MAPPING;
+      desc.Texture2DArray.ArraySize = ~0U;
+      desc.Texture2DArray.MipLevels = ~0U;
+
+      dev->CreateShaderResourceView(testTex, &desc, cpu);
+    }
+
     ID3D12ResourcePtr rawBuf = MakeBuffer().Data(testdata);
     MakeSRV(rawBuf)
         .Format(DXGI_FORMAT_R32_TYPELESS)
@@ -851,11 +949,87 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
     ID3D12ResourcePtr msTex = MakeTexture(DXGI_FORMAT_R32_FLOAT, 16, 16).Multisampled(4).RTV();
     MakeSRV(msTex).CreateGPU(4);
 
+    Texture rgba8;
+    LoadXPM(SmileyTexture, rgba8);
+
+    ID3D12ResourcePtr smiley = MakeTexture(DXGI_FORMAT_R8G8B8A8_TYPELESS, 48, 48)
+                                   .Mips(1)
+                                   .InitialState(D3D12_RESOURCE_STATE_COPY_DEST);
+
+    ID3D12ResourcePtr uploadBuf = MakeBuffer().Size(1024 * 1024).Upload();
+    ID3D12ResourcePtr constBuf = MakeBuffer().Size(256).Upload();
+    ID3D12ResourcePtr outUAV = MakeBuffer().Size(256).UAV();
+    {
+      byte *mapptr = NULL;
+      constBuf->Map(0, NULL, (void **)&mapptr);
+      uint32_t value = 6;
+      memcpy(mapptr, &value, sizeof(uint32_t));
+      constBuf->Unmap(0, NULL);
+    }
+
+    {
+      D3D12_PLACED_SUBRESOURCE_FOOTPRINT layout = {};
+
+      D3D12_RESOURCE_DESC desc = smiley->GetDesc();
+
+      dev->GetCopyableFootprints(&desc, 0, 1, 0, &layout, NULL, NULL, NULL);
+
+      byte *srcptr = (byte *)rgba8.data.data();
+      byte *mapptr = NULL;
+      uploadBuf->Map(0, NULL, (void **)&mapptr);
+
+      ID3D12GraphicsCommandListPtr cmd = GetCommandBuffer();
+
+      Reset(cmd);
+
+      {
+        D3D12_TEXTURE_COPY_LOCATION dst, src;
+
+        dst.Type = D3D12_TEXTURE_COPY_TYPE_SUBRESOURCE_INDEX;
+        dst.pResource = smiley;
+        dst.SubresourceIndex = 0;
+
+        byte *dstptr = mapptr + layout.Offset;
+
+        for(UINT row = 0; row < rgba8.height; row++)
+        {
+          memcpy(dstptr, srcptr, rgba8.width * sizeof(uint32_t));
+          srcptr += rgba8.width * sizeof(uint32_t);
+          dstptr += layout.Footprint.RowPitch;
+        }
+
+        src.Type = D3D12_TEXTURE_COPY_TYPE_PLACED_FOOTPRINT;
+        src.pResource = uploadBuf;
+        src.PlacedFootprint = layout;
+
+        cmd->CopyTextureRegion(&dst, 0, 0, 0, &src, NULL);
+
+        D3D12_RESOURCE_BARRIER b = {};
+        b.Transition.pResource = smiley;
+        b.Transition.Subresource = 0;
+        b.Transition.StateBefore = D3D12_RESOURCE_STATE_COPY_DEST;
+        b.Transition.StateAfter = D3D12_RESOURCE_STATE_PIXEL_SHADER_RESOURCE |
+                                  D3D12_RESOURCE_STATE_NON_PIXEL_SHADER_RESOURCE;
+        cmd->ResourceBarrier(1, &b);
+      }
+
+      cmd->Close();
+
+      uploadBuf->Unmap(0, NULL);
+
+      Submit({cmd});
+      GPUSync();
+    }
+
+    MakeSRV(smiley).Format(DXGI_FORMAT_R8G8B8A8_UNORM).CreateGPU(5);
+    MakeSRV(smiley).Format(DXGI_FORMAT_R8G8B8A8_SINT).CreateGPU(6);
+    MakeSRV(smiley).Format(DXGI_FORMAT_R8G8B8A8_UINT).CreateGPU(7);
+
     ID3D12ResourcePtr rawBuf2 = MakeBuffer().Size(1024).UAV();
     D3D12ViewCreator uavView1 =
         MakeUAV(rawBuf2).Format(DXGI_FORMAT_R32_TYPELESS).ByteAddressed().FirstElement(4).NumElements(12);
-    D3D12_CPU_DESCRIPTOR_HANDLE uav1cpu = uavView1.CreateClearCPU(5);
-    D3D12_GPU_DESCRIPTOR_HANDLE uav1gpu = uavView1.CreateGPU(5);
+    D3D12_CPU_DESCRIPTOR_HANDLE uav1cpu = uavView1.CreateClearCPU(10);
+    D3D12_GPU_DESCRIPTOR_HANDLE uav1gpu = uavView1.CreateGPU(10);
 
     uint16_t narrowdata[32];
     for(size_t i = 0; i < ARRAY_COUNT(narrowdata); i++)
@@ -880,19 +1054,28 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
         .StructureStride(11 * sizeof(float))
         .CreateGPU(2);
 
+    ID3D12ResourcePtr rootStruct = MakeBuffer().Data(structdata);
+    MakeSRV(rootStruct)
+        .Format(DXGI_FORMAT_UNKNOWN)
+        .FirstElement(3)
+        .NumElements(5)
+        .StructureStride(11 * sizeof(float))
+        .CreateGPU(35);
+    ID3D12ResourcePtr rootDummy = MakeBuffer().Data(structdata);
+
     ID3D12ResourcePtr structBuf2 = MakeBuffer().Size(880).UAV();
     D3D12ViewCreator uavView2 = MakeUAV(structBuf2)
                                     .Format(DXGI_FORMAT_UNKNOWN)
                                     .FirstElement(3)
                                     .NumElements(5)
                                     .StructureStride(11 * sizeof(float));
-    D3D12_CPU_DESCRIPTOR_HANDLE uav2cpu = uavView2.CreateClearCPU(6);
-    D3D12_GPU_DESCRIPTOR_HANDLE uav2gpu = uavView2.CreateGPU(6);
+    D3D12_CPU_DESCRIPTOR_HANDLE uav2cpu = uavView2.CreateClearCPU(11);
+    D3D12_GPU_DESCRIPTOR_HANDLE uav2gpu = uavView2.CreateGPU(11);
 
     // need to create non-structured version for clearing
     uavView2 = MakeUAV(structBuf2).Format(DXGI_FORMAT_R32_UINT);
-    uav2cpu = uavView2.CreateClearCPU(8);
-    uav2gpu = uavView2.CreateGPU(8);
+    uav2cpu = uavView2.CreateClearCPU(9);
+    uav2gpu = uavView2.CreateGPU(9);
 
     // Create resources for MSAA draw
     ID3DBlobPtr vsmsaablob = Compile(D3DDefaultVertex, "main", "vs_5_0");
@@ -919,7 +1102,7 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
     psblob = Compile(pixelBlit, "main", "ps_5_0");
     ID3D12RootSignaturePtr blitSig = MakeSig({
         constParam(D3D12_SHADER_VISIBILITY_PIXEL, 0, 0, 1),
-        tableParam(D3D12_SHADER_VISIBILITY_PIXEL, D3D12_DESCRIPTOR_RANGE_TYPE_SRV, 0, 0, 1, 7),
+        tableParam(D3D12_SHADER_VISIBILITY_PIXEL, D3D12_DESCRIPTOR_RANGE_TYPE_SRV, 0, 0, 1, 8),
     });
     ID3D12PipelineStatePtr blitpso = MakePSO().RootSig(blitSig).VS(vsblob).PS(psblob);
 
@@ -927,7 +1110,7 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
     psblob = Compile(vertexSamplePS, "main", "ps_5_0");
     ID3D12RootSignaturePtr vertexSampleSig = MakeSig(
         {
-            tableParam(D3D12_SHADER_VISIBILITY_VERTEX, D3D12_DESCRIPTOR_RANGE_TYPE_SRV, 0, 0, 1, 7),
+            tableParam(D3D12_SHADER_VISIBILITY_VERTEX, D3D12_DESCRIPTOR_RANGE_TYPE_SRV, 0, 0, 1, 8),
         },
         D3D12_ROOT_SIGNATURE_FLAG_DENY_PIXEL_SHADER_ROOT_ACCESS);
     ID3D12PipelineStatePtr vertexSamplePSO = MakePSO().RootSig(vertexSampleSig).VS(vsblob).PS(psblob);
@@ -1000,6 +1183,8 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
           MakeRTV(bb).Format(DXGI_FORMAT_R8G8B8A8_UNORM_SRGB).CreateCPU(2);
       ClearRenderTargetView(cmd, rtv, {0.2f, 0.2f, 0.2f, 1.0f});
 
+      setMarker(cmd, undefined_tests);
+
       ID3D12PipelineStatePtr psos[2] = {pso_5_0, pso_5_1};
       float blitOffsets[2] = {0.0f, 4.0f};
       D3D12_RECT scissors[2] = {{0, 0, (int)texDim, 4}, {0, 4, (int)texDim, 8}};
@@ -1020,6 +1205,10 @@ float4 main(v2f IN, uint samp : SV_SampleIndex) : SV_Target0
         cmd->SetGraphicsRootDescriptorTable(1, m_CBVUAVSRV->GetGPUDescriptorHandleForHeapStart());
         cmd->SetGraphicsRootDescriptorTable(2, m_CBVUAVSRV->GetGPUDescriptorHandleForHeapStart());
         cmd->SetGraphicsRootDescriptorTable(3, m_CBVUAVSRV->GetGPUDescriptorHandleForHeapStart());
+        cmd->SetGraphicsRootDescriptorTable(4, m_CBVUAVSRV->GetGPUDescriptorHandleForHeapStart());
+        cmd->SetGraphicsRootUnorderedAccessView(5, rootDummy->GetGPUVirtualAddress());
+        cmd->SetGraphicsRootShaderResourceView(
+            6, rootStruct->GetGPUVirtualAddress() + sizeof(float) * 22);
 
         cmd->SetPipelineState(psos[i]);
 
