@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2019-2021 Baldur Karlsson
+* Copyright (c) 2019-2022 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -40,7 +40,7 @@ struct OpenGLGraphicsTest : public GraphicsTest
   GraphicsWindow *MakeWindow(int width, int height, const char *title);
   void *MakeContext(GraphicsWindow *win, void *share);
   void DestroyContext(void *ctx);
-  void ActivateContext(GraphicsWindow *win, void *ctx);
+  void ActivateContext(GraphicsWindow *win, void *ctx, bool alt = false);
 
   void PostInit();
 
