@@ -214,6 +214,68 @@ void NuklearShutdown()
   XCloseDisplay(dpy);
 }
 
+#elif defined(__APPLE__)
+
+#include "apple/official/metal-cpp.h"
+extern "C" void *const NSDefaultRunLoopMode;
+
+#define NK_APPKIT_IMPLEMENTATION
+#include "apple/nuklear_appkit.h"
+
+static nk_appkit_window *window;
+static AppkitFont *font;
+static NS::Application *pSharedApplication = NULL;
+
+nk_context *NuklearInit(int width, int height, const char *title)
+{
+  if(!nk_appkit_core_initialize())
+    exit(EXIT_FAILURE);
+
+  pSharedApplication = NS::Application::sharedApplication();
+
+  window = nk_appkit_window_create(width, height, title);
+  if(!window)
+    exit(EXIT_FAILURE);
+
+  nk_context *ctx = nk_appkit_create(window);
+  font = nk_appkit_create_font("Menlo Regular", 9);
+  nk_appkit_init(font);
+  return ctx;
+}
+
+bool NuklearTick(nk_context *ctx)
+{
+  NS::AutoreleasePool *pAutoreleasePool = NS::AutoreleasePool::alloc()->init();
+  while(true)
+  {
+    NS::Event *event = pSharedApplication->nextEventMatchingMask(
+        (int)NS::EventMaskAny, NS::Date::distantPast(), (NS::String *)NSDefaultRunLoopMode, true);
+    if(event == NULL)
+      break;
+    pSharedApplication->sendEvent(event);
+  }
+  pAutoreleasePool->release();
+  nk_appkit_new_frame();
+
+  if(nk_appkit_window_is_closed(window))
+    return false;
+
+  return true;
+}
+
+void NuklearRender()
+{
+  nk_appkit_render(nk_rgb(30, 30, 30));
+}
+
+void NuklearShutdown()
+{
+  nk_appkit_delete_font(font);
+  nk_appkit_shutdown();
+  nk_appkit_window_delete(window);
+  nk_appkit_core_shutdown();
+}
+
 #else
 
 #error UNKNOWN PLATFORM
