@@ -87,6 +87,23 @@ void main()
 	Color = vec4(0, 1, 0, 1) * textureLod(tex, vec2(0), 0);
 }
 
+)EOSHADER";
+
+  const std::string dynarraypixel = R"EOSHADER(
+#version 450 core
+
+layout(location = 0, index = 0) out vec4 Color;
+
+layout(binding=0) uniform myUniformBuffer
+{
+  vec4 data;
+} ubos[8];
+
+void main()
+{
+	Color = ubos[7].data;
+}
+
 )EOSHADER";
 
   const std::string refpixel = R"EOSHADER(
@@ -251,6 +268,9 @@ void main()
     optDevExts.push_back(VK_EXT_TRANSFORM_FEEDBACK_EXTENSION_NAME);
     optDevExts.push_back(VK_KHR_TIMELINE_SEMAPHORE_EXTENSION_NAME);
     optDevExts.push_back(VK_KHR_BIND_MEMORY_2_EXTENSION_NAME);
+    optDevExts.push_back(VK_KHR_MULTIVIEW_EXTENSION_NAME);
+    optDevExts.push_back(VK_KHR_MAINTENANCE2_EXTENSION_NAME);
+    optDevExts.push_back(VK_KHR_SYNCHRONIZATION_2_EXTENSION_NAME);
 
     VulkanGraphicsTest::Prepare(argc, argv);
 
@@ -258,8 +278,7 @@ void main()
         VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TIMELINE_SEMAPHORE_FEATURES_KHR,
     };
 
-    if(std::find(devExts.begin(), devExts.end(), VK_KHR_TIMELINE_SEMAPHORE_EXTENSION_NAME) !=
-       devExts.end())
+    if(hasExt(VK_KHR_TIMELINE_SEMAPHORE_EXTENSION_NAME))
     {
       timeline.timelineSemaphore = VK_TRUE;
       timeline.pNext = (void *)devInfoNext;
@@ -291,13 +310,34 @@ void main()
         VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT,
     };
 
-    if(std::find(devExts.begin(), devExts.end(), VK_EXT_TRANSFORM_FEEDBACK_EXTENSION_NAME) !=
-       devExts.end())
+    if(hasExt(VK_EXT_TRANSFORM_FEEDBACK_EXTENSION_NAME))
     {
       xfbFeats.transformFeedback = VK_TRUE;
       xfbFeats.pNext = (void *)devInfoNext;
       devInfoNext = &xfbFeats;
     }
+
+    static VkPhysicalDeviceMultiviewFeatures multiview = {
+        VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_FEATURES,
+    };
+
+    if(hasExt(VK_KHR_MULTIVIEW_EXTENSION_NAME))
+    {
+      multiview.multiview = VK_TRUE;
+      multiview.pNext = (void *)devInfoNext;
+      devInfoNext = &multiview;
+    }
+
+    static VkPhysicalDeviceSynchronization2FeaturesKHR sync2Features = {
+        VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SYNCHRONIZATION_2_FEATURES_KHR,
+    };
+
+    if(hasExt(VK_KHR_SYNCHRONIZATION_2_EXTENSION_NAME))
+    {
+      sync2Features.synchronization2 = VK_TRUE;
+      sync2Features.pNext = (void *)devInfoNext;
+      devInfoNext = &sync2Features;
+    }
   }
 
   int main()
@@ -347,6 +387,27 @@ void main()
       }
     }
 
+    {
+      uint32_t physCount = 0;
+      vkEnumeratePhysicalDevices(instance, &physCount, NULL);
+
+      if(physCount > 1)
+      {
+        std::vector<VkPhysicalDevice> physArray;
+        physArray.resize(physCount);
+
+        physCount = 1;
+        VkResult vkr = vkEnumeratePhysicalDevices(instance, &physCount, physArray.data());
+
+        if(vkr != VK_INCOMPLETE || physCount != 1 || physArray[1] != VK_NULL_HANDLE)
+        {
+          TEST_ERROR(
+              "vkEnumeratePhysicalDevices didn't return correct results for truncated array");
+          return 3;
+        }
+      }
+    }
+
     bool KHR_descriptor_update_template =
         std::find(devExts.begin(), devExts.end(),
                   VK_KHR_DESCRIPTOR_UPDATE_TEMPLATE_EXTENSION_NAME) != devExts.end();
@@ -466,6 +527,14 @@ void main()
 
     VkPipelineLayout layout2 = createPipelineLayout(vkh::PipelineLayoutCreateInfo({setlayout2}));
 
+    VkDescriptorSetLayout dynamicarraysetlayout =
+        createDescriptorSetLayout(vkh::DescriptorSetLayoutCreateInfo({
+            {0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC, 8, VK_SHADER_STAGE_FRAGMENT_BIT},
+        }));
+
+    VkPipelineLayout dynamicarraylayout =
+        createPipelineLayout(vkh::PipelineLayoutCreateInfo({dynamicarraysetlayout}));
+
     VkDescriptorSet descset2 = allocateDescriptorSet(setlayout2);
 
     VkDescriptorSetLayout asm_setlayout =
@@ -484,10 +553,61 @@ void main()
     VkPipelineLayout asm_layout = createPipelineLayout(vkh::PipelineLayoutCreateInfo(
         {asm_setlayout}, {vkh::PushConstantRange(VK_SHADER_STAGE_VERTEX_BIT, 0, 4)}));
 
+    vkh::RenderPassCreator renderPassCreateInfo;
+
+    renderPassCreateInfo.attachments.push_back(vkh::AttachmentDescription(
+        mainWindow->format, VK_IMAGE_LAYOUT_GENERAL, VK_IMAGE_LAYOUT_GENERAL));
+    renderPassCreateInfo.attachments.push_back(vkh::AttachmentDescription(
+        VK_FORMAT_R32G32B32A32_SFLOAT, VK_IMAGE_LAYOUT_GENERAL, VK_IMAGE_LAYOUT_GENERAL));
+
+    // make the renderpass multi-pass for testing, and give each one an input attachment
+    renderPassCreateInfo.addSubpass({VkAttachmentReference({0, VK_IMAGE_LAYOUT_GENERAL})},
+                                    VK_ATTACHMENT_UNUSED, VK_IMAGE_LAYOUT_UNDEFINED, {},
+                                    {VkAttachmentReference({1, VK_IMAGE_LAYOUT_GENERAL})});
+    renderPassCreateInfo.addSubpass({VkAttachmentReference({0, VK_IMAGE_LAYOUT_GENERAL})},
+                                    VK_ATTACHMENT_UNUSED, VK_IMAGE_LAYOUT_UNDEFINED, {},
+                                    {VkAttachmentReference({1, VK_IMAGE_LAYOUT_GENERAL})});
+
+    renderPassCreateInfo.dependencies.push_back(vkh::SubpassDependency(
+        0, 1, VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT,
+        VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT, VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT,
+        VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT));
+
+    // add pointless structures to ensure they are ignored
+    VkRenderPassMultiviewCreateInfo nonMultiview = {
+        VK_STRUCTURE_TYPE_RENDER_PASS_MULTIVIEW_CREATE_INFO,
+    };
+
+    if(hasExt(VK_KHR_MULTIVIEW_EXTENSION_NAME))
+    {
+      nonMultiview.pNext = renderPassCreateInfo.pNext;
+      renderPassCreateInfo.pNext = &nonMultiview;
+    }
+
+    // add struct that references input attachments in multiple passes
+    VkRenderPassInputAttachmentAspectCreateInfo inputAspects = {
+        VK_STRUCTURE_TYPE_RENDER_PASS_INPUT_ATTACHMENT_ASPECT_CREATE_INFO,
+    };
+
+    VkInputAttachmentAspectReference inputAspectReferences[2] = {
+        {0, 0, VK_IMAGE_ASPECT_COLOR_BIT}, {1, 0, VK_IMAGE_ASPECT_COLOR_BIT},
+    };
+
+    inputAspects.aspectReferenceCount = 2;
+    inputAspects.pAspectReferences = inputAspectReferences;
+
+    if(hasExt(VK_KHR_MAINTENANCE2_EXTENSION_NAME))
+    {
+      inputAspects.pNext = renderPassCreateInfo.pNext;
+      renderPassCreateInfo.pNext = &inputAspects;
+    }
+
+    VkRenderPass renderPass = createRenderPass(renderPassCreateInfo);
+
     vkh::GraphicsPipelineCreateInfo pipeCreateInfo;
 
     pipeCreateInfo.layout = layout;
-    pipeCreateInfo.renderPass = mainWindow->rp;
+    pipeCreateInfo.renderPass = renderPass;
 
     pipeCreateInfo.vertexInputState.vertexBindingDescriptions = {vkh::vertexBind(0, DefaultA2V)};
     pipeCreateInfo.vertexInputState.vertexAttributeDescriptions = {
@@ -515,6 +635,15 @@ void main()
 
     VkPipeline immutpipe = createGraphicsPipeline(pipeCreateInfo);
 
+    pipeCreateInfo.layout = dynamicarraylayout;
+
+    pipeCreateInfo.stages = {
+        CompileShaderModule(VKDefaultVertex, ShaderLang::glsl, ShaderStage::vert, "main"),
+        CompileShaderModule(dynarraypixel, ShaderLang::glsl, ShaderStage::frag, "main"),
+    };
+
+    VkPipeline dynarraypipe = createGraphicsPipeline(pipeCreateInfo);
+
     pipeCreateInfo.stages = {
         CompileShaderModule(VKDefaultVertex, ShaderLang::glsl, ShaderStage::vert, "main"),
         CompileShaderModule(pixel2, ShaderLang::glsl, ShaderStage::frag, "main"),
@@ -674,6 +803,7 @@ void main()
     VkDescriptorSet reftempldescset = allocateDescriptorSet(refsetlayout);
 
     VkDescriptorSet immutdescset = allocateDescriptorSet(immutsetlayout);
+    VkDescriptorSet dynamicarrayset = allocateDescriptorSet(dynamicarraysetlayout);
 
     AllocatedBuffer buf(this,
                         vkh::BufferCreateInfo(1024, VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT |
@@ -682,6 +812,13 @@ void main()
                                                         VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT),
                         VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_CPU_TO_GPU}));
 
+    float bufData[1024 / sizeof(float)] = {};
+    bufData[64] = 0.0f;
+    bufData[65] = 1.0f;
+    bufData[66] = 0.0f;
+    bufData[67] = 1.0f;
+    buf.upload(bufData);
+
     VkBuffer invalidBuffer = (VkBuffer)0x1234;
     VkBuffer validBuffer = buf.buffer;
 
@@ -697,6 +834,21 @@ void main()
         vkh::ImageViewCreateInfo(validImage, VK_IMAGE_VIEW_TYPE_2D, VK_FORMAT_R32G32B32A32_SFLOAT));
     VkImageView invalidImgView = (VkImageView)0x1234;
 
+    AllocatedImage inputattach(
+        this,
+        vkh::ImageCreateInfo(mainWindow->scissor.extent.width, mainWindow->scissor.extent.height, 0,
+                             VK_FORMAT_R32G32B32A32_SFLOAT,
+                             VK_IMAGE_USAGE_TRANSFER_DST_BIT | VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT),
+        VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_GPU_ONLY}));
+
+    VkImageView inputview = createImageView(vkh::ImageViewCreateInfo(
+        inputattach.image, VK_IMAGE_VIEW_TYPE_2D, VK_FORMAT_R32G32B32A32_SFLOAT));
+
+    VkFramebuffer fbs[8];
+    for(size_t i = 0; i < mainWindow->GetCount(); i++)
+      fbs[i] = createFramebuffer(vkh::FramebufferCreateInfo(
+          renderPass, {mainWindow->GetView(i), inputview}, mainWindow->scissor.extent));
+
     {
       VkCommandBuffer cmd = GetCommandBuffer();
       vkBeginCommandBuffer(cmd, vkh::CommandBufferBeginInfo());
@@ -712,6 +864,9 @@ void main()
           {
               vkh::ImageMemoryBarrier(VK_ACCESS_TRANSFER_WRITE_BIT, VK_ACCESS_SHADER_READ_BIT,
                                       VK_IMAGE_LAYOUT_GENERAL, VK_IMAGE_LAYOUT_GENERAL, img.image),
+              vkh::ImageMemoryBarrier(VK_ACCESS_NONE, VK_ACCESS_INPUT_ATTACHMENT_READ_BIT,
+                                      VK_IMAGE_LAYOUT_UNDEFINED, VK_IMAGE_LAYOUT_GENERAL,
+                                      inputattach.image),
           });
       vkEndCommandBuffer(cmd);
       Submit(99, 99, {cmd});
@@ -753,8 +908,29 @@ void main()
                                 validCombinedImgs),
         vkh::WriteDescriptorSet(immutdescset, 0, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER,
                                 validSoloImgs),
+        vkh::WriteDescriptorSet(dynamicarrayset, 0, 0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
+                                validBufInfos),
+        vkh::WriteDescriptorSet(dynamicarrayset, 0, 1, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
+                                validBufInfos),
+        vkh::WriteDescriptorSet(dynamicarrayset, 0, 2, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
+                                validBufInfos),
+        vkh::WriteDescriptorSet(dynamicarrayset, 0, 3, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
+                                validBufInfos),
+        vkh::WriteDescriptorSet(dynamicarrayset, 0, 4, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
+                                validBufInfos),
+        vkh::WriteDescriptorSet(dynamicarrayset, 0, 5, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
+                                validBufInfos),
+        vkh::WriteDescriptorSet(dynamicarrayset, 0, 6, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
+                                validBufInfos),
     };
 
+    vkh::updateDescriptorSets(
+        device,
+        {
+            vkh::WriteDescriptorSet(dynamicarrayset, 0, 7, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC,
+                                    {vkh::DescriptorBufferInfo(validBuffer, 0, 256)}),
+        });
+
     // do a first update
     vkh::updateDescriptorSets(device, writes);
 
@@ -1349,9 +1525,9 @@ void main()
         if(KHR_push_descriptor)
           setMarker(cmd, "KHR_push_descriptor");
 
-        vkCmdBeginRenderPass(
-            cmd, vkh::RenderPassBeginInfo(mainWindow->rp, mainWindow->GetFB(), mainWindow->scissor),
-            VK_SUBPASS_CONTENTS_INLINE);
+        vkCmdBeginRenderPass(cmd, vkh::RenderPassBeginInfo(renderPass, fbs[mainWindow->imgIndex],
+                                                           mainWindow->scissor),
+                             VK_SUBPASS_CONTENTS_INLINE);
 
         vkCmdBindPipeline(cmd, VK_PIPELINE_BIND_POINT_GRAPHICS, refpipe);
 
@@ -1407,6 +1583,8 @@ void main()
           vkCmdDraw(cmd, 1, 1, 0, 0);
         }
 
+        vkCmdNextSubpass(cmd, VK_SUBPASS_CONTENTS_INLINE);
+
         vkCmdEndRenderPass(cmd);
 
         vkEndCommandBuffer(cmd);
@@ -1461,9 +1639,9 @@ void main()
                              vkh::ClearColorValue(0.2f, 0.2f, 0.2f, 1.0f), 1,
                              vkh::ImageSubresourceRange());
 
-        vkCmdBeginRenderPass(
-            cmd, vkh::RenderPassBeginInfo(mainWindow->rp, mainWindow->GetFB(), mainWindow->scissor),
-            VK_SUBPASS_CONTENTS_INLINE);
+        vkCmdBeginRenderPass(cmd, vkh::RenderPassBeginInfo(renderPass, fbs[mainWindow->imgIndex],
+                                                           mainWindow->scissor),
+                             VK_SUBPASS_CONTENTS_INLINE);
 
         if(!tools.empty())
         {
@@ -1504,6 +1682,13 @@ void main()
         setMarker(cmd, "Immutable Draw");
         vkCmdDraw(cmd, 3, 1, 0, 0);
 
+        vkCmdBindPipeline(cmd, VK_PIPELINE_BIND_POINT_GRAPHICS, dynarraypipe);
+        vkh::cmdBindDescriptorSets(cmd, VK_PIPELINE_BIND_POINT_GRAPHICS, dynamicarraylayout, 0,
+                                   {dynamicarrayset}, {0, 0, 0, 0, 0, 0, 0, 256});
+
+        setMarker(cmd, "Dynamic Array Draw");
+        vkCmdDraw(cmd, 3, 1, 0, 0);
+
         vkCmdBindPipeline(cmd, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe2);
         vkh::cmdBindDescriptorSets(cmd, VK_PIPELINE_BIND_POINT_GRAPHICS, layout2, 0, {descset2}, {});
 
@@ -1526,6 +1711,8 @@ void main()
           vkCmdEndTransformFeedbackEXT(cmd, 0, 0, NULL, NULL);
         }
 
+        vkCmdNextSubpass(cmd, VK_SUBPASS_CONTENTS_INLINE);
+
         vkCmdEndRenderPass(cmd);
 
         vkEndCommandBuffer(cmd);
@@ -1548,6 +1735,21 @@ void main()
         Submit(3, 4, {cmd});
       }
 
+      // make some empty submits
+
+      setMarker(queue, "before_empty");
+
+      {
+        VkSubmitInfo submit = vkh::SubmitInfo({});
+        CHECK_VKR(vkQueueSubmit(queue, 1, &submit, VK_NULL_HANDLE));
+      }
+      if(hasExt(VK_KHR_SYNCHRONIZATION_2_EXTENSION_NAME))
+      {
+        VkSubmitInfo2KHR submit = {VK_STRUCTURE_TYPE_SUBMIT_INFO_2_KHR};
+        CHECK_VKR(vkQueueSubmit2KHR(queue, 1, &submit, VK_NULL_HANDLE));
+      }
+      setMarker(queue, "after_empty");
+
       Present();
     }
 
