@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2019-2021 Baldur Karlsson
+* Copyright (c) 2019-2022 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -68,7 +68,7 @@ void regClass()
   }
 }
 
-Win32Window::Win32Window(int width, int height, const char *title)
+Win32Window::Win32Window(int width, int height, const char *title) : GraphicsWindow(title)
 {
   regClass();
 
