@@ -7,13 +7,13 @@ class D3D11_Shader_ISA(rdtest.TestCase):
     demos_test_name = 'D3D11_Shader_ISA'
 
     def check_capture(self):
-        draw = self.find_draw("GPU=")
+        action = self.find_action("GPU=")
 
-        self.check(draw is not None)
+        self.check(action is not None)
 
-        is_amd = 'AMD' in draw.name
+        is_amd = 'AMD' in action.customName
 
-        self.controller.SetFrameEvent(draw.next.eventId, False)
+        self.controller.SetFrameEvent(action.next.eventId, False)
 
         pipe: rd.PipeState = self.controller.GetPipelineState()
 
@@ -52,11 +52,11 @@ class D3D11_Shader_ISA(rdtest.TestCase):
                 raise rdtest.TestFailureException(
                     "AMDIL ISA doesn't contain '{}' as expected: {}".format(fragment, disasm))
 
-        if 'RDNA (Navi 10)' not in isas:
+        if 'RDNA (gfx1010)' not in isas:
             raise rdtest.TestFailureException(
-                "RDNA (Navi 10) is not an available disassembly target. Are you missing plugins?")
+                "RDNA (gfx1010) is not an available disassembly target. Are you missing plugins?")
 
-        disasm: str = self.controller.DisassembleShader(pipe.GetGraphicsPipelineObject(), refl, 'RDNA (Navi 10)')
+        disasm: str = self.controller.DisassembleShader(pipe.GetGraphicsPipelineObject(), refl, 'RDNA (gfx1010)')
 
         expected = [
             'asic(GFX10)',
