@@ -44,12 +44,12 @@ class GL_Buffer_Updates(rdtest.TestCase):
         cap = rd.OpenCaptureFile()
 
         # Open a particular file
-        status = cap.OpenFile(self.capture_filename, '', None)
+        result = cap.OpenFile(self.capture_filename, '', None)
 
         # Make sure the file opened successfully
-        if status != rd.ReplayStatus.Succeeded:
+        if result != rd.ResultCode.Succeeded:
             cap.Shutdown()
-            raise rdtest.TestFailureException("Couldn't open '{}': {}".format(self.capture_filename, str(status)))
+            raise rdtest.TestFailureException("Couldn't open '{}': {}".format(self.capture_filename, str(result)))
 
         thumb: rd.Thumbnail = cap.GetThumbnail(rd.FileType.PNG, 0)
 
