@@ -7,9 +7,9 @@ class GL_Unshared_Context(rdtest.TestCase):
 
     def check_capture(self):
         
-        draw = self.find_draw("Draw")
+        action = self.find_action("Draw")
 
-        self.controller.SetFrameEvent(draw.eventId, False)
+        self.controller.SetFrameEvent(action.eventId, False)
 
         pipe: rd.PipeState = self.controller.GetPipelineState()
 
