@@ -3,6 +3,10 @@ import struct
 import renderdoc as rd
 
 
+def real_action_children(action):
+    return [c for c in action.children if not c.flags & rd.ActionFlags.PopMarker]
+
+
 class VK_Indirect(rdtest.TestCase):
     demos_test_name = 'VK_Indirect'
 
@@ -71,7 +75,7 @@ class VK_Indirect(rdtest.TestCase):
 
         # Every sample that isn't passing should be off
         off_alpha = 0.5
-        # If the overlay isn't even for a draw, it will be cleared to black
+        # If the overlay isn't even for a action, it will be cleared to black
         if no_overlay:
             off_alpha = 0.0
             self.check(len(pass_samples) == 0)
@@ -83,7 +87,7 @@ class VK_Indirect(rdtest.TestCase):
             self.check_pixel_value(overlay_id, s[0], s[1], [0.8, 0.1, 0.8, 1.0], eps=1.0/256.0)
 
     def check_capture(self):
-        fill = self.find_draw("vkCmdFillBuffer")
+        fill = self.find_action("vkCmdFillBuffer")
 
         self.check(fill is not None)
 
@@ -96,14 +100,14 @@ class VK_Indirect(rdtest.TestCase):
             buffer_usage[usage.eventId].append(usage.usage)
 
         # The texture is the backbuffer
-        tex = self.get_last_draw().copyDestination
+        tex = self.get_last_action().copyDestination
 
         for level in ["Primary", "Secondary"]:
             rdtest.log.print("Checking {} indirect calls".format(level))
 
-            final = self.find_draw("{}: Final".format(level))
+            final = self.find_action("{}: Final".format(level))
 
-            indirect_count_root = self.find_draw("{}: KHR_draw_indirect_count".format(level))
+            indirect_count_root = self.find_action("{}: KHR_action_indirect_count".format(level))
 
             self.controller.SetFrameEvent(final.eventId, False)
 
@@ -124,31 +128,31 @@ class VK_Indirect(rdtest.TestCase):
                 self.check_pixel_value(tex, 340, 115, [1.0, 0.5, 0.5, 1.0])
                 self.check_pixel_value(tex, 340, 190, [1.0, 0.0, 0.5, 1.0])
 
-            dispatches = self.find_draw("{}: Dispatches".format(level))
+            dispatches = self.find_action("{}: Dispatches".format(level))
 
-            # Set up a ReplayOutput and TextureSave for quickly testing the drawcall highlight overlay
+            # Set up a ReplayOutput and TextureSave for quickly testing the action highlight overlay
             self.out: rd.ReplayOutput = self.controller.CreateOutput(rd.CreateHeadlessWindowingData(100, 100),
                                                                      rd.ReplayOutputType.Texture)
 
             self.check(self.out is not None)
 
             # Rewind to the start of the capture
-            draw: rd.DrawcallDescription = dispatches.children[0]
-            while draw.previous is not None:
-                draw = draw.previous
+            action: rd.ActionDescription = dispatches.children[0]
+            while action.previous is not None:
+                action = action.previous
 
-            # Ensure we can select all draws
-            while draw is not None:
-                self.controller.SetFrameEvent(draw.eventId, False)
-                draw = draw.next
+            # Ensure we can select all actions
+            while action is not None:
+                self.controller.SetFrameEvent(action.eventId, False)
+                action = action.next
 
-            rdtest.log.success("Selected all {} draws".format(level))
+            rdtest.log.success("Selected all {} actions".format(level))
 
-            self.check(dispatches and len(dispatches.children) == 3)
+            self.check(dispatches and len(real_action_children(dispatches)) == 3)
 
-            self.check(dispatches.children[0].dispatchDimension == [0,0,0])
-            self.check(dispatches.children[1].dispatchDimension == [1,1,1])
-            self.check(dispatches.children[2].dispatchDimension == [3,4,5])
+            self.check(dispatches.children[0].dispatchDimension == (0, 0, 0))
+            self.check(dispatches.children[1].dispatchDimension == (1, 1, 1))
+            self.check(dispatches.children[2].dispatchDimension == (3, 4, 5))
 
             rdtest.log.success("{} Indirect dispatches are the correct dimensions".format(level))
 
@@ -174,45 +178,45 @@ class VK_Indirect(rdtest.TestCase):
 
             rdtest.log.success("Dispatched buffer contents are as expected for {}".format(level))
 
-            empties = self.find_draw("{}: Empty draws".format(level))
+            empties = self.find_action("{}: Empty draws".format(level))
 
-            self.check(empties and len(empties.children) == 2)
+            self.check(empties and len(real_action_children(empties)) == 2)
 
-            draw: rd.DrawcallDescription
-            for draw in empties.children:
-                self.check(draw.numIndices == 0)
-                self.check(draw.numInstances == 0)
+            action: rd.ActionDescription
+            for action in real_action_children(empties):
+                self.check(action.numIndices == 0)
+                self.check(action.numInstances == 0)
 
-                self.controller.SetFrameEvent(draw.eventId, False)
+                self.controller.SetFrameEvent(action.eventId, False)
 
                 # Check that we have empty PostVS
-                postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut, 0, 1)
+                postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut, 0, 1)
                 self.check(len(postvs_data) == 0)
 
                 # No samples should be passing in the empties
                 self.check_overlay([])
 
-            rdtest.log.success("{} empty draws are empty".format(level))
+            rdtest.log.success("{} empty actions are empty".format(level))
 
-            indirects = self.find_draw("{}: Indirect draws".format(level))
+            indirects = self.find_action("{}: Indirect draws".format(level))
 
-            self.check('vkCmdDrawIndirect' in indirects.children[0].name)
-            self.check('vkCmdDrawIndexedIndirect' in indirects.children[1].name)
-            self.check(len(indirects.children[1].children) == 2)
+            self.check('vkCmdDrawIndirect' in indirects.children[0].customName)
+            self.check('vkCmdDrawIndexedIndirect' in indirects.children[1].customName)
+            self.check(len(real_action_children(indirects.children[1])) == 2)
 
             rdtest.log.success("Correct number of {} indirect draws".format(level))
 
             # vkCmdDrawIndirect(...)
-            draw = indirects.children[0]
-            self.check(draw.numIndices == 3)
-            self.check(draw.numInstances == 2)
+            action = indirects.children[0]
+            self.check(action.numIndices == 3)
+            self.check(action.numInstances == 2)
 
-            self.controller.SetFrameEvent(draw.eventId, False)
+            self.controller.SetFrameEvent(action.eventId, False)
 
-            self.check(rd.ResourceUsage.Indirect in buffer_usage[draw.eventId])
+            self.check(rd.ResourceUsage.Indirect in buffer_usage[action.eventId])
 
             # Check that we have PostVS as expected
-            postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut)
+            postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut)
 
             postvs_ref = {
                 0: {'vtx': 0, 'idx': 0, 'gl_PerVertex_var.gl_Position': [-0.8, -0.5, 0.0, 1.0]},
@@ -225,19 +229,19 @@ class VK_Indirect(rdtest.TestCase):
 
             self.check_overlay([(60, 40)])
 
-            rdtest.log.success("{} {} is as expected".format(level, draw.name))
+            rdtest.log.success("{} {} is as expected".format(level, action.customName))
 
             self.check(rd.ResourceUsage.Indirect in buffer_usage[indirects.children[1].eventId])
 
             # vkCmdDrawIndexedIndirect[0](...)
-            draw = indirects.children[1].children[0]
-            self.check(draw.numIndices == 3)
-            self.check(draw.numInstances == 3)
+            action = indirects.children[1].children[0]
+            self.check(action.numIndices == 3)
+            self.check(action.numInstances == 3)
 
-            self.controller.SetFrameEvent(draw.eventId, False)
+            self.controller.SetFrameEvent(action.eventId, False)
 
             # Check that we have PostVS as expected
-            postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut)
+            postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut)
 
             # These indices are the *output* indices, which have been rebased/remapped, so are not the same as the input
             # indices
@@ -252,17 +256,17 @@ class VK_Indirect(rdtest.TestCase):
 
             self.check_overlay([(100, 40)])
 
-            rdtest.log.success("{} {} is as expected".format(level, draw.name))
+            rdtest.log.success("{} {} is as expected".format(level, action.customName))
 
             # vkCmdDrawIndexedIndirect[1](...)
-            draw = indirects.children[1].children[1]
-            self.check(draw.numIndices == 6)
-            self.check(draw.numInstances == 2)
+            action = indirects.children[1].children[1]
+            self.check(action.numIndices == 6)
+            self.check(action.numInstances == 2)
 
-            self.controller.SetFrameEvent(draw.eventId, False)
+            self.controller.SetFrameEvent(action.eventId, False)
 
             # Check that we have PostVS as expected
-            postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut)
+            postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut)
 
             postvs_ref = {
                 0: {'vtx': 0, 'idx': 9, 'gl_PerVertex_var.gl_Position': [-0.4, -0.5, 0.0, 1.0]},
@@ -279,46 +283,46 @@ class VK_Indirect(rdtest.TestCase):
 
             self.check_overlay([(140, 40), (200, 40)])
 
-            rdtest.log.success("{} {} is as expected".format(level, draw.name))
+            rdtest.log.success("{} {} is as expected".format(level, action.customName))
 
             if indirect_count_root is not None:
-                self.check(indirect_count_root.children[0].name == '{}: Empty count draws'.format(level))
-                self.check(indirect_count_root.children[1].name == '{}: Indirect count draws'.format(level))
+                self.check(indirect_count_root.children[0].customName == '{}: Empty count draws'.format(level))
+                self.check(indirect_count_root.children[1].customName == '{}: Indirect count draws'.format(level))
 
                 empties = indirect_count_root.children[0]
 
-                self.check(empties and len(empties.children) == 3)
+                self.check(empties and len(real_action_children(empties)) == 3)
 
-                draw: rd.DrawcallDescription
-                for draw in empties.children:
-                    self.check(draw.numIndices == 0)
-                    self.check(draw.numInstances == 0)
+                action: rd.ActionDescription
+                for action in real_action_children(empties.children):
+                    self.check(action.numIndices == 0)
+                    self.check(action.numInstances == 0)
 
-                    self.controller.SetFrameEvent(draw.eventId, False)
+                    self.controller.SetFrameEvent(action.eventId, False)
 
                     # Check that we have empty PostVS
-                    postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut, 0, 1)
+                    postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut, 0, 1)
                     self.check(len(postvs_data) == 0)
 
                     self.check_overlay([], no_overlay=True)
 
                 # vkCmdDrawIndirectCountKHR
-                draw_indirect = indirect_count_root.children[1].children[0]
+                action_indirect = indirect_count_root.children[1].children[0]
 
-                self.check(rd.ResourceUsage.Indirect in buffer_usage[draw_indirect.eventId])
+                self.check(rd.ResourceUsage.Indirect in buffer_usage[action_indirect.eventId])
 
-                self.check(draw_indirect and len(draw_indirect.children) == 1)
+                self.check(action_indirect and len(real_action_children(action_indirect)) == 1)
 
                 # vkCmdDrawIndirectCountKHR[0]
-                draw = draw_indirect.children[0]
+                action = action_indirect.children[0]
 
-                self.check(draw.numIndices == 3)
-                self.check(draw.numInstances == 4)
+                self.check(action.numIndices == 3)
+                self.check(action.numInstances == 4)
 
-                self.controller.SetFrameEvent(draw.eventId, False)
+                self.controller.SetFrameEvent(action.eventId, False)
 
                 # Check that we have PostVS as expected
-                postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut)
+                postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut)
 
                 # These indices are the *output* indices, which have been rebased/remapped, so are not the same as the input
                 # indices
@@ -333,22 +337,22 @@ class VK_Indirect(rdtest.TestCase):
 
                 self.check_overlay([(60, 190)])
 
-                rdtest.log.success("{} {} is as expected".format(level, draw.name))
+                rdtest.log.success("{} {} is as expected".format(level, action.customName))
 
                 # vkCmdDrawIndexedIndirectCountKHR
-                draw_indirect = indirect_count_root.children[1].children[1]
+                action_indirect = indirect_count_root.children[1].children[1]
 
-                self.check(draw_indirect and len(draw_indirect.children) == 3)
+                self.check(action_indirect and len(real_action_children(action_indirect)) == 3)
 
                 # vkCmdDrawIndirectCountKHR[0]
-                draw = draw_indirect.children[0]
-                self.check(draw.numIndices == 3)
-                self.check(draw.numInstances == 1)
+                action = action_indirect.children[0]
+                self.check(action.numIndices == 3)
+                self.check(action.numInstances == 1)
 
-                self.controller.SetFrameEvent(draw.eventId, False)
+                self.controller.SetFrameEvent(action.eventId, False)
 
                 # Check that we have PostVS as expected
-                postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut)
+                postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut)
 
                 # These indices are the *output* indices, which have been rebased/remapped, so are not the same as the input
                 # indices
@@ -363,32 +367,32 @@ class VK_Indirect(rdtest.TestCase):
 
                 self.check_overlay([(100, 190)])
 
-                rdtest.log.success("{} {} is as expected".format(level, draw.name))
+                rdtest.log.success("{} {} is as expected".format(level, action.customName))
 
                 # vkCmdDrawIndirectCountKHR[1]
-                draw = draw_indirect.children[1]
-                self.check(draw.numIndices == 0)
-                self.check(draw.numInstances == 0)
+                action = action_indirect.children[1]
+                self.check(action.numIndices == 0)
+                self.check(action.numInstances == 0)
 
-                self.controller.SetFrameEvent(draw.eventId, False)
+                self.controller.SetFrameEvent(action.eventId, False)
 
-                postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut)
+                postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut)
 
                 self.check(len(postvs_data) == 0)
 
                 self.check_overlay([])
 
-                rdtest.log.success("{} {} is as expected".format(level, draw.name))
+                rdtest.log.success("{} {} is as expected".format(level, action.customName))
 
                 # vkCmdDrawIndirectCountKHR[2]
-                draw = draw_indirect.children[2]
-                self.check(draw.numIndices == 6)
-                self.check(draw.numInstances == 2)
+                action = action_indirect.children[2]
+                self.check(action.numIndices == 6)
+                self.check(action.numInstances == 2)
 
-                self.controller.SetFrameEvent(draw.eventId, False)
+                self.controller.SetFrameEvent(action.eventId, False)
 
                 # Check that we have PostVS as expected
-                postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut)
+                postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut)
 
                 # These indices are the *output* indices, which have been rebased/remapped, so are not the same as the input
                 # indices
@@ -407,14 +411,14 @@ class VK_Indirect(rdtest.TestCase):
 
                 self.check_overlay([(140, 190), (200, 190)])
 
-                rdtest.log.success("{} {} is as expected".format(level, draw.name))
+                rdtest.log.success("{} {} is as expected".format(level, action.customName))
 
                 # Now check that the draws post-count are correctly highlighted
-                self.controller.SetFrameEvent(self.find_draw("{}: Post-count 1".format(level)).children[0].eventId, False)
+                self.controller.SetFrameEvent(self.find_action("{}: Post-count 1".format(level)).children[0].eventId, False)
                 self.check_overlay([(340, 40)])
-                self.controller.SetFrameEvent(self.find_draw("{}: Post-count 2".format(level)).children[0].eventId, False)
+                self.controller.SetFrameEvent(self.find_action("{}: Post-count 2".format(level)).children[0].eventId, False)
                 self.check_overlay([(340, 190)])
-                self.controller.SetFrameEvent(self.find_draw("{}: Post-count 3".format(level)).children[0].eventId, False)
+                self.controller.SetFrameEvent(self.find_action("{}: Post-count 3".format(level)).children[0].eventId, False)
                 self.check_overlay([(340, 115)])
             else:
-                rdtest.log.print("KHR_draw_indirect_count not tested")
+                rdtest.log.print("KHR_action_indirect_count not tested")
