@@ -6,11 +6,6 @@ import os
 class VK_RGP_Capture(rdtest.TestCase):
     demos_test_name = 'VK_Simple_Triangle'
 
-    # Need to enable RGP mode before opening the capture
-    def run(self):
-        rd.SetConfigSetting("ExternalTool_RGPIntegration", "1")
-        super().run()
-
     def check_capture(self):
         apiprops: rd.APIProperties = self.controller.GetAPIProperties()
 
