@@ -7,17 +7,17 @@ class VK_VS_Max_Desc_Set(rdtest.TestCase):
 
     def check_capture(self):
 
-        draw = self.find_draw("Draw")
+        action = self.find_action("Draw")
 
-        self.check(draw is not None)
+        self.check(action is not None)
 
-        self.controller.SetFrameEvent(draw.eventId, False)
+        self.controller.SetFrameEvent(action.eventId, False)
 
         pipe: rd.PipeState = self.controller.GetPipelineState()
 
         # We only need to check the color output for the first vertex - if we got that, the test succeeded.
         # We're not testing VS out fetch in general here, just that it works when there's no spare descriptor set
-        postvs_data = self.get_postvs(draw, rd.MeshDataStage.VSOut, 0, 1)
+        postvs_data = self.get_postvs(action, rd.MeshDataStage.VSOut, 0, 1)
 
         postvs_ref = {
             0: {
